// ============================================================================
// Rocks'n'Diamonds - McDuffin Strikes Back!
// ----------------------------------------------------------------------------
// (c) 1995-2014 by Artsoft Entertainment
//                  Holger Schemel
//                  info@artsoft.org
//                  https://www.artsoft.org/
// ----------------------------------------------------------------------------
// conf_act.c
// ============================================================================

// ------- this file was automatically generated -- do not edit by hand -------

#ifndef CONF_ACT_C
#define CONF_ACT_C

// values for active states of elements and fonts

static struct
{
  int element;
  int element_active;
}
element_with_active_state[] =
{
  {
    EL_BD_MAGIC_WALL,
    EL_BD_MAGIC_WALL_ACTIVE
  },
  {
    EL_CONVEYOR_BELT_1_LEFT,
    EL_CONVEYOR_BELT_1_LEFT_ACTIVE
  },
  {
    EL_CONVEYOR_BELT_1_MIDDLE,
    EL_CONVEYOR_BELT_1_MIDDLE_ACTIVE
  },
  {
    EL_CONVEYOR_BELT_1_RIGHT,
    EL_CONVEYOR_BELT_1_RIGHT_ACTIVE
  },
  {
    EL_CONVEYOR_BELT_2_LEFT,
    EL_CONVEYOR_BELT_2_LEFT_ACTIVE
  },
  {
    EL_CONVEYOR_BELT_2_MIDDLE,
    EL_CONVEYOR_BELT_2_MIDDLE_ACTIVE
  },
  {
    EL_CONVEYOR_BELT_2_RIGHT,
    EL_CONVEYOR_BELT_2_RIGHT_ACTIVE
  },
  {
    EL_CONVEYOR_BELT_3_LEFT,
    EL_CONVEYOR_BELT_3_LEFT_ACTIVE
  },
  {
    EL_CONVEYOR_BELT_3_MIDDLE,
    EL_CONVEYOR_BELT_3_MIDDLE_ACTIVE
  },
  {
    EL_CONVEYOR_BELT_3_RIGHT,
    EL_CONVEYOR_BELT_3_RIGHT_ACTIVE
  },
  {
    EL_CONVEYOR_BELT_4_LEFT,
    EL_CONVEYOR_BELT_4_LEFT_ACTIVE
  },
  {
    EL_CONVEYOR_BELT_4_MIDDLE,
    EL_CONVEYOR_BELT_4_MIDDLE_ACTIVE
  },
  {
    EL_CONVEYOR_BELT_4_RIGHT,
    EL_CONVEYOR_BELT_4_RIGHT_ACTIVE
  },
  {
    EL_DC_GATE_WHITE_GRAY,
    EL_DC_GATE_WHITE_GRAY_ACTIVE
  },
  {
    EL_DC_MAGIC_WALL,
    EL_DC_MAGIC_WALL_ACTIVE
  },
  {
    EL_DC_TIMEGATE_SWITCH,
    EL_DC_TIMEGATE_SWITCH_ACTIVE
  },
  {
    EL_DYNABOMB,
    EL_DYNABOMB_ACTIVE
  },
  {
    EL_DYNABOMB_PLAYER_1,
    EL_DYNABOMB_PLAYER_1_ACTIVE
  },
  {
    EL_DYNABOMB_PLAYER_2,
    EL_DYNABOMB_PLAYER_2_ACTIVE
  },
  {
    EL_DYNABOMB_PLAYER_3,
    EL_DYNABOMB_PLAYER_3_ACTIVE
  },
  {
    EL_DYNABOMB_PLAYER_4,
    EL_DYNABOMB_PLAYER_4_ACTIVE
  },
  {
    EL_DYNAMITE,
    EL_DYNAMITE_ACTIVE
  },
  {
    EL_EMC_DRIPPER,
    EL_EMC_DRIPPER_ACTIVE
  },
  {
    EL_EMC_FAKE_GRASS,
    EL_EMC_FAKE_GRASS_ACTIVE
  },
  {
    EL_EMC_GATE_5_GRAY,
    EL_EMC_GATE_5_GRAY_ACTIVE
  },
  {
    EL_EMC_GATE_6_GRAY,
    EL_EMC_GATE_6_GRAY_ACTIVE
  },
  {
    EL_EMC_GATE_7_GRAY,
    EL_EMC_GATE_7_GRAY_ACTIVE
  },
  {
    EL_EMC_GATE_8_GRAY,
    EL_EMC_GATE_8_GRAY_ACTIVE
  },
  {
    EL_EMC_MAGIC_BALL,
    EL_EMC_MAGIC_BALL_ACTIVE
  },
  {
    EL_EMC_MAGIC_BALL_SWITCH,
    EL_EMC_MAGIC_BALL_SWITCH_ACTIVE
  },
  {
    EL_EMC_SPRING_BUMPER,
    EL_EMC_SPRING_BUMPER_ACTIVE
  },
  {
    EL_EM_DYNAMITE,
    EL_EM_DYNAMITE_ACTIVE
  },
  {
    EL_EM_GATE_1_GRAY,
    EL_EM_GATE_1_GRAY_ACTIVE
  },
  {
    EL_EM_GATE_2_GRAY,
    EL_EM_GATE_2_GRAY_ACTIVE
  },
  {
    EL_EM_GATE_3_GRAY,
    EL_EM_GATE_3_GRAY_ACTIVE
  },
  {
    EL_EM_GATE_4_GRAY,
    EL_EM_GATE_4_GRAY_ACTIVE
  },
  {
    EL_GATE_1_GRAY,
    EL_GATE_1_GRAY_ACTIVE
  },
  {
    EL_GATE_2_GRAY,
    EL_GATE_2_GRAY_ACTIVE
  },
  {
    EL_GATE_3_GRAY,
    EL_GATE_3_GRAY_ACTIVE
  },
  {
    EL_GATE_4_GRAY,
    EL_GATE_4_GRAY_ACTIVE
  },
  {
    EL_INTERNAL_CASCADE_BD,
    EL_INTERNAL_CASCADE_BD_ACTIVE
  },
  {
    EL_INTERNAL_CASCADE_CE,
    EL_INTERNAL_CASCADE_CE_ACTIVE
  },
  {
    EL_INTERNAL_CASCADE_CHARS,
    EL_INTERNAL_CASCADE_CHARS_ACTIVE
  },
  {
    EL_INTERNAL_CASCADE_DC,
    EL_INTERNAL_CASCADE_DC_ACTIVE
  },
  {
    EL_INTERNAL_CASCADE_DF,
    EL_INTERNAL_CASCADE_DF_ACTIVE
  },
  {
    EL_INTERNAL_CASCADE_DX,
    EL_INTERNAL_CASCADE_DX_ACTIVE
  },
  {
    EL_INTERNAL_CASCADE_DYNAMIC,
    EL_INTERNAL_CASCADE_DYNAMIC_ACTIVE
  },
  {
    EL_INTERNAL_CASCADE_EM,
    EL_INTERNAL_CASCADE_EM_ACTIVE
  },
  {
    EL_INTERNAL_CASCADE_EMC,
    EL_INTERNAL_CASCADE_EMC_ACTIVE
  },
  {
    EL_INTERNAL_CASCADE_GE,
    EL_INTERNAL_CASCADE_GE_ACTIVE
  },
  {
    EL_INTERNAL_CASCADE_MM,
    EL_INTERNAL_CASCADE_MM_ACTIVE
  },
  {
    EL_INTERNAL_CASCADE_REF,
    EL_INTERNAL_CASCADE_REF_ACTIVE
  },
  {
    EL_INTERNAL_CASCADE_RND,
    EL_INTERNAL_CASCADE_RND_ACTIVE
  },
  {
    EL_INTERNAL_CASCADE_SB,
    EL_INTERNAL_CASCADE_SB_ACTIVE
  },
  {
    EL_INTERNAL_CASCADE_SP,
    EL_INTERNAL_CASCADE_SP_ACTIVE
  },
  {
    EL_INTERNAL_CASCADE_STEEL_CHARS,
    EL_INTERNAL_CASCADE_STEEL_CHARS_ACTIVE
  },
  {
    EL_INTERNAL_CASCADE_USER,
    EL_INTERNAL_CASCADE_USER_ACTIVE
  },
  {
    EL_INVISIBLE_SAND,
    EL_INVISIBLE_SAND_ACTIVE
  },
  {
    EL_INVISIBLE_STEELWALL,
    EL_INVISIBLE_STEELWALL_ACTIVE
  },
  {
    EL_INVISIBLE_WALL,
    EL_INVISIBLE_WALL_ACTIVE
  },
  {
    EL_LAMP,
    EL_LAMP_ACTIVE
  },
  {
    EL_LIGHT_SWITCH,
    EL_LIGHT_SWITCH_ACTIVE
  },
  {
    EL_MAGIC_WALL,
    EL_MAGIC_WALL_ACTIVE
  },
  {
    EL_MM_FUSE,
    EL_MM_FUSE_ACTIVE
  },
  {
    EL_MM_LIGHTBULB,
    EL_MM_LIGHTBULB_ACTIVE
  },
  {
    EL_ROBOT_WHEEL,
    EL_ROBOT_WHEEL_ACTIVE
  },
  {
    EL_SHIELD_DEADLY,
    EL_SHIELD_DEADLY_ACTIVE
  },
  {
    EL_SHIELD_NORMAL,
    EL_SHIELD_NORMAL_ACTIVE
  },
  {
    EL_SP_BUGGY_BASE,
    EL_SP_BUGGY_BASE_ACTIVE
  },
  {
    EL_SP_DISK_RED,
    EL_SP_DISK_RED_ACTIVE
  },
  {
    EL_SP_TERMINAL,
    EL_SP_TERMINAL_ACTIVE
  },
  {
    EL_TIMEGATE_SWITCH,
    EL_TIMEGATE_SWITCH_ACTIVE
  },
  {
    EL_TRAP,
    EL_TRAP_ACTIVE
  },
  {
    -1,
    -1
  },
};


static struct
{
  int button;
  int button_active;
}
button_with_active_state[] =
{
  {
    IMG_MENU_BUTTON,
    IMG_MENU_BUTTON_ACTIVE
  },
  {
    IMG_MENU_BUTTON_DOWN,
    IMG_MENU_BUTTON_DOWN_ACTIVE
  },
  {
    IMG_MENU_BUTTON_EDITOR,
    IMG_MENU_BUTTON_EDITOR_ACTIVE
  },
  {
    IMG_MENU_BUTTON_ENTER_MENU,
    IMG_MENU_BUTTON_ENTER_MENU_ACTIVE
  },
  {
    IMG_MENU_BUTTON_FIRST_LEVEL,
    IMG_MENU_BUTTON_FIRST_LEVEL_ACTIVE
  },
  {
    IMG_MENU_BUTTON_GAME,
    IMG_MENU_BUTTON_GAME_ACTIVE
  },
  {
    IMG_MENU_BUTTON_INFO,
    IMG_MENU_BUTTON_INFO_ACTIVE
  },
  {
    IMG_MENU_BUTTON_INSERT_SOLUTION,
    IMG_MENU_BUTTON_INSERT_SOLUTION_ACTIVE
  },
  {
    IMG_MENU_BUTTON_LAST_LEVEL,
    IMG_MENU_BUTTON_LAST_LEVEL_ACTIVE
  },
  {
    IMG_MENU_BUTTON_LEAVE_MENU,
    IMG_MENU_BUTTON_LEAVE_MENU_ACTIVE
  },
  {
    IMG_MENU_BUTTON_LEFT,
    IMG_MENU_BUTTON_LEFT_ACTIVE
  },
  {
    IMG_MENU_BUTTON_LEVELS,
    IMG_MENU_BUTTON_LEVELS_ACTIVE
  },
  {
    IMG_MENU_BUTTON_LEVEL_NUMBER,
    IMG_MENU_BUTTON_LEVEL_NUMBER_ACTIVE
  },
  {
    IMG_MENU_BUTTON_NAME,
    IMG_MENU_BUTTON_NAME_ACTIVE
  },
  {
    IMG_MENU_BUTTON_NEXT_LEVEL,
    IMG_MENU_BUTTON_NEXT_LEVEL_ACTIVE
  },
  {
    IMG_MENU_BUTTON_PLAY_SOLUTION,
    IMG_MENU_BUTTON_PLAY_SOLUTION_ACTIVE
  },
  {
    IMG_MENU_BUTTON_PREV_LEVEL,
    IMG_MENU_BUTTON_PREV_LEVEL_ACTIVE
  },
  {
    IMG_MENU_BUTTON_QUIT,
    IMG_MENU_BUTTON_QUIT_ACTIVE
  },
  {
    IMG_MENU_BUTTON_RIGHT,
    IMG_MENU_BUTTON_RIGHT_ACTIVE
  },
  {
    IMG_MENU_BUTTON_SCORES,
    IMG_MENU_BUTTON_SCORES_ACTIVE
  },
  {
    IMG_MENU_BUTTON_SETUP,
    IMG_MENU_BUTTON_SETUP_ACTIVE
  },
  {
    IMG_MENU_BUTTON_SWITCH_ECS_AGA,
    IMG_MENU_BUTTON_SWITCH_ECS_AGA_ACTIVE
  },
  {
    IMG_MENU_BUTTON_UP,
    IMG_MENU_BUTTON_UP_ACTIVE
  },
  {
    -1,
    -1
  },
};


static struct
{
  int font_nr;
  int font_nr_active;
}
font_with_active_state[] =
{
  {
    FONT_INPUT_1,
    FONT_INPUT_1_ACTIVE
  },
  {
    FONT_INPUT_2,
    FONT_INPUT_2_ACTIVE
  },
  {
    FONT_LEVEL_NUMBER,
    FONT_LEVEL_NUMBER_ACTIVE
  },
  {
    FONT_MENU_1,
    FONT_MENU_1_ACTIVE
  },
  {
    FONT_MENU_2,
    FONT_MENU_2_ACTIVE
  },
  {
    FONT_TEXT_1,
    FONT_TEXT_1_ACTIVE
  },
  {
    FONT_TEXT_2,
    FONT_TEXT_2_ACTIVE
  },
  {
    FONT_TEXT_3,
    FONT_TEXT_3_ACTIVE
  },
  {
    FONT_TEXT_4,
    FONT_TEXT_4_ACTIVE
  },
  {
    -1,
    -1
  },
};

#endif	// CONF_ACT_C
