// ============================================================================
// Rocks'n'Diamonds - McDuffin Strikes Back!
// ----------------------------------------------------------------------------
// (c) 1995-2014 by Artsoft Entertainment
//                  Holger Schemel
//                  info@artsoft.org
//                  https://www.artsoft.org/
// ----------------------------------------------------------------------------
// conf_gfx.h
// ============================================================================

// ------- this file was automatically generated -- do not edit by hand -------

#ifndef CONF_GFX_H
#define CONF_GFX_H

// values for graphics configuration (normal elements)

#define IMG_BD_WALL					0
#define IMG_BD_WALL_EDITOR				1
#define IMG_BD_ROCK					2
#define IMG_BD_ROCK_EDITOR				3
#define IMG_BD_ROCK_MOVING_LEFT				4
#define IMG_BD_ROCK_MOVING_RIGHT			5
#define IMG_BD_ROCK_PUSHING_LEFT			6
#define IMG_BD_ROCK_PUSHING_RIGHT			7
#define IMG_BD_DIAMOND					8
#define IMG_BD_DIAMOND_MOVING				9
#define IMG_BD_DIAMOND_FALLING				10
#define IMG_BD_MAGIC_WALL				11
#define IMG_BD_MAGIC_WALL_ACTIVE			12
#define IMG_BD_MAGIC_WALL_FILLING			13
#define IMG_BD_MAGIC_WALL_FULL				14
#define IMG_BD_MAGIC_WALL_EMPTYING			15
#define IMG_BD_MAGIC_WALL_DEAD				16
#define IMG_BD_AMOEBA					17
#define IMG_BD_AMOEBA_EDITOR				18
#define IMG_BD_BUTTERFLY				19
#define IMG_BD_BUTTERFLY_RIGHT				20
#define IMG_BD_BUTTERFLY_RIGHT_EDITOR			21
#define IMG_BD_BUTTERFLY_UP				22
#define IMG_BD_BUTTERFLY_UP_EDITOR			23
#define IMG_BD_BUTTERFLY_LEFT				24
#define IMG_BD_BUTTERFLY_LEFT_EDITOR			25
#define IMG_BD_BUTTERFLY_DOWN				26
#define IMG_BD_BUTTERFLY_DOWN_EDITOR			27
#define IMG_BD_FIREFLY					28
#define IMG_BD_FIREFLY_RIGHT				29
#define IMG_BD_FIREFLY_RIGHT_EDITOR			30
#define IMG_BD_FIREFLY_UP				31
#define IMG_BD_FIREFLY_UP_EDITOR			32
#define IMG_BD_FIREFLY_LEFT				33
#define IMG_BD_FIREFLY_LEFT_EDITOR			34
#define IMG_BD_FIREFLY_DOWN				35
#define IMG_BD_FIREFLY_DOWN_EDITOR			36
#define IMG_SP_DEFAULT_EXPLODING			37
#define IMG_SP_ZONK					38
#define IMG_SP_ZONK_MOVING_LEFT				39
#define IMG_SP_ZONK_MOVING_RIGHT			40
#define IMG_SP_ZONK_PUSHING_LEFT			41
#define IMG_SP_ZONK_PUSHING_RIGHT			42
#define IMG_SP_BASE					43
#define IMG_SP_BASE_DIGGING				44
#define IMG_SP_BASE_SNAPPING				45
#define IMG_SP_MURPHY					46
#define IMG_SP_MURPHY_MOVING_LEFT			47
#define IMG_SP_MURPHY_MOVING_RIGHT			48
#define IMG_SP_MURPHY_DIGGING_LEFT			49
#define IMG_SP_MURPHY_DIGGING_RIGHT			50
#define IMG_SP_MURPHY_COLLECTING_LEFT			51
#define IMG_SP_MURPHY_COLLECTING_RIGHT			52
#define IMG_SP_MURPHY_PUSHING_LEFT			53
#define IMG_SP_MURPHY_PUSHING_RIGHT			54
#define IMG_SP_MURPHY_SNAPPING_LEFT			55
#define IMG_SP_MURPHY_SNAPPING_RIGHT			56
#define IMG_SP_MURPHY_SNAPPING_UP			57
#define IMG_SP_MURPHY_SNAPPING_DOWN			58
#define IMG_SP_MURPHY_BORING				59
#define IMG_SP_MURPHY_BORING_1				60
#define IMG_SP_MURPHY_SLEEPING_LEFT			61
#define IMG_SP_MURPHY_SLEEPING_RIGHT			62
#define IMG_SP_MURPHY_DROPPING				63
#define IMG_SP_MURPHY_SHRINKING				64
#define IMG_SP_MURPHY_CLONE				65
#define IMG_SP_INFOTRON					66
#define IMG_SP_INFOTRON_EDITOR				67
#define IMG_SP_INFOTRON_MOVING_LEFT			68
#define IMG_SP_INFOTRON_MOVING_RIGHT			69
#define IMG_SP_INFOTRON_COLLECTING			70
#define IMG_SP_CHIP_SINGLE				71
#define IMG_SP_CHIP_LEFT				72
#define IMG_SP_CHIP_RIGHT				73
#define IMG_SP_CHIP_TOP					74
#define IMG_SP_CHIP_BOTTOM				75
#define IMG_SP_HARDWARE_GRAY				76
#define IMG_SP_HARDWARE_GREEN				77
#define IMG_SP_HARDWARE_BLUE				78
#define IMG_SP_HARDWARE_RED				79
#define IMG_SP_HARDWARE_YELLOW				80
#define IMG_SP_EXIT_CLOSED				81
#define IMG_SP_EXIT_OPENING				82
#define IMG_SP_EXIT_OPEN				83
#define IMG_SP_EXIT_CLOSING				84
#define IMG_SP_DISK_ORANGE				85
#define IMG_SP_DISK_YELLOW				86
#define IMG_SP_DISK_RED					87
#define IMG_SP_DISK_RED_COLLECTING			88
#define IMG_SP_DISK_RED_ACTIVE				89
#define IMG_SP_PORT_RIGHT				90
#define IMG_SP_PORT_DOWN				91
#define IMG_SP_PORT_LEFT				92
#define IMG_SP_PORT_UP					93
#define IMG_SP_PORT_HORIZONTAL				94
#define IMG_SP_PORT_VERTICAL				95
#define IMG_SP_PORT_ANY					96
#define IMG_SP_GRAVITY_PORT_RIGHT			97
#define IMG_SP_GRAVITY_PORT_RIGHT_EDITOR		98
#define IMG_SP_GRAVITY_PORT_DOWN			99
#define IMG_SP_GRAVITY_PORT_DOWN_EDITOR			100
#define IMG_SP_GRAVITY_PORT_LEFT			101
#define IMG_SP_GRAVITY_PORT_LEFT_EDITOR			102
#define IMG_SP_GRAVITY_PORT_UP				103
#define IMG_SP_GRAVITY_PORT_UP_EDITOR			104
#define IMG_SP_GRAVITY_ON_PORT_RIGHT			105
#define IMG_SP_GRAVITY_ON_PORT_RIGHT_EDITOR		106
#define IMG_SP_GRAVITY_ON_PORT_DOWN			107
#define IMG_SP_GRAVITY_ON_PORT_DOWN_EDITOR		108
#define IMG_SP_GRAVITY_ON_PORT_LEFT			109
#define IMG_SP_GRAVITY_ON_PORT_LEFT_EDITOR		110
#define IMG_SP_GRAVITY_ON_PORT_UP			111
#define IMG_SP_GRAVITY_ON_PORT_UP_EDITOR		112
#define IMG_SP_GRAVITY_OFF_PORT_RIGHT			113
#define IMG_SP_GRAVITY_OFF_PORT_RIGHT_EDITOR		114
#define IMG_SP_GRAVITY_OFF_PORT_DOWN			115
#define IMG_SP_GRAVITY_OFF_PORT_DOWN_EDITOR		116
#define IMG_SP_GRAVITY_OFF_PORT_LEFT			117
#define IMG_SP_GRAVITY_OFF_PORT_LEFT_EDITOR		118
#define IMG_SP_GRAVITY_OFF_PORT_UP			119
#define IMG_SP_GRAVITY_OFF_PORT_UP_EDITOR		120
#define IMG_SP_SNIKSNAK					121
#define IMG_SP_SNIKSNAK_LEFT				122
#define IMG_SP_SNIKSNAK_RIGHT				123
#define IMG_SP_SNIKSNAK_UP				124
#define IMG_SP_SNIKSNAK_DOWN				125
#define IMG_SP_SNIKSNAK_TURNING_FROM_LEFT_UP		126
#define IMG_SP_SNIKSNAK_TURNING_FROM_LEFT_DOWN		127
#define IMG_SP_SNIKSNAK_TURNING_FROM_RIGHT_UP		128
#define IMG_SP_SNIKSNAK_TURNING_FROM_RIGHT_DOWN		129
#define IMG_SP_SNIKSNAK_TURNING_FROM_UP_LEFT		130
#define IMG_SP_SNIKSNAK_TURNING_FROM_UP_RIGHT		131
#define IMG_SP_SNIKSNAK_TURNING_FROM_DOWN_LEFT		132
#define IMG_SP_SNIKSNAK_TURNING_FROM_DOWN_RIGHT		133
#define IMG_SP_ELECTRON					134
#define IMG_SP_ELECTRON_EDITOR				135
#define IMG_SP_ELECTRON_EXPLODING			136
#define IMG_SP_TERMINAL					137
#define IMG_SP_TERMINAL_EDITOR				138
#define IMG_SP_TERMINAL_ACTIVE				139
#define IMG_SP_BUGGY_BASE				140
#define IMG_SP_BUGGY_BASE_EDITOR			141
#define IMG_SP_BUGGY_BASE_ACTIVATING			142
#define IMG_SP_BUGGY_BASE_ACTIVE			143
#define IMG_SP_HARDWARE_BASE_1				144
#define IMG_SP_HARDWARE_BASE_2				145
#define IMG_SP_HARDWARE_BASE_3				146
#define IMG_SP_HARDWARE_BASE_4				147
#define IMG_SP_HARDWARE_BASE_5				148
#define IMG_SP_HARDWARE_BASE_6				149
#define IMG_SOKOBAN_OBJECT				150
#define IMG_SOKOBAN_FIELD_EMPTY				151
#define IMG_SOKOBAN_FIELD_FULL				152
#define IMG_SOKOBAN_FIELD_PLAYER			153
#define IMG_EMPTY_SPACE					154
#define IMG_SAND					155
#define IMG_SAND_CRUMBLED				156
#define IMG_SAND_DIGGING_LEFT				157
#define IMG_SAND_DIGGING_RIGHT				158
#define IMG_SAND_DIGGING_UP				159
#define IMG_SAND_DIGGING_DOWN				160
#define IMG_SAND_DIGGING_LEFT_CRUMBLED			161
#define IMG_SAND_DIGGING_RIGHT_CRUMBLED			162
#define IMG_SAND_DIGGING_UP_CRUMBLED			163
#define IMG_SAND_DIGGING_DOWN_CRUMBLED			164
#define IMG_WALL					165
#define IMG_WALL_SLIPPERY				166
#define IMG_STEELWALL					167
#define IMG_ROCK					168
#define IMG_ROCK_MOVING_LEFT				169
#define IMG_ROCK_MOVING_RIGHT				170
#define IMG_ROCK_PUSHING_LEFT				171
#define IMG_ROCK_PUSHING_RIGHT				172
#define IMG_EMERALD					173
#define IMG_EMERALD_MOVING				174
#define IMG_EMERALD_FALLING				175
#define IMG_EMERALD_COLLECTING				176
#define IMG_DIAMOND					177
#define IMG_DIAMOND_MOVING				178
#define IMG_DIAMOND_FALLING				179
#define IMG_DIAMOND_COLLECTING				180
#define IMG_BOMB					181
#define IMG_NUT						182
#define IMG_NUT_BREAKING				183
#define IMG_DYNAMITE					184
#define IMG_DYNAMITE_ACTIVE				185
#define IMG_EM_DYNAMITE					186
#define IMG_EM_DYNAMITE_ACTIVE				187
#define IMG_EM_DYNAMITE_ACTIVE_EDITOR			188
#define IMG_WALL_EMERALD				189
#define IMG_WALL_DIAMOND				190
#define IMG_BUG						191
#define IMG_BUG_RIGHT					192
#define IMG_BUG_UP					193
#define IMG_BUG_LEFT					194
#define IMG_BUG_DOWN					195
#define IMG_BUG_MOVING_RIGHT				196
#define IMG_BUG_MOVING_UP				197
#define IMG_BUG_MOVING_LEFT				198
#define IMG_BUG_MOVING_DOWN				199
#define IMG_BUG_TURNING_FROM_RIGHT_UP			200
#define IMG_BUG_TURNING_FROM_UP_LEFT			201
#define IMG_BUG_TURNING_FROM_LEFT_DOWN			202
#define IMG_BUG_TURNING_FROM_DOWN_RIGHT			203
#define IMG_BUG_TURNING_FROM_RIGHT_DOWN			204
#define IMG_BUG_TURNING_FROM_UP_RIGHT			205
#define IMG_BUG_TURNING_FROM_LEFT_UP			206
#define IMG_BUG_TURNING_FROM_DOWN_LEFT			207
#define IMG_SPACESHIP					208
#define IMG_SPACESHIP_RIGHT				209
#define IMG_SPACESHIP_UP				210
#define IMG_SPACESHIP_LEFT				211
#define IMG_SPACESHIP_DOWN				212
#define IMG_SPACESHIP_MOVING_RIGHT			213
#define IMG_SPACESHIP_MOVING_UP				214
#define IMG_SPACESHIP_MOVING_LEFT			215
#define IMG_SPACESHIP_MOVING_DOWN			216
#define IMG_SPACESHIP_TURNING_FROM_RIGHT_UP		217
#define IMG_SPACESHIP_TURNING_FROM_UP_LEFT		218
#define IMG_SPACESHIP_TURNING_FROM_LEFT_DOWN		219
#define IMG_SPACESHIP_TURNING_FROM_DOWN_RIGHT		220
#define IMG_SPACESHIP_TURNING_FROM_RIGHT_DOWN		221
#define IMG_SPACESHIP_TURNING_FROM_UP_RIGHT		222
#define IMG_SPACESHIP_TURNING_FROM_LEFT_UP		223
#define IMG_SPACESHIP_TURNING_FROM_DOWN_LEFT		224
#define IMG_YAMYAM					225
#define IMG_YAMYAM_LEFT					226
#define IMG_YAMYAM_LEFT_EDITOR				227
#define IMG_YAMYAM_RIGHT				228
#define IMG_YAMYAM_RIGHT_EDITOR				229
#define IMG_YAMYAM_UP					230
#define IMG_YAMYAM_UP_EDITOR				231
#define IMG_YAMYAM_DOWN					232
#define IMG_YAMYAM_DOWN_EDITOR				233
#define IMG_YAMYAM_MOVING				234
#define IMG_ROBOT					235
#define IMG_ROBOT_MOVING				236
#define IMG_ROBOT_WHEEL					237
#define IMG_ROBOT_WHEEL_ACTIVE				238
#define IMG_MAGIC_WALL					239
#define IMG_MAGIC_WALL_ACTIVE				240
#define IMG_MAGIC_WALL_FILLING				241
#define IMG_MAGIC_WALL_FULL				242
#define IMG_MAGIC_WALL_EMPTYING				243
#define IMG_MAGIC_WALL_DEAD				244
#define IMG_DC_MAGIC_WALL				245
#define IMG_DC_MAGIC_WALL_ACTIVE			246
#define IMG_DC_MAGIC_WALL_FILLING			247
#define IMG_DC_MAGIC_WALL_FULL				248
#define IMG_DC_MAGIC_WALL_EMPTYING			249
#define IMG_DC_MAGIC_WALL_DEAD				250
#define IMG_QUICKSAND_EMPTY				251
#define IMG_QUICKSAND_FILLING				252
#define IMG_QUICKSAND_FULL				253
#define IMG_QUICKSAND_FULL_EDITOR			254
#define IMG_QUICKSAND_EMPTYING				255
#define IMG_QUICKSAND_FAST_EMPTY			256
#define IMG_QUICKSAND_FAST_FILLING			257
#define IMG_QUICKSAND_FAST_FULL				258
#define IMG_QUICKSAND_FAST_FULL_EDITOR			259
#define IMG_QUICKSAND_FAST_EMPTYING			260
#define IMG_ACID_POOL_TOPLEFT				261
#define IMG_ACID_POOL_TOPRIGHT				262
#define IMG_ACID_POOL_BOTTOMLEFT			263
#define IMG_ACID_POOL_BOTTOM				264
#define IMG_ACID_POOL_BOTTOMRIGHT			265
#define IMG_ACID					266
#define IMG_ACID_SPLASH_LEFT				267
#define IMG_ACID_SPLASH_RIGHT				268
#define IMG_AMOEBA_DROP					269
#define IMG_AMOEBA_GROWING				270
#define IMG_AMOEBA_SHRINKING				271
#define IMG_AMOEBA_WET					272
#define IMG_AMOEBA_WET_EDITOR				273
#define IMG_AMOEBA_DROPPING				274
#define IMG_AMOEBA_DRY					275
#define IMG_AMOEBA_FULL					276
#define IMG_AMOEBA_FULL_EDITOR				277
#define IMG_AMOEBA_DEAD					278
#define IMG_AMOEBA_DEAD_EDITOR				279
#define IMG_EM_KEY_1					280
#define IMG_EM_KEY_2					281
#define IMG_EM_KEY_3					282
#define IMG_EM_KEY_4					283
#define IMG_DC_KEY_WHITE				284
#define IMG_EM_GATE_1					285
#define IMG_EM_GATE_2					286
#define IMG_EM_GATE_3					287
#define IMG_EM_GATE_4					288
#define IMG_DC_GATE_WHITE				289
#define IMG_EM_GATE_1_GRAY				290
#define IMG_EM_GATE_1_GRAY_EDITOR			291
#define IMG_EM_GATE_1_GRAY_ACTIVE			292
#define IMG_EM_GATE_2_GRAY				293
#define IMG_EM_GATE_2_GRAY_EDITOR			294
#define IMG_EM_GATE_2_GRAY_ACTIVE			295
#define IMG_EM_GATE_3_GRAY				296
#define IMG_EM_GATE_3_GRAY_EDITOR			297
#define IMG_EM_GATE_3_GRAY_ACTIVE			298
#define IMG_EM_GATE_4_GRAY				299
#define IMG_EM_GATE_4_GRAY_EDITOR			300
#define IMG_EM_GATE_4_GRAY_ACTIVE			301
#define IMG_DC_GATE_WHITE_GRAY				302
#define IMG_DC_GATE_WHITE_GRAY_EDITOR			303
#define IMG_DC_GATE_WHITE_GRAY_ACTIVE			304
#define IMG_DC_GATE_FAKE_GRAY				305
#define IMG_EXIT_CLOSED					306
#define IMG_EXIT_OPENING				307
#define IMG_EXIT_OPEN					308
#define IMG_EXIT_CLOSING				309
#define IMG_STEEL_EXIT_CLOSED				310
#define IMG_STEEL_EXIT_OPENING				311
#define IMG_STEEL_EXIT_OPEN				312
#define IMG_STEEL_EXIT_CLOSING				313
#define IMG_EM_EXIT_CLOSED				314
#define IMG_EM_EXIT_OPENING				315
#define IMG_EM_EXIT_OPEN				316
#define IMG_EM_EXIT_CLOSING				317
#define IMG_EM_STEEL_EXIT_CLOSED			318
#define IMG_EM_STEEL_EXIT_OPENING			319
#define IMG_EM_STEEL_EXIT_OPEN				320
#define IMG_EM_STEEL_EXIT_CLOSING			321
#define IMG_BALLOON					322
#define IMG_BALLOON_MOVING				323
#define IMG_BALLOON_PUSHING				324
#define IMG_BALLOON_SWITCH_LEFT				325
#define IMG_BALLOON_SWITCH_RIGHT			326
#define IMG_BALLOON_SWITCH_UP				327
#define IMG_BALLOON_SWITCH_DOWN				328
#define IMG_BALLOON_SWITCH_ANY				329
#define IMG_BALLOON_SWITCH_NONE				330
#define IMG_SPRING					331
#define IMG_SPRING_LEFT					332
#define IMG_SPRING_RIGHT				333
#define IMG_EMC_STEELWALL_1				334
#define IMG_EMC_STEELWALL_2				335
#define IMG_EMC_STEELWALL_3				336
#define IMG_EMC_STEELWALL_4				337
#define IMG_EMC_WALL_1					338
#define IMG_EMC_WALL_2					339
#define IMG_EMC_WALL_3					340
#define IMG_EMC_WALL_4					341
#define IMG_EMC_WALL_5					342
#define IMG_EMC_WALL_6					343
#define IMG_EMC_WALL_7					344
#define IMG_EMC_WALL_8					345
#define IMG_INVISIBLE_STEELWALL				346
#define IMG_INVISIBLE_STEELWALL_EDITOR			347
#define IMG_INVISIBLE_STEELWALL_ACTIVE			348
#define IMG_INVISIBLE_WALL				349
#define IMG_INVISIBLE_WALL_EDITOR			350
#define IMG_INVISIBLE_WALL_ACTIVE			351
#define IMG_INVISIBLE_SAND				352
#define IMG_INVISIBLE_SAND_EDITOR			353
#define IMG_INVISIBLE_SAND_ACTIVE			354
#define IMG_INVISIBLE_SAND_ACTIVE_CRUMBLED		355
#define IMG_INVISIBLE_SAND_ACTIVE_DIGGING_LEFT		356
#define IMG_INVISIBLE_SAND_ACTIVE_DIGGING_RIGHT		357
#define IMG_INVISIBLE_SAND_ACTIVE_DIGGING_UP		358
#define IMG_INVISIBLE_SAND_ACTIVE_DIGGING_DOWN		359
#define IMG_INVISIBLE_SAND_ACTIVE_DIGGING_LEFT_CRUMBLED	360
#define IMG_INVISIBLE_SAND_ACTIVE_DIGGING_RIGHT_CRUMBLED	361
#define IMG_INVISIBLE_SAND_ACTIVE_DIGGING_UP_CRUMBLED	362
#define IMG_INVISIBLE_SAND_ACTIVE_DIGGING_DOWN_CRUMBLED	363
#define IMG_CONVEYOR_BELT_1_MIDDLE			364
#define IMG_CONVEYOR_BELT_1_MIDDLE_ACTIVE		365
#define IMG_CONVEYOR_BELT_1_LEFT			366
#define IMG_CONVEYOR_BELT_1_LEFT_ACTIVE			367
#define IMG_CONVEYOR_BELT_1_RIGHT			368
#define IMG_CONVEYOR_BELT_1_RIGHT_ACTIVE		369
#define IMG_CONVEYOR_BELT_1_SWITCH_LEFT			370
#define IMG_CONVEYOR_BELT_1_SWITCH_MIDDLE		371
#define IMG_CONVEYOR_BELT_1_SWITCH_RIGHT		372
#define IMG_CONVEYOR_BELT_2_MIDDLE			373
#define IMG_CONVEYOR_BELT_2_MIDDLE_ACTIVE		374
#define IMG_CONVEYOR_BELT_2_LEFT			375
#define IMG_CONVEYOR_BELT_2_LEFT_ACTIVE			376
#define IMG_CONVEYOR_BELT_2_RIGHT			377
#define IMG_CONVEYOR_BELT_2_RIGHT_ACTIVE		378
#define IMG_CONVEYOR_BELT_2_SWITCH_LEFT			379
#define IMG_CONVEYOR_BELT_2_SWITCH_MIDDLE		380
#define IMG_CONVEYOR_BELT_2_SWITCH_RIGHT		381
#define IMG_CONVEYOR_BELT_3_MIDDLE			382
#define IMG_CONVEYOR_BELT_3_MIDDLE_ACTIVE		383
#define IMG_CONVEYOR_BELT_3_LEFT			384
#define IMG_CONVEYOR_BELT_3_LEFT_ACTIVE			385
#define IMG_CONVEYOR_BELT_3_RIGHT			386
#define IMG_CONVEYOR_BELT_3_RIGHT_ACTIVE		387
#define IMG_CONVEYOR_BELT_3_SWITCH_LEFT			388
#define IMG_CONVEYOR_BELT_3_SWITCH_MIDDLE		389
#define IMG_CONVEYOR_BELT_3_SWITCH_RIGHT		390
#define IMG_CONVEYOR_BELT_4_MIDDLE			391
#define IMG_CONVEYOR_BELT_4_MIDDLE_ACTIVE		392
#define IMG_CONVEYOR_BELT_4_LEFT			393
#define IMG_CONVEYOR_BELT_4_LEFT_ACTIVE			394
#define IMG_CONVEYOR_BELT_4_RIGHT			395
#define IMG_CONVEYOR_BELT_4_RIGHT_ACTIVE		396
#define IMG_CONVEYOR_BELT_4_SWITCH_LEFT			397
#define IMG_CONVEYOR_BELT_4_SWITCH_MIDDLE		398
#define IMG_CONVEYOR_BELT_4_SWITCH_RIGHT		399
#define IMG_SWITCHGATE_SWITCH_UP			400
#define IMG_SWITCHGATE_SWITCH_DOWN			401
#define IMG_DC_SWITCHGATE_SWITCH_UP			402
#define IMG_DC_SWITCHGATE_SWITCH_DOWN			403
#define IMG_LIGHT_SWITCH				404
#define IMG_LIGHT_SWITCH_ACTIVE				405
#define IMG_TIMEGATE_SWITCH				406
#define IMG_TIMEGATE_SWITCH_ACTIVE			407
#define IMG_DC_TIMEGATE_SWITCH				408
#define IMG_DC_TIMEGATE_SWITCH_ACTIVE			409
#define IMG_ENVELOPE_1					410
#define IMG_ENVELOPE_1_COLLECTING			411
#define IMG_ENVELOPE_2					412
#define IMG_ENVELOPE_2_COLLECTING			413
#define IMG_ENVELOPE_3					414
#define IMG_ENVELOPE_3_COLLECTING			415
#define IMG_ENVELOPE_4					416
#define IMG_ENVELOPE_4_COLLECTING			417
#define IMG_SIGN_RADIOACTIVITY				418
#define IMG_SIGN_GIVE_WAY				419
#define IMG_SIGN_NO_ENTRY				420
#define IMG_SIGN_EMERGENCY_EXIT				421
#define IMG_SIGN_YIN_YANG				422
#define IMG_SIGN_EXCLAMATION				423
#define IMG_SIGN_STOP					424
#define IMG_SIGN_PARKING				425
#define IMG_SIGN_WHEELCHAIR				426
#define IMG_SIGN_ENTRY_FORBIDDEN			427
#define IMG_SPERMS					428
#define IMG_BULLET					429
#define IMG_HEART					430
#define IMG_CROSS					431
#define IMG_FRANKIE					432
#define IMG_SIGN_SPERMS					433
#define IMG_SIGN_BULLET					434
#define IMG_SIGN_HEART					435
#define IMG_SIGN_CROSS					436
#define IMG_SIGN_FRANKIE				437
#define IMG_LANDMINE					438
#define IMG_DC_LANDMINE					439
#define IMG_STEELWALL_SLIPPERY				440
#define IMG_EXTRA_TIME					441
#define IMG_SHIELD_NORMAL				442
#define IMG_SHIELD_NORMAL_ACTIVE			443
#define IMG_SHIELD_DEADLY				444
#define IMG_SHIELD_DEADLY_ACTIVE			445
#define IMG_SWITCHGATE_CLOSED				446
#define IMG_SWITCHGATE_OPENING				447
#define IMG_SWITCHGATE_OPEN				448
#define IMG_SWITCHGATE_CLOSING				449
#define IMG_TIMEGATE_CLOSED				450
#define IMG_TIMEGATE_OPENING				451
#define IMG_TIMEGATE_OPEN				452
#define IMG_TIMEGATE_CLOSING				453
#define IMG_PEARL					454
#define IMG_PEARL_BREAKING				455
#define IMG_CRYSTAL					456
#define IMG_WALL_PEARL					457
#define IMG_WALL_CRYSTAL				458
#define IMG_DC_STEELWALL_1_LEFT				459
#define IMG_DC_STEELWALL_1_RIGHT			460
#define IMG_DC_STEELWALL_1_TOP				461
#define IMG_DC_STEELWALL_1_BOTTOM			462
#define IMG_DC_STEELWALL_1_HORIZONTAL			463
#define IMG_DC_STEELWALL_1_VERTICAL			464
#define IMG_DC_STEELWALL_1_TOPLEFT			465
#define IMG_DC_STEELWALL_1_TOPRIGHT			466
#define IMG_DC_STEELWALL_1_BOTTOMLEFT			467
#define IMG_DC_STEELWALL_1_BOTTOMRIGHT			468
#define IMG_DC_STEELWALL_1_TOPLEFT_2			469
#define IMG_DC_STEELWALL_1_TOPRIGHT_2			470
#define IMG_DC_STEELWALL_1_BOTTOMLEFT_2			471
#define IMG_DC_STEELWALL_1_BOTTOMRIGHT_2		472
#define IMG_DC_STEELWALL_2_LEFT				473
#define IMG_DC_STEELWALL_2_RIGHT			474
#define IMG_DC_STEELWALL_2_TOP				475
#define IMG_DC_STEELWALL_2_BOTTOM			476
#define IMG_DC_STEELWALL_2_HORIZONTAL			477
#define IMG_DC_STEELWALL_2_VERTICAL			478
#define IMG_DC_STEELWALL_2_MIDDLE			479
#define IMG_DC_STEELWALL_2_SINGLE			480
#define IMG_TUBE_RIGHT_DOWN				481
#define IMG_TUBE_HORIZONTAL_DOWN			482
#define IMG_TUBE_LEFT_DOWN				483
#define IMG_TUBE_HORIZONTAL				484
#define IMG_TUBE_VERTICAL_RIGHT				485
#define IMG_TUBE_ANY					486
#define IMG_TUBE_VERTICAL_LEFT				487
#define IMG_TUBE_VERTICAL				488
#define IMG_TUBE_RIGHT_UP				489
#define IMG_TUBE_HORIZONTAL_UP				490
#define IMG_TUBE_LEFT_UP				491
#define IMG_TRAP					492
#define IMG_TRAP_ACTIVE					493
#define IMG_DX_SUPABOMB					494
#define IMG_KEY_1					495
#define IMG_KEY_2					496
#define IMG_KEY_3					497
#define IMG_KEY_4					498
#define IMG_GATE_1					499
#define IMG_GATE_2					500
#define IMG_GATE_3					501
#define IMG_GATE_4					502
#define IMG_GATE_1_GRAY					503
#define IMG_GATE_1_GRAY_EDITOR				504
#define IMG_GATE_1_GRAY_ACTIVE				505
#define IMG_GATE_2_GRAY					506
#define IMG_GATE_2_GRAY_EDITOR				507
#define IMG_GATE_2_GRAY_ACTIVE				508
#define IMG_GATE_3_GRAY					509
#define IMG_GATE_3_GRAY_EDITOR				510
#define IMG_GATE_3_GRAY_ACTIVE				511
#define IMG_GATE_4_GRAY					512
#define IMG_GATE_4_GRAY_EDITOR				513
#define IMG_GATE_4_GRAY_ACTIVE				514
#define IMG_GAME_OF_LIFE				515
#define IMG_BIOMAZE					516
#define IMG_PACMAN					517
#define IMG_PACMAN_RIGHT				518
#define IMG_PACMAN_UP					519
#define IMG_PACMAN_LEFT					520
#define IMG_PACMAN_DOWN					521
#define IMG_PACMAN_TURNING_FROM_RIGHT			522
#define IMG_PACMAN_TURNING_FROM_UP			523
#define IMG_PACMAN_TURNING_FROM_LEFT			524
#define IMG_PACMAN_TURNING_FROM_DOWN			525
#define IMG_LAMP					526
#define IMG_LAMP_ACTIVE					527
#define IMG_TIME_ORB_FULL				528
#define IMG_TIME_ORB_EMPTY				529
#define IMG_EMERALD_YELLOW				530
#define IMG_EMERALD_YELLOW_MOVING			531
#define IMG_EMERALD_YELLOW_FALLING			532
#define IMG_EMERALD_RED					533
#define IMG_EMERALD_RED_MOVING				534
#define IMG_EMERALD_RED_FALLING				535
#define IMG_EMERALD_PURPLE				536
#define IMG_EMERALD_PURPLE_MOVING			537
#define IMG_EMERALD_PURPLE_FALLING			538
#define IMG_WALL_EMERALD_YELLOW				539
#define IMG_WALL_EMERALD_RED				540
#define IMG_WALL_EMERALD_PURPLE				541
#define IMG_WALL_BD_DIAMOND				542
#define IMG_EXPANDABLE_WALL				543
#define IMG_EXPANDABLE_WALL_HORIZONTAL			544
#define IMG_EXPANDABLE_WALL_HORIZONTAL_EDITOR		545
#define IMG_EXPANDABLE_WALL_VERTICAL			546
#define IMG_EXPANDABLE_WALL_VERTICAL_EDITOR		547
#define IMG_EXPANDABLE_WALL_ANY				548
#define IMG_EXPANDABLE_WALL_ANY_EDITOR			549
#define IMG_EXPANDABLE_STEELWALL_HORIZONTAL		550
#define IMG_EXPANDABLE_STEELWALL_HORIZONTAL_EDITOR	551
#define IMG_EXPANDABLE_STEELWALL_VERTICAL		552
#define IMG_EXPANDABLE_STEELWALL_VERTICAL_EDITOR	553
#define IMG_EXPANDABLE_STEELWALL_ANY			554
#define IMG_EXPANDABLE_STEELWALL_ANY_EDITOR		555
#define IMG_BD_EXPANDABLE_WALL				556
#define IMG_BD_EXPANDABLE_WALL_EDITOR			557
#define IMG_EXPANDABLE_WALL_GROWING_LEFT		558
#define IMG_EXPANDABLE_WALL_GROWING_RIGHT		559
#define IMG_EXPANDABLE_WALL_GROWING_UP			560
#define IMG_EXPANDABLE_WALL_GROWING_DOWN		561
#define IMG_EXPANDABLE_STEELWALL_GROWING_LEFT		562
#define IMG_EXPANDABLE_STEELWALL_GROWING_RIGHT		563
#define IMG_EXPANDABLE_STEELWALL_GROWING_UP		564
#define IMG_EXPANDABLE_STEELWALL_GROWING_DOWN		565
#define IMG_BLACK_ORB					566
#define IMG_SPEED_PILL					567
#define IMG_DARK_YAMYAM					568
#define IMG_DYNABOMB					569
#define IMG_DYNABOMB_ACTIVE				570
#define IMG_DYNABOMB_PLAYER_1				571
#define IMG_DYNABOMB_PLAYER_1_ACTIVE			572
#define IMG_DYNABOMB_PLAYER_2				573
#define IMG_DYNABOMB_PLAYER_2_ACTIVE			574
#define IMG_DYNABOMB_PLAYER_3				575
#define IMG_DYNABOMB_PLAYER_3_ACTIVE			576
#define IMG_DYNABOMB_PLAYER_4				577
#define IMG_DYNABOMB_PLAYER_4_ACTIVE			578
#define IMG_DYNABOMB_INCREASE_NUMBER			579
#define IMG_DYNABOMB_INCREASE_SIZE			580
#define IMG_DYNABOMB_INCREASE_POWER			581
#define IMG_PIG						582
#define IMG_PIG_DOWN					583
#define IMG_PIG_UP					584
#define IMG_PIG_LEFT					585
#define IMG_PIG_RIGHT					586
#define IMG_PIG_MOVING_DOWN				587
#define IMG_PIG_MOVING_UP				588
#define IMG_PIG_MOVING_LEFT				589
#define IMG_PIG_MOVING_RIGHT				590
#define IMG_PIG_DIGGING_DOWN				591
#define IMG_PIG_DIGGING_UP				592
#define IMG_PIG_DIGGING_LEFT				593
#define IMG_PIG_DIGGING_RIGHT				594
#define IMG_DRAGON					595
#define IMG_DRAGON_DOWN					596
#define IMG_DRAGON_UP					597
#define IMG_DRAGON_LEFT					598
#define IMG_DRAGON_RIGHT				599
#define IMG_DRAGON_MOVING_DOWN				600
#define IMG_DRAGON_MOVING_UP				601
#define IMG_DRAGON_MOVING_LEFT				602
#define IMG_DRAGON_MOVING_RIGHT				603
#define IMG_DRAGON_ATTACKING_DOWN			604
#define IMG_DRAGON_ATTACKING_UP				605
#define IMG_DRAGON_ATTACKING_LEFT			606
#define IMG_DRAGON_ATTACKING_RIGHT			607
#define IMG_MOLE					608
#define IMG_MOLE_DOWN					609
#define IMG_MOLE_UP					610
#define IMG_MOLE_LEFT					611
#define IMG_MOLE_RIGHT					612
#define IMG_MOLE_MOVING_DOWN				613
#define IMG_MOLE_MOVING_UP				614
#define IMG_MOLE_MOVING_LEFT				615
#define IMG_MOLE_MOVING_RIGHT				616
#define IMG_MOLE_DIGGING_DOWN				617
#define IMG_MOLE_DIGGING_UP				618
#define IMG_MOLE_DIGGING_LEFT				619
#define IMG_MOLE_DIGGING_RIGHT				620
#define IMG_PENGUIN					621
#define IMG_PENGUIN_DOWN				622
#define IMG_PENGUIN_UP					623
#define IMG_PENGUIN_LEFT				624
#define IMG_PENGUIN_RIGHT				625
#define IMG_PENGUIN_MOVING_DOWN				626
#define IMG_PENGUIN_MOVING_UP				627
#define IMG_PENGUIN_MOVING_LEFT				628
#define IMG_PENGUIN_MOVING_RIGHT			629
#define IMG_SATELLITE					630
#define IMG_FLAMES_1_LEFT				631
#define IMG_FLAMES_2_LEFT				632
#define IMG_FLAMES_3_LEFT				633
#define IMG_FLAMES_1_RIGHT				634
#define IMG_FLAMES_2_RIGHT				635
#define IMG_FLAMES_3_RIGHT				636
#define IMG_FLAMES_1_UP					637
#define IMG_FLAMES_2_UP					638
#define IMG_FLAMES_3_UP					639
#define IMG_FLAMES_1_DOWN				640
#define IMG_FLAMES_2_DOWN				641
#define IMG_FLAMES_3_DOWN				642
#define IMG_STONEBLOCK					643
#define IMG_PLAYER_1					644
#define IMG_PLAYER_1_DOWN				645
#define IMG_PLAYER_1_UP					646
#define IMG_PLAYER_1_LEFT				647
#define IMG_PLAYER_1_RIGHT				648
#define IMG_PLAYER_1_MOVING_DOWN			649
#define IMG_PLAYER_1_MOVING_UP				650
#define IMG_PLAYER_1_MOVING_LEFT			651
#define IMG_PLAYER_1_MOVING_RIGHT			652
#define IMG_PLAYER_1_DIGGING_DOWN			653
#define IMG_PLAYER_1_DIGGING_UP				654
#define IMG_PLAYER_1_DIGGING_LEFT			655
#define IMG_PLAYER_1_DIGGING_RIGHT			656
#define IMG_PLAYER_1_COLLECTING_DOWN			657
#define IMG_PLAYER_1_COLLECTING_UP			658
#define IMG_PLAYER_1_COLLECTING_LEFT			659
#define IMG_PLAYER_1_COLLECTING_RIGHT			660
#define IMG_PLAYER_1_PUSHING_DOWN			661
#define IMG_PLAYER_1_PUSHING_UP				662
#define IMG_PLAYER_1_PUSHING_LEFT			663
#define IMG_PLAYER_1_PUSHING_RIGHT			664
#define IMG_PLAYER_1_SNAPPING_DOWN			665
#define IMG_PLAYER_1_SNAPPING_UP			666
#define IMG_PLAYER_1_SNAPPING_LEFT			667
#define IMG_PLAYER_1_SNAPPING_RIGHT			668
#define IMG_PLAYER_2					669
#define IMG_PLAYER_2_DOWN				670
#define IMG_PLAYER_2_UP					671
#define IMG_PLAYER_2_LEFT				672
#define IMG_PLAYER_2_RIGHT				673
#define IMG_PLAYER_2_MOVING_DOWN			674
#define IMG_PLAYER_2_MOVING_UP				675
#define IMG_PLAYER_2_MOVING_LEFT			676
#define IMG_PLAYER_2_MOVING_RIGHT			677
#define IMG_PLAYER_2_DIGGING_DOWN			678
#define IMG_PLAYER_2_DIGGING_UP				679
#define IMG_PLAYER_2_DIGGING_LEFT			680
#define IMG_PLAYER_2_DIGGING_RIGHT			681
#define IMG_PLAYER_2_COLLECTING_DOWN			682
#define IMG_PLAYER_2_COLLECTING_UP			683
#define IMG_PLAYER_2_COLLECTING_LEFT			684
#define IMG_PLAYER_2_COLLECTING_RIGHT			685
#define IMG_PLAYER_2_PUSHING_DOWN			686
#define IMG_PLAYER_2_PUSHING_UP				687
#define IMG_PLAYER_2_PUSHING_LEFT			688
#define IMG_PLAYER_2_PUSHING_RIGHT			689
#define IMG_PLAYER_2_SNAPPING_DOWN			690
#define IMG_PLAYER_2_SNAPPING_UP			691
#define IMG_PLAYER_2_SNAPPING_LEFT			692
#define IMG_PLAYER_2_SNAPPING_RIGHT			693
#define IMG_PLAYER_3					694
#define IMG_PLAYER_3_DOWN				695
#define IMG_PLAYER_3_UP					696
#define IMG_PLAYER_3_LEFT				697
#define IMG_PLAYER_3_RIGHT				698
#define IMG_PLAYER_3_MOVING_DOWN			699
#define IMG_PLAYER_3_MOVING_UP				700
#define IMG_PLAYER_3_MOVING_LEFT			701
#define IMG_PLAYER_3_MOVING_RIGHT			702
#define IMG_PLAYER_3_DIGGING_DOWN			703
#define IMG_PLAYER_3_DIGGING_UP				704
#define IMG_PLAYER_3_DIGGING_LEFT			705
#define IMG_PLAYER_3_DIGGING_RIGHT			706
#define IMG_PLAYER_3_COLLECTING_DOWN			707
#define IMG_PLAYER_3_COLLECTING_UP			708
#define IMG_PLAYER_3_COLLECTING_LEFT			709
#define IMG_PLAYER_3_COLLECTING_RIGHT			710
#define IMG_PLAYER_3_PUSHING_DOWN			711
#define IMG_PLAYER_3_PUSHING_UP				712
#define IMG_PLAYER_3_PUSHING_LEFT			713
#define IMG_PLAYER_3_PUSHING_RIGHT			714
#define IMG_PLAYER_3_SNAPPING_DOWN			715
#define IMG_PLAYER_3_SNAPPING_UP			716
#define IMG_PLAYER_3_SNAPPING_LEFT			717
#define IMG_PLAYER_3_SNAPPING_RIGHT			718
#define IMG_PLAYER_4					719
#define IMG_PLAYER_4_DOWN				720
#define IMG_PLAYER_4_UP					721
#define IMG_PLAYER_4_LEFT				722
#define IMG_PLAYER_4_RIGHT				723
#define IMG_PLAYER_4_MOVING_DOWN			724
#define IMG_PLAYER_4_MOVING_UP				725
#define IMG_PLAYER_4_MOVING_LEFT			726
#define IMG_PLAYER_4_MOVING_RIGHT			727
#define IMG_PLAYER_4_DIGGING_DOWN			728
#define IMG_PLAYER_4_DIGGING_UP				729
#define IMG_PLAYER_4_DIGGING_LEFT			730
#define IMG_PLAYER_4_DIGGING_RIGHT			731
#define IMG_PLAYER_4_COLLECTING_DOWN			732
#define IMG_PLAYER_4_COLLECTING_UP			733
#define IMG_PLAYER_4_COLLECTING_LEFT			734
#define IMG_PLAYER_4_COLLECTING_RIGHT			735
#define IMG_PLAYER_4_PUSHING_DOWN			736
#define IMG_PLAYER_4_PUSHING_UP				737
#define IMG_PLAYER_4_PUSHING_LEFT			738
#define IMG_PLAYER_4_PUSHING_RIGHT			739
#define IMG_PLAYER_4_SNAPPING_DOWN			740
#define IMG_PLAYER_4_SNAPPING_UP			741
#define IMG_PLAYER_4_SNAPPING_LEFT			742
#define IMG_PLAYER_4_SNAPPING_RIGHT			743
#define IMG_DEFAULT_EXPLODING				744
#define IMG_TWINKLE_BLUE				745
#define IMG_TWINKLE_WHITE				746
#define IMG_STEELWALL_TOPLEFT				747
#define IMG_STEELWALL_TOPRIGHT				748
#define IMG_STEELWALL_BOTTOMLEFT			749
#define IMG_STEELWALL_BOTTOMRIGHT			750
#define IMG_STEELWALL_HORIZONTAL			751
#define IMG_STEELWALL_VERTICAL				752
#define IMG_STEELWALL_TOPLEFT_EDITOR			753
#define IMG_STEELWALL_TOPRIGHT_EDITOR			754
#define IMG_STEELWALL_BOTTOMLEFT_EDITOR			755
#define IMG_STEELWALL_BOTTOMRIGHT_EDITOR		756
#define IMG_STEELWALL_HORIZONTAL_EDITOR			757
#define IMG_STEELWALL_VERTICAL_EDITOR			758
#define IMG_INVISIBLE_STEELWALL_TOPLEFT			759
#define IMG_INVISIBLE_STEELWALL_TOPRIGHT		760
#define IMG_INVISIBLE_STEELWALL_BOTTOMLEFT		761
#define IMG_INVISIBLE_STEELWALL_BOTTOMRIGHT		762
#define IMG_INVISIBLE_STEELWALL_HORIZONTAL		763
#define IMG_INVISIBLE_STEELWALL_VERTICAL		764
#define IMG_INVISIBLE_STEELWALL_TOPLEFT_EDITOR		765
#define IMG_INVISIBLE_STEELWALL_TOPRIGHT_EDITOR		766
#define IMG_INVISIBLE_STEELWALL_BOTTOMLEFT_EDITOR	767
#define IMG_INVISIBLE_STEELWALL_BOTTOMRIGHT_EDITOR	768
#define IMG_INVISIBLE_STEELWALL_HORIZONTAL_EDITOR	769
#define IMG_INVISIBLE_STEELWALL_VERTICAL_EDITOR		770
#define IMG_ARROW_LEFT					771
#define IMG_ARROW_RIGHT					772
#define IMG_ARROW_UP					773
#define IMG_ARROW_DOWN					774
#define IMG_UNKNOWN					775
#define IMG_TRIGGER_ELEMENT				776
#define IMG_TRIGGER_PLAYER				777
#define IMG_TRIGGER_CE_VALUE				778
#define IMG_TRIGGER_CE_SCORE				779
#define IMG_CURRENT_CE_VALUE				780
#define IMG_CURRENT_CE_SCORE				781
#define IMG_PREV_CE_1					782
#define IMG_PREV_CE_2					783
#define IMG_PREV_CE_3					784
#define IMG_PREV_CE_4					785
#define IMG_PREV_CE_5					786
#define IMG_PREV_CE_6					787
#define IMG_PREV_CE_7					788
#define IMG_PREV_CE_8					789
#define IMG_NEXT_CE_1					790
#define IMG_NEXT_CE_2					791
#define IMG_NEXT_CE_3					792
#define IMG_NEXT_CE_4					793
#define IMG_NEXT_CE_5					794
#define IMG_NEXT_CE_6					795
#define IMG_NEXT_CE_7					796
#define IMG_NEXT_CE_8					797
#define IMG_SELF					798
#define IMG_ANY_ELEMENT					799
#define IMG_EMC_KEY_5					800
#define IMG_EMC_KEY_6					801
#define IMG_EMC_KEY_7					802
#define IMG_EMC_KEY_8					803
#define IMG_EMC_GATE_5					804
#define IMG_EMC_GATE_6					805
#define IMG_EMC_GATE_7					806
#define IMG_EMC_GATE_8					807
#define IMG_EMC_GATE_5_GRAY				808
#define IMG_EMC_GATE_5_GRAY_EDITOR			809
#define IMG_EMC_GATE_5_GRAY_ACTIVE			810
#define IMG_EMC_GATE_6_GRAY				811
#define IMG_EMC_GATE_6_GRAY_EDITOR			812
#define IMG_EMC_GATE_6_GRAY_ACTIVE			813
#define IMG_EMC_GATE_7_GRAY				814
#define IMG_EMC_GATE_7_GRAY_EDITOR			815
#define IMG_EMC_GATE_7_GRAY_ACTIVE			816
#define IMG_EMC_GATE_8_GRAY				817
#define IMG_EMC_GATE_8_GRAY_EDITOR			818
#define IMG_EMC_GATE_8_GRAY_ACTIVE			819
#define IMG_EMC_ANDROID					820
#define IMG_EMC_ANDROID_SHRINKING_UPLEFT		821
#define IMG_EMC_ANDROID_GROWING_DOWNRIGHT		822
#define IMG_EMC_ANDROID_SHRINKING_DOWNLEFT		823
#define IMG_EMC_ANDROID_GROWING_UPRIGHT			824
#define IMG_EMC_ANDROID_SHRINKING_UPRIGHT		825
#define IMG_EMC_ANDROID_GROWING_DOWNLEFT		826
#define IMG_EMC_ANDROID_SHRINKING_DOWNRIGHT		827
#define IMG_EMC_ANDROID_GROWING_UPLEFT			828
#define IMG_EMC_GRASS					829
#define IMG_EMC_GRASS_CRUMBLED				830
#define IMG_EMC_GRASS_DIGGING_LEFT			831
#define IMG_EMC_GRASS_DIGGING_RIGHT			832
#define IMG_EMC_GRASS_DIGGING_UP			833
#define IMG_EMC_GRASS_DIGGING_DOWN			834
#define IMG_EMC_GRASS_DIGGING_LEFT_CRUMBLED		835
#define IMG_EMC_GRASS_DIGGING_RIGHT_CRUMBLED		836
#define IMG_EMC_GRASS_DIGGING_UP_CRUMBLED		837
#define IMG_EMC_GRASS_DIGGING_DOWN_CRUMBLED		838
#define IMG_EMC_MAGIC_BALL				839
#define IMG_EMC_MAGIC_BALL_ACTIVE			840
#define IMG_EMC_MAGIC_BALL_DROPPING			841
#define IMG_EMC_MAGIC_BALL_SWITCH			842
#define IMG_EMC_MAGIC_BALL_SWITCH_ACTIVE		843
#define IMG_EMC_SPRING_BUMPER				844
#define IMG_EMC_SPRING_BUMPER_ACTIVE			845
#define IMG_EMC_PLANT					846
#define IMG_EMC_PLANT_CRUMBLED				847
#define IMG_EMC_LENSES					848
#define IMG_EMC_MAGNIFIER				849
#define IMG_EMC_WALL_9					850
#define IMG_EMC_WALL_10					851
#define IMG_EMC_WALL_11					852
#define IMG_EMC_WALL_12					853
#define IMG_EMC_WALL_13					854
#define IMG_EMC_WALL_14					855
#define IMG_EMC_WALL_15					856
#define IMG_EMC_WALL_16					857
#define IMG_EMC_WALL_SLIPPERY_1				858
#define IMG_EMC_WALL_SLIPPERY_2				859
#define IMG_EMC_WALL_SLIPPERY_3				860
#define IMG_EMC_WALL_SLIPPERY_4				861
#define IMG_EMC_FAKE_GRASS				862
#define IMG_EMC_FAKE_GRASS_CRUMBLED			863
#define IMG_EMC_FAKE_GRASS_ACTIVE			864
#define IMG_EMC_FAKE_GRASS_ACTIVE_CRUMBLED		865
#define IMG_EMC_FAKE_GRASS_EDITOR			866
#define IMG_EMC_FAKE_ACID				867
#define IMG_EMC_DRIPPER					868
#define IMG_EMC_DRIPPER_EDITOR				869
#define IMG_EMC_DRIPPER_ACTIVE				870
#define IMG_MM_MCDUFFIN					871
#define IMG_MM_MCDUFFIN_RIGHT				872
#define IMG_MM_MCDUFFIN_UP				873
#define IMG_MM_MCDUFFIN_LEFT				874
#define IMG_MM_MCDUFFIN_DOWN				875
#define IMG_MM_EXIT_CLOSED				876
#define IMG_MM_EXIT_OPENING				877
#define IMG_MM_EXIT_OPEN				878
#define IMG_MM_EXIT_CLOSING				879
#define IMG_MM_MIRROR_1					880
#define IMG_MM_MIRROR_2					881
#define IMG_MM_MIRROR_3					882
#define IMG_MM_MIRROR_4					883
#define IMG_MM_MIRROR_5					884
#define IMG_MM_MIRROR_6					885
#define IMG_MM_MIRROR_7					886
#define IMG_MM_MIRROR_8					887
#define IMG_MM_MIRROR_9					888
#define IMG_MM_MIRROR_10				889
#define IMG_MM_MIRROR_11				890
#define IMG_MM_MIRROR_12				891
#define IMG_MM_MIRROR_13				892
#define IMG_MM_MIRROR_14				893
#define IMG_MM_MIRROR_15				894
#define IMG_MM_MIRROR_16				895
#define IMG_MM_MIRROR_FIXED_1				896
#define IMG_MM_MIRROR_FIXED_2				897
#define IMG_MM_MIRROR_FIXED_3				898
#define IMG_MM_MIRROR_FIXED_4				899
#define IMG_MM_STEEL_GRID_FIXED_1			900
#define IMG_MM_STEEL_GRID_FIXED_2			901
#define IMG_MM_STEEL_GRID_FIXED_3			902
#define IMG_MM_STEEL_GRID_FIXED_4			903
#define IMG_MM_WOODEN_GRID_FIXED_1			904
#define IMG_MM_WOODEN_GRID_FIXED_2			905
#define IMG_MM_WOODEN_GRID_FIXED_3			906
#define IMG_MM_WOODEN_GRID_FIXED_4			907
#define IMG_MM_POLARIZER_1				908
#define IMG_MM_POLARIZER_2				909
#define IMG_MM_POLARIZER_3				910
#define IMG_MM_POLARIZER_4				911
#define IMG_MM_POLARIZER_5				912
#define IMG_MM_POLARIZER_6				913
#define IMG_MM_POLARIZER_7				914
#define IMG_MM_POLARIZER_8				915
#define IMG_MM_POLARIZER_9				916
#define IMG_MM_POLARIZER_10				917
#define IMG_MM_POLARIZER_11				918
#define IMG_MM_POLARIZER_12				919
#define IMG_MM_POLARIZER_13				920
#define IMG_MM_POLARIZER_14				921
#define IMG_MM_POLARIZER_15				922
#define IMG_MM_POLARIZER_16				923
#define IMG_MM_POLARIZER_CROSS_1			924
#define IMG_MM_POLARIZER_CROSS_2			925
#define IMG_MM_POLARIZER_CROSS_3			926
#define IMG_MM_POLARIZER_CROSS_4			927
#define IMG_MM_TELEPORTER_1				928
#define IMG_MM_TELEPORTER_2				929
#define IMG_MM_TELEPORTER_3				930
#define IMG_MM_TELEPORTER_4				931
#define IMG_MM_TELEPORTER_5				932
#define IMG_MM_TELEPORTER_6				933
#define IMG_MM_TELEPORTER_7				934
#define IMG_MM_TELEPORTER_8				935
#define IMG_MM_TELEPORTER_9				936
#define IMG_MM_TELEPORTER_10				937
#define IMG_MM_TELEPORTER_11				938
#define IMG_MM_TELEPORTER_12				939
#define IMG_MM_TELEPORTER_13				940
#define IMG_MM_TELEPORTER_14				941
#define IMG_MM_TELEPORTER_15				942
#define IMG_MM_TELEPORTER_16				943
#define IMG_MM_TELEPORTER_RED_1				944
#define IMG_MM_TELEPORTER_RED_2				945
#define IMG_MM_TELEPORTER_RED_3				946
#define IMG_MM_TELEPORTER_RED_4				947
#define IMG_MM_TELEPORTER_RED_5				948
#define IMG_MM_TELEPORTER_RED_6				949
#define IMG_MM_TELEPORTER_RED_7				950
#define IMG_MM_TELEPORTER_RED_8				951
#define IMG_MM_TELEPORTER_RED_9				952
#define IMG_MM_TELEPORTER_RED_10			953
#define IMG_MM_TELEPORTER_RED_11			954
#define IMG_MM_TELEPORTER_RED_12			955
#define IMG_MM_TELEPORTER_RED_13			956
#define IMG_MM_TELEPORTER_RED_14			957
#define IMG_MM_TELEPORTER_RED_15			958
#define IMG_MM_TELEPORTER_RED_16			959
#define IMG_MM_TELEPORTER_YELLOW_1			960
#define IMG_MM_TELEPORTER_YELLOW_2			961
#define IMG_MM_TELEPORTER_YELLOW_3			962
#define IMG_MM_TELEPORTER_YELLOW_4			963
#define IMG_MM_TELEPORTER_YELLOW_5			964
#define IMG_MM_TELEPORTER_YELLOW_6			965
#define IMG_MM_TELEPORTER_YELLOW_7			966
#define IMG_MM_TELEPORTER_YELLOW_8			967
#define IMG_MM_TELEPORTER_YELLOW_9			968
#define IMG_MM_TELEPORTER_YELLOW_10			969
#define IMG_MM_TELEPORTER_YELLOW_11			970
#define IMG_MM_TELEPORTER_YELLOW_12			971
#define IMG_MM_TELEPORTER_YELLOW_13			972
#define IMG_MM_TELEPORTER_YELLOW_14			973
#define IMG_MM_TELEPORTER_YELLOW_15			974
#define IMG_MM_TELEPORTER_YELLOW_16			975
#define IMG_MM_TELEPORTER_GREEN_1			976
#define IMG_MM_TELEPORTER_GREEN_2			977
#define IMG_MM_TELEPORTER_GREEN_3			978
#define IMG_MM_TELEPORTER_GREEN_4			979
#define IMG_MM_TELEPORTER_GREEN_5			980
#define IMG_MM_TELEPORTER_GREEN_6			981
#define IMG_MM_TELEPORTER_GREEN_7			982
#define IMG_MM_TELEPORTER_GREEN_8			983
#define IMG_MM_TELEPORTER_GREEN_9			984
#define IMG_MM_TELEPORTER_GREEN_10			985
#define IMG_MM_TELEPORTER_GREEN_11			986
#define IMG_MM_TELEPORTER_GREEN_12			987
#define IMG_MM_TELEPORTER_GREEN_13			988
#define IMG_MM_TELEPORTER_GREEN_14			989
#define IMG_MM_TELEPORTER_GREEN_15			990
#define IMG_MM_TELEPORTER_GREEN_16			991
#define IMG_MM_TELEPORTER_BLUE_1			992
#define IMG_MM_TELEPORTER_BLUE_2			993
#define IMG_MM_TELEPORTER_BLUE_3			994
#define IMG_MM_TELEPORTER_BLUE_4			995
#define IMG_MM_TELEPORTER_BLUE_5			996
#define IMG_MM_TELEPORTER_BLUE_6			997
#define IMG_MM_TELEPORTER_BLUE_7			998
#define IMG_MM_TELEPORTER_BLUE_8			999
#define IMG_MM_TELEPORTER_BLUE_9			1000
#define IMG_MM_TELEPORTER_BLUE_10			1001
#define IMG_MM_TELEPORTER_BLUE_11			1002
#define IMG_MM_TELEPORTER_BLUE_12			1003
#define IMG_MM_TELEPORTER_BLUE_13			1004
#define IMG_MM_TELEPORTER_BLUE_14			1005
#define IMG_MM_TELEPORTER_BLUE_15			1006
#define IMG_MM_TELEPORTER_BLUE_16			1007
#define IMG_MM_KETTLE					1008
#define IMG_MM_KETTLE_EXPLODING				1009
#define IMG_MM_BOMB					1010
#define IMG_MM_PRISM					1011
#define IMG_MM_FUSE					1012
#define IMG_MM_FUSE_ACTIVE				1013
#define IMG_MM_STEEL_LOCK				1014
#define IMG_MM_WOODEN_LOCK				1015
#define IMG_MM_STEEL_BLOCK				1016
#define IMG_MM_WOODEN_BLOCK				1017
#define IMG_MM_KEY					1018
#define IMG_MM_LIGHTBULB				1019
#define IMG_MM_LIGHTBULB_ACTIVE				1020
#define IMG_MM_LIGHTBALL				1021
#define IMG_MM_LIGHTBALL_RED				1022
#define IMG_MM_LIGHTBALL_BLUE				1023
#define IMG_MM_LIGHTBALL_YELLOW				1024
#define IMG_MM_GRAY_BALL				1025
#define IMG_MM_FUEL_FULL				1026
#define IMG_MM_FUEL_EMPTY				1027
#define IMG_MM_STEEL_WALL				1028
#define IMG_MM_WOODEN_WALL				1029
#define IMG_MM_ICE_WALL					1030
#define IMG_MM_ICE_WALL_SHRINKING			1031
#define IMG_MM_AMOEBA_WALL				1032
#define IMG_MM_AMOEBA_WALL_GROWING			1033
#define IMG_MM_PACMAN					1034
#define IMG_MM_PACMAN_RIGHT				1035
#define IMG_MM_PACMAN_UP				1036
#define IMG_MM_PACMAN_LEFT				1037
#define IMG_MM_PACMAN_DOWN				1038
#define IMG_MM_PACMAN_EATING_RIGHT			1039
#define IMG_MM_PACMAN_EATING_UP				1040
#define IMG_MM_PACMAN_EATING_LEFT			1041
#define IMG_MM_PACMAN_EATING_DOWN			1042
#define IMG_MM_MASK_MCDUFFIN_RIGHT			1043
#define IMG_MM_MASK_MCDUFFIN_UP				1044
#define IMG_MM_MASK_MCDUFFIN_LEFT			1045
#define IMG_MM_MASK_MCDUFFIN_DOWN			1046
#define IMG_MM_MASK_GRID_1				1047
#define IMG_MM_MASK_GRID_2				1048
#define IMG_MM_MASK_GRID_3				1049
#define IMG_MM_MASK_GRID_4				1050
#define IMG_MM_MASK_RECTANGLE				1051
#define IMG_MM_MASK_CIRCLE				1052
#define IMG_MM_DEFAULT_EXPLODING			1053
#define IMG_DF_LASER					1054
#define IMG_DF_LASER_RIGHT				1055
#define IMG_DF_LASER_UP					1056
#define IMG_DF_LASER_LEFT				1057
#define IMG_DF_LASER_DOWN				1058
#define IMG_DF_RECEIVER					1059
#define IMG_DF_RECEIVER_RIGHT				1060
#define IMG_DF_RECEIVER_UP				1061
#define IMG_DF_RECEIVER_LEFT				1062
#define IMG_DF_RECEIVER_DOWN				1063
#define IMG_DF_MIRROR_1					1064
#define IMG_DF_MIRROR_2					1065
#define IMG_DF_MIRROR_3					1066
#define IMG_DF_MIRROR_4					1067
#define IMG_DF_MIRROR_5					1068
#define IMG_DF_MIRROR_6					1069
#define IMG_DF_MIRROR_7					1070
#define IMG_DF_MIRROR_8					1071
#define IMG_DF_MIRROR_9					1072
#define IMG_DF_MIRROR_10				1073
#define IMG_DF_MIRROR_11				1074
#define IMG_DF_MIRROR_12				1075
#define IMG_DF_MIRROR_13				1076
#define IMG_DF_MIRROR_14				1077
#define IMG_DF_MIRROR_15				1078
#define IMG_DF_MIRROR_16				1079
#define IMG_DF_MIRROR_ROTATING_1			1080
#define IMG_DF_MIRROR_ROTATING_1_EDITOR			1081
#define IMG_DF_MIRROR_ROTATING_2			1082
#define IMG_DF_MIRROR_ROTATING_2_EDITOR			1083
#define IMG_DF_MIRROR_ROTATING_3			1084
#define IMG_DF_MIRROR_ROTATING_3_EDITOR			1085
#define IMG_DF_MIRROR_ROTATING_4			1086
#define IMG_DF_MIRROR_ROTATING_4_EDITOR			1087
#define IMG_DF_MIRROR_ROTATING_5			1088
#define IMG_DF_MIRROR_ROTATING_5_EDITOR			1089
#define IMG_DF_MIRROR_ROTATING_6			1090
#define IMG_DF_MIRROR_ROTATING_6_EDITOR			1091
#define IMG_DF_MIRROR_ROTATING_7			1092
#define IMG_DF_MIRROR_ROTATING_7_EDITOR			1093
#define IMG_DF_MIRROR_ROTATING_8			1094
#define IMG_DF_MIRROR_ROTATING_8_EDITOR			1095
#define IMG_DF_MIRROR_ROTATING_9			1096
#define IMG_DF_MIRROR_ROTATING_9_EDITOR			1097
#define IMG_DF_MIRROR_ROTATING_10			1098
#define IMG_DF_MIRROR_ROTATING_10_EDITOR		1099
#define IMG_DF_MIRROR_ROTATING_11			1100
#define IMG_DF_MIRROR_ROTATING_11_EDITOR		1101
#define IMG_DF_MIRROR_ROTATING_12			1102
#define IMG_DF_MIRROR_ROTATING_12_EDITOR		1103
#define IMG_DF_MIRROR_ROTATING_13			1104
#define IMG_DF_MIRROR_ROTATING_13_EDITOR		1105
#define IMG_DF_MIRROR_ROTATING_14			1106
#define IMG_DF_MIRROR_ROTATING_14_EDITOR		1107
#define IMG_DF_MIRROR_ROTATING_15			1108
#define IMG_DF_MIRROR_ROTATING_15_EDITOR		1109
#define IMG_DF_MIRROR_ROTATING_16			1110
#define IMG_DF_MIRROR_ROTATING_16_EDITOR		1111
#define IMG_DF_STEEL_GRID_FIXED_1			1112
#define IMG_DF_STEEL_GRID_FIXED_2			1113
#define IMG_DF_STEEL_GRID_FIXED_3			1114
#define IMG_DF_STEEL_GRID_FIXED_4			1115
#define IMG_DF_STEEL_GRID_FIXED_5			1116
#define IMG_DF_STEEL_GRID_FIXED_6			1117
#define IMG_DF_STEEL_GRID_FIXED_7			1118
#define IMG_DF_STEEL_GRID_FIXED_8			1119
#define IMG_DF_WOODEN_GRID_FIXED_1			1120
#define IMG_DF_WOODEN_GRID_FIXED_2			1121
#define IMG_DF_WOODEN_GRID_FIXED_3			1122
#define IMG_DF_WOODEN_GRID_FIXED_4			1123
#define IMG_DF_WOODEN_GRID_FIXED_5			1124
#define IMG_DF_WOODEN_GRID_FIXED_6			1125
#define IMG_DF_WOODEN_GRID_FIXED_7			1126
#define IMG_DF_WOODEN_GRID_FIXED_8			1127
#define IMG_DF_STEEL_GRID_ROTATING_1			1128
#define IMG_DF_STEEL_GRID_ROTATING_1_EDITOR		1129
#define IMG_DF_STEEL_GRID_ROTATING_2			1130
#define IMG_DF_STEEL_GRID_ROTATING_2_EDITOR		1131
#define IMG_DF_STEEL_GRID_ROTATING_3			1132
#define IMG_DF_STEEL_GRID_ROTATING_3_EDITOR		1133
#define IMG_DF_STEEL_GRID_ROTATING_4			1134
#define IMG_DF_STEEL_GRID_ROTATING_4_EDITOR		1135
#define IMG_DF_STEEL_GRID_ROTATING_5			1136
#define IMG_DF_STEEL_GRID_ROTATING_5_EDITOR		1137
#define IMG_DF_STEEL_GRID_ROTATING_6			1138
#define IMG_DF_STEEL_GRID_ROTATING_6_EDITOR		1139
#define IMG_DF_STEEL_GRID_ROTATING_7			1140
#define IMG_DF_STEEL_GRID_ROTATING_7_EDITOR		1141
#define IMG_DF_STEEL_GRID_ROTATING_8			1142
#define IMG_DF_STEEL_GRID_ROTATING_8_EDITOR		1143
#define IMG_DF_WOODEN_GRID_ROTATING_1			1144
#define IMG_DF_WOODEN_GRID_ROTATING_1_EDITOR		1145
#define IMG_DF_WOODEN_GRID_ROTATING_2			1146
#define IMG_DF_WOODEN_GRID_ROTATING_2_EDITOR		1147
#define IMG_DF_WOODEN_GRID_ROTATING_3			1148
#define IMG_DF_WOODEN_GRID_ROTATING_3_EDITOR		1149
#define IMG_DF_WOODEN_GRID_ROTATING_4			1150
#define IMG_DF_WOODEN_GRID_ROTATING_4_EDITOR		1151
#define IMG_DF_WOODEN_GRID_ROTATING_5			1152
#define IMG_DF_WOODEN_GRID_ROTATING_5_EDITOR		1153
#define IMG_DF_WOODEN_GRID_ROTATING_6			1154
#define IMG_DF_WOODEN_GRID_ROTATING_6_EDITOR		1155
#define IMG_DF_WOODEN_GRID_ROTATING_7			1156
#define IMG_DF_WOODEN_GRID_ROTATING_7_EDITOR		1157
#define IMG_DF_WOODEN_GRID_ROTATING_8			1158
#define IMG_DF_WOODEN_GRID_ROTATING_8_EDITOR		1159
#define IMG_DF_FIBRE_OPTIC_RED_1			1160
#define IMG_DF_FIBRE_OPTIC_RED_1_EDITOR			1161
#define IMG_DF_FIBRE_OPTIC_RED_2			1162
#define IMG_DF_FIBRE_OPTIC_RED_2_EDITOR			1163
#define IMG_DF_FIBRE_OPTIC_YELLOW_1			1164
#define IMG_DF_FIBRE_OPTIC_YELLOW_1_EDITOR		1165
#define IMG_DF_FIBRE_OPTIC_YELLOW_2			1166
#define IMG_DF_FIBRE_OPTIC_YELLOW_2_EDITOR		1167
#define IMG_DF_FIBRE_OPTIC_GREEN_1			1168
#define IMG_DF_FIBRE_OPTIC_GREEN_1_EDITOR		1169
#define IMG_DF_FIBRE_OPTIC_GREEN_2			1170
#define IMG_DF_FIBRE_OPTIC_GREEN_2_EDITOR		1171
#define IMG_DF_FIBRE_OPTIC_BLUE_1			1172
#define IMG_DF_FIBRE_OPTIC_BLUE_1_EDITOR		1173
#define IMG_DF_FIBRE_OPTIC_BLUE_2			1174
#define IMG_DF_FIBRE_OPTIC_BLUE_2_EDITOR		1175
#define IMG_DF_STEEL_WALL				1176
#define IMG_DF_WOODEN_WALL				1177
#define IMG_DF_REFRACTOR				1178
#define IMG_DF_CELL					1179
#define IMG_DF_MINE					1180
#define IMG_GRAPHIC_1					1181
#define IMG_GRAPHIC_2					1182
#define IMG_GRAPHIC_3					1183
#define IMG_GRAPHIC_4					1184
#define IMG_GRAPHIC_5					1185
#define IMG_GRAPHIC_6					1186
#define IMG_GRAPHIC_7					1187
#define IMG_GRAPHIC_8					1188
#define IMG_CHAR_SPACE					1189
#define IMG_CHAR_SPACE_EDITOR				1190
#define IMG_CHAR_EXCLAM					1191
#define IMG_CHAR_QUOTEDBL				1192
#define IMG_CHAR_NUMBERSIGN				1193
#define IMG_CHAR_DOLLAR					1194
#define IMG_CHAR_PERCENT				1195
#define IMG_CHAR_AMPERSAND				1196
#define IMG_CHAR_APOSTROPHE				1197
#define IMG_CHAR_PARENLEFT				1198
#define IMG_CHAR_PARENRIGHT				1199
#define IMG_CHAR_ASTERISK				1200
#define IMG_CHAR_PLUS					1201
#define IMG_CHAR_COMMA					1202
#define IMG_CHAR_MINUS					1203
#define IMG_CHAR_PERIOD					1204
#define IMG_CHAR_SLASH					1205
#define IMG_CHAR_0					1206
#define IMG_CHAR_1					1207
#define IMG_CHAR_2					1208
#define IMG_CHAR_3					1209
#define IMG_CHAR_4					1210
#define IMG_CHAR_5					1211
#define IMG_CHAR_6					1212
#define IMG_CHAR_7					1213
#define IMG_CHAR_8					1214
#define IMG_CHAR_9					1215
#define IMG_CHAR_COLON					1216
#define IMG_CHAR_SEMICOLON				1217
#define IMG_CHAR_LESS					1218
#define IMG_CHAR_EQUAL					1219
#define IMG_CHAR_GREATER				1220
#define IMG_CHAR_QUESTION				1221
#define IMG_CHAR_AT					1222
#define IMG_CHAR_A					1223
#define IMG_CHAR_B					1224
#define IMG_CHAR_C					1225
#define IMG_CHAR_D					1226
#define IMG_CHAR_E					1227
#define IMG_CHAR_F					1228
#define IMG_CHAR_G					1229
#define IMG_CHAR_H					1230
#define IMG_CHAR_I					1231
#define IMG_CHAR_J					1232
#define IMG_CHAR_K					1233
#define IMG_CHAR_L					1234
#define IMG_CHAR_M					1235
#define IMG_CHAR_N					1236
#define IMG_CHAR_O					1237
#define IMG_CHAR_P					1238
#define IMG_CHAR_Q					1239
#define IMG_CHAR_R					1240
#define IMG_CHAR_S					1241
#define IMG_CHAR_T					1242
#define IMG_CHAR_U					1243
#define IMG_CHAR_V					1244
#define IMG_CHAR_W					1245
#define IMG_CHAR_X					1246
#define IMG_CHAR_Y					1247
#define IMG_CHAR_Z					1248
#define IMG_CHAR_BRACKETLEFT				1249
#define IMG_CHAR_BACKSLASH				1250
#define IMG_CHAR_BRACKETRIGHT				1251
#define IMG_CHAR_ASCIICIRCUM				1252
#define IMG_CHAR_UNDERSCORE				1253
#define IMG_CHAR_COPYRIGHT				1254
#define IMG_CHAR_AUMLAUT				1255
#define IMG_CHAR_OUMLAUT				1256
#define IMG_CHAR_UUMLAUT				1257
#define IMG_CHAR_DEGREE					1258
#define IMG_CHAR_TRADEMARK				1259
#define IMG_CHAR_CURSOR					1260
#define IMG_CHAR_BUTTON					1261
#define IMG_CHAR_UP					1262
#define IMG_CHAR_DOWN					1263
#define IMG_STEEL_CHAR_SPACE				1264
#define IMG_STEEL_CHAR_SPACE_EDITOR			1265
#define IMG_STEEL_CHAR_EXCLAM				1266
#define IMG_STEEL_CHAR_QUOTEDBL				1267
#define IMG_STEEL_CHAR_NUMBERSIGN			1268
#define IMG_STEEL_CHAR_DOLLAR				1269
#define IMG_STEEL_CHAR_PERCENT				1270
#define IMG_STEEL_CHAR_AMPERSAND			1271
#define IMG_STEEL_CHAR_APOSTROPHE			1272
#define IMG_STEEL_CHAR_PARENLEFT			1273
#define IMG_STEEL_CHAR_PARENRIGHT			1274
#define IMG_STEEL_CHAR_ASTERISK				1275
#define IMG_STEEL_CHAR_PLUS				1276
#define IMG_STEEL_CHAR_COMMA				1277
#define IMG_STEEL_CHAR_MINUS				1278
#define IMG_STEEL_CHAR_PERIOD				1279
#define IMG_STEEL_CHAR_SLASH				1280
#define IMG_STEEL_CHAR_0				1281
#define IMG_STEEL_CHAR_1				1282
#define IMG_STEEL_CHAR_2				1283
#define IMG_STEEL_CHAR_3				1284
#define IMG_STEEL_CHAR_4				1285
#define IMG_STEEL_CHAR_5				1286
#define IMG_STEEL_CHAR_6				1287
#define IMG_STEEL_CHAR_7				1288
#define IMG_STEEL_CHAR_8				1289
#define IMG_STEEL_CHAR_9				1290
#define IMG_STEEL_CHAR_COLON				1291
#define IMG_STEEL_CHAR_SEMICOLON			1292
#define IMG_STEEL_CHAR_LESS				1293
#define IMG_STEEL_CHAR_EQUAL				1294
#define IMG_STEEL_CHAR_GREATER				1295
#define IMG_STEEL_CHAR_QUESTION				1296
#define IMG_STEEL_CHAR_AT				1297
#define IMG_STEEL_CHAR_A				1298
#define IMG_STEEL_CHAR_B				1299
#define IMG_STEEL_CHAR_C				1300
#define IMG_STEEL_CHAR_D				1301
#define IMG_STEEL_CHAR_E				1302
#define IMG_STEEL_CHAR_F				1303
#define IMG_STEEL_CHAR_G				1304
#define IMG_STEEL_CHAR_H				1305
#define IMG_STEEL_CHAR_I				1306
#define IMG_STEEL_CHAR_J				1307
#define IMG_STEEL_CHAR_K				1308
#define IMG_STEEL_CHAR_L				1309
#define IMG_STEEL_CHAR_M				1310
#define IMG_STEEL_CHAR_N				1311
#define IMG_STEEL_CHAR_O				1312
#define IMG_STEEL_CHAR_P				1313
#define IMG_STEEL_CHAR_Q				1314
#define IMG_STEEL_CHAR_R				1315
#define IMG_STEEL_CHAR_S				1316
#define IMG_STEEL_CHAR_T				1317
#define IMG_STEEL_CHAR_U				1318
#define IMG_STEEL_CHAR_V				1319
#define IMG_STEEL_CHAR_W				1320
#define IMG_STEEL_CHAR_X				1321
#define IMG_STEEL_CHAR_Y				1322
#define IMG_STEEL_CHAR_Z				1323
#define IMG_STEEL_CHAR_BRACKETLEFT			1324
#define IMG_STEEL_CHAR_BACKSLASH			1325
#define IMG_STEEL_CHAR_BRACKETRIGHT			1326
#define IMG_STEEL_CHAR_ASCIICIRCUM			1327
#define IMG_STEEL_CHAR_UNDERSCORE			1328
#define IMG_STEEL_CHAR_COPYRIGHT			1329
#define IMG_STEEL_CHAR_AUMLAUT				1330
#define IMG_STEEL_CHAR_OUMLAUT				1331
#define IMG_STEEL_CHAR_UUMLAUT				1332
#define IMG_STEEL_CHAR_DEGREE				1333
#define IMG_STEEL_CHAR_TRADEMARK			1334
#define IMG_STEEL_CHAR_CURSOR				1335
#define IMG_STEEL_CHAR_BUTTON				1336
#define IMG_STEEL_CHAR_UP				1337
#define IMG_STEEL_CHAR_DOWN				1338
#define IMG_CUSTOM_1					1339
#define IMG_CUSTOM_1_EDITOR				1340
#define IMG_CUSTOM_2					1341
#define IMG_CUSTOM_2_EDITOR				1342
#define IMG_CUSTOM_3					1343
#define IMG_CUSTOM_3_EDITOR				1344
#define IMG_CUSTOM_4					1345
#define IMG_CUSTOM_4_EDITOR				1346
#define IMG_CUSTOM_5					1347
#define IMG_CUSTOM_5_EDITOR				1348
#define IMG_CUSTOM_6					1349
#define IMG_CUSTOM_6_EDITOR				1350
#define IMG_CUSTOM_7					1351
#define IMG_CUSTOM_7_EDITOR				1352
#define IMG_CUSTOM_8					1353
#define IMG_CUSTOM_8_EDITOR				1354
#define IMG_CUSTOM_9					1355
#define IMG_CUSTOM_9_EDITOR				1356
#define IMG_CUSTOM_10					1357
#define IMG_CUSTOM_10_EDITOR				1358
#define IMG_CUSTOM_11					1359
#define IMG_CUSTOM_11_EDITOR				1360
#define IMG_CUSTOM_12					1361
#define IMG_CUSTOM_12_EDITOR				1362
#define IMG_CUSTOM_13					1363
#define IMG_CUSTOM_13_EDITOR				1364
#define IMG_CUSTOM_14					1365
#define IMG_CUSTOM_14_EDITOR				1366
#define IMG_CUSTOM_15					1367
#define IMG_CUSTOM_15_EDITOR				1368
#define IMG_CUSTOM_16					1369
#define IMG_CUSTOM_16_EDITOR				1370
#define IMG_CUSTOM_17					1371
#define IMG_CUSTOM_17_EDITOR				1372
#define IMG_CUSTOM_18					1373
#define IMG_CUSTOM_18_EDITOR				1374
#define IMG_CUSTOM_19					1375
#define IMG_CUSTOM_19_EDITOR				1376
#define IMG_CUSTOM_20					1377
#define IMG_CUSTOM_20_EDITOR				1378
#define IMG_CUSTOM_21					1379
#define IMG_CUSTOM_21_EDITOR				1380
#define IMG_CUSTOM_22					1381
#define IMG_CUSTOM_22_EDITOR				1382
#define IMG_CUSTOM_23					1383
#define IMG_CUSTOM_23_EDITOR				1384
#define IMG_CUSTOM_24					1385
#define IMG_CUSTOM_24_EDITOR				1386
#define IMG_CUSTOM_25					1387
#define IMG_CUSTOM_25_EDITOR				1388
#define IMG_CUSTOM_26					1389
#define IMG_CUSTOM_26_EDITOR				1390
#define IMG_CUSTOM_27					1391
#define IMG_CUSTOM_27_EDITOR				1392
#define IMG_CUSTOM_28					1393
#define IMG_CUSTOM_28_EDITOR				1394
#define IMG_CUSTOM_29					1395
#define IMG_CUSTOM_29_EDITOR				1396
#define IMG_CUSTOM_30					1397
#define IMG_CUSTOM_30_EDITOR				1398
#define IMG_CUSTOM_31					1399
#define IMG_CUSTOM_31_EDITOR				1400
#define IMG_CUSTOM_32					1401
#define IMG_CUSTOM_32_EDITOR				1402
#define IMG_CUSTOM_33					1403
#define IMG_CUSTOM_33_EDITOR				1404
#define IMG_CUSTOM_34					1405
#define IMG_CUSTOM_34_EDITOR				1406
#define IMG_CUSTOM_35					1407
#define IMG_CUSTOM_35_EDITOR				1408
#define IMG_CUSTOM_36					1409
#define IMG_CUSTOM_36_EDITOR				1410
#define IMG_CUSTOM_37					1411
#define IMG_CUSTOM_37_EDITOR				1412
#define IMG_CUSTOM_38					1413
#define IMG_CUSTOM_38_EDITOR				1414
#define IMG_CUSTOM_39					1415
#define IMG_CUSTOM_39_EDITOR				1416
#define IMG_CUSTOM_40					1417
#define IMG_CUSTOM_40_EDITOR				1418
#define IMG_CUSTOM_41					1419
#define IMG_CUSTOM_41_EDITOR				1420
#define IMG_CUSTOM_42					1421
#define IMG_CUSTOM_42_EDITOR				1422
#define IMG_CUSTOM_43					1423
#define IMG_CUSTOM_43_EDITOR				1424
#define IMG_CUSTOM_44					1425
#define IMG_CUSTOM_44_EDITOR				1426
#define IMG_CUSTOM_45					1427
#define IMG_CUSTOM_45_EDITOR				1428
#define IMG_CUSTOM_46					1429
#define IMG_CUSTOM_46_EDITOR				1430
#define IMG_CUSTOM_47					1431
#define IMG_CUSTOM_47_EDITOR				1432
#define IMG_CUSTOM_48					1433
#define IMG_CUSTOM_48_EDITOR				1434
#define IMG_CUSTOM_49					1435
#define IMG_CUSTOM_49_EDITOR				1436
#define IMG_CUSTOM_50					1437
#define IMG_CUSTOM_50_EDITOR				1438
#define IMG_CUSTOM_51					1439
#define IMG_CUSTOM_51_EDITOR				1440
#define IMG_CUSTOM_52					1441
#define IMG_CUSTOM_52_EDITOR				1442
#define IMG_CUSTOM_53					1443
#define IMG_CUSTOM_53_EDITOR				1444
#define IMG_CUSTOM_54					1445
#define IMG_CUSTOM_54_EDITOR				1446
#define IMG_CUSTOM_55					1447
#define IMG_CUSTOM_55_EDITOR				1448
#define IMG_CUSTOM_56					1449
#define IMG_CUSTOM_56_EDITOR				1450
#define IMG_CUSTOM_57					1451
#define IMG_CUSTOM_57_EDITOR				1452
#define IMG_CUSTOM_58					1453
#define IMG_CUSTOM_58_EDITOR				1454
#define IMG_CUSTOM_59					1455
#define IMG_CUSTOM_59_EDITOR				1456
#define IMG_CUSTOM_60					1457
#define IMG_CUSTOM_60_EDITOR				1458
#define IMG_CUSTOM_61					1459
#define IMG_CUSTOM_61_EDITOR				1460
#define IMG_CUSTOM_62					1461
#define IMG_CUSTOM_62_EDITOR				1462
#define IMG_CUSTOM_63					1463
#define IMG_CUSTOM_63_EDITOR				1464
#define IMG_CUSTOM_64					1465
#define IMG_CUSTOM_64_EDITOR				1466
#define IMG_CUSTOM_65					1467
#define IMG_CUSTOM_65_EDITOR				1468
#define IMG_CUSTOM_66					1469
#define IMG_CUSTOM_66_EDITOR				1470
#define IMG_CUSTOM_67					1471
#define IMG_CUSTOM_67_EDITOR				1472
#define IMG_CUSTOM_68					1473
#define IMG_CUSTOM_68_EDITOR				1474
#define IMG_CUSTOM_69					1475
#define IMG_CUSTOM_69_EDITOR				1476
#define IMG_CUSTOM_70					1477
#define IMG_CUSTOM_70_EDITOR				1478
#define IMG_CUSTOM_71					1479
#define IMG_CUSTOM_71_EDITOR				1480
#define IMG_CUSTOM_72					1481
#define IMG_CUSTOM_72_EDITOR				1482
#define IMG_CUSTOM_73					1483
#define IMG_CUSTOM_73_EDITOR				1484
#define IMG_CUSTOM_74					1485
#define IMG_CUSTOM_74_EDITOR				1486
#define IMG_CUSTOM_75					1487
#define IMG_CUSTOM_75_EDITOR				1488
#define IMG_CUSTOM_76					1489
#define IMG_CUSTOM_76_EDITOR				1490
#define IMG_CUSTOM_77					1491
#define IMG_CUSTOM_77_EDITOR				1492
#define IMG_CUSTOM_78					1493
#define IMG_CUSTOM_78_EDITOR				1494
#define IMG_CUSTOM_79					1495
#define IMG_CUSTOM_79_EDITOR				1496
#define IMG_CUSTOM_80					1497
#define IMG_CUSTOM_80_EDITOR				1498
#define IMG_CUSTOM_81					1499
#define IMG_CUSTOM_81_EDITOR				1500
#define IMG_CUSTOM_82					1501
#define IMG_CUSTOM_82_EDITOR				1502
#define IMG_CUSTOM_83					1503
#define IMG_CUSTOM_83_EDITOR				1504
#define IMG_CUSTOM_84					1505
#define IMG_CUSTOM_84_EDITOR				1506
#define IMG_CUSTOM_85					1507
#define IMG_CUSTOM_85_EDITOR				1508
#define IMG_CUSTOM_86					1509
#define IMG_CUSTOM_86_EDITOR				1510
#define IMG_CUSTOM_87					1511
#define IMG_CUSTOM_87_EDITOR				1512
#define IMG_CUSTOM_88					1513
#define IMG_CUSTOM_88_EDITOR				1514
#define IMG_CUSTOM_89					1515
#define IMG_CUSTOM_89_EDITOR				1516
#define IMG_CUSTOM_90					1517
#define IMG_CUSTOM_90_EDITOR				1518
#define IMG_CUSTOM_91					1519
#define IMG_CUSTOM_91_EDITOR				1520
#define IMG_CUSTOM_92					1521
#define IMG_CUSTOM_92_EDITOR				1522
#define IMG_CUSTOM_93					1523
#define IMG_CUSTOM_93_EDITOR				1524
#define IMG_CUSTOM_94					1525
#define IMG_CUSTOM_94_EDITOR				1526
#define IMG_CUSTOM_95					1527
#define IMG_CUSTOM_95_EDITOR				1528
#define IMG_CUSTOM_96					1529
#define IMG_CUSTOM_96_EDITOR				1530
#define IMG_CUSTOM_97					1531
#define IMG_CUSTOM_97_EDITOR				1532
#define IMG_CUSTOM_98					1533
#define IMG_CUSTOM_98_EDITOR				1534
#define IMG_CUSTOM_99					1535
#define IMG_CUSTOM_99_EDITOR				1536
#define IMG_CUSTOM_100					1537
#define IMG_CUSTOM_100_EDITOR				1538
#define IMG_CUSTOM_101					1539
#define IMG_CUSTOM_101_EDITOR				1540
#define IMG_CUSTOM_102					1541
#define IMG_CUSTOM_102_EDITOR				1542
#define IMG_CUSTOM_103					1543
#define IMG_CUSTOM_103_EDITOR				1544
#define IMG_CUSTOM_104					1545
#define IMG_CUSTOM_104_EDITOR				1546
#define IMG_CUSTOM_105					1547
#define IMG_CUSTOM_105_EDITOR				1548
#define IMG_CUSTOM_106					1549
#define IMG_CUSTOM_106_EDITOR				1550
#define IMG_CUSTOM_107					1551
#define IMG_CUSTOM_107_EDITOR				1552
#define IMG_CUSTOM_108					1553
#define IMG_CUSTOM_108_EDITOR				1554
#define IMG_CUSTOM_109					1555
#define IMG_CUSTOM_109_EDITOR				1556
#define IMG_CUSTOM_110					1557
#define IMG_CUSTOM_110_EDITOR				1558
#define IMG_CUSTOM_111					1559
#define IMG_CUSTOM_111_EDITOR				1560
#define IMG_CUSTOM_112					1561
#define IMG_CUSTOM_112_EDITOR				1562
#define IMG_CUSTOM_113					1563
#define IMG_CUSTOM_113_EDITOR				1564
#define IMG_CUSTOM_114					1565
#define IMG_CUSTOM_114_EDITOR				1566
#define IMG_CUSTOM_115					1567
#define IMG_CUSTOM_115_EDITOR				1568
#define IMG_CUSTOM_116					1569
#define IMG_CUSTOM_116_EDITOR				1570
#define IMG_CUSTOM_117					1571
#define IMG_CUSTOM_117_EDITOR				1572
#define IMG_CUSTOM_118					1573
#define IMG_CUSTOM_118_EDITOR				1574
#define IMG_CUSTOM_119					1575
#define IMG_CUSTOM_119_EDITOR				1576
#define IMG_CUSTOM_120					1577
#define IMG_CUSTOM_120_EDITOR				1578
#define IMG_CUSTOM_121					1579
#define IMG_CUSTOM_121_EDITOR				1580
#define IMG_CUSTOM_122					1581
#define IMG_CUSTOM_122_EDITOR				1582
#define IMG_CUSTOM_123					1583
#define IMG_CUSTOM_123_EDITOR				1584
#define IMG_CUSTOM_124					1585
#define IMG_CUSTOM_124_EDITOR				1586
#define IMG_CUSTOM_125					1587
#define IMG_CUSTOM_125_EDITOR				1588
#define IMG_CUSTOM_126					1589
#define IMG_CUSTOM_126_EDITOR				1590
#define IMG_CUSTOM_127					1591
#define IMG_CUSTOM_127_EDITOR				1592
#define IMG_CUSTOM_128					1593
#define IMG_CUSTOM_128_EDITOR				1594
#define IMG_CUSTOM_129					1595
#define IMG_CUSTOM_129_EDITOR				1596
#define IMG_CUSTOM_130					1597
#define IMG_CUSTOM_130_EDITOR				1598
#define IMG_CUSTOM_131					1599
#define IMG_CUSTOM_131_EDITOR				1600
#define IMG_CUSTOM_132					1601
#define IMG_CUSTOM_132_EDITOR				1602
#define IMG_CUSTOM_133					1603
#define IMG_CUSTOM_133_EDITOR				1604
#define IMG_CUSTOM_134					1605
#define IMG_CUSTOM_134_EDITOR				1606
#define IMG_CUSTOM_135					1607
#define IMG_CUSTOM_135_EDITOR				1608
#define IMG_CUSTOM_136					1609
#define IMG_CUSTOM_136_EDITOR				1610
#define IMG_CUSTOM_137					1611
#define IMG_CUSTOM_137_EDITOR				1612
#define IMG_CUSTOM_138					1613
#define IMG_CUSTOM_138_EDITOR				1614
#define IMG_CUSTOM_139					1615
#define IMG_CUSTOM_139_EDITOR				1616
#define IMG_CUSTOM_140					1617
#define IMG_CUSTOM_140_EDITOR				1618
#define IMG_CUSTOM_141					1619
#define IMG_CUSTOM_141_EDITOR				1620
#define IMG_CUSTOM_142					1621
#define IMG_CUSTOM_142_EDITOR				1622
#define IMG_CUSTOM_143					1623
#define IMG_CUSTOM_143_EDITOR				1624
#define IMG_CUSTOM_144					1625
#define IMG_CUSTOM_144_EDITOR				1626
#define IMG_CUSTOM_145					1627
#define IMG_CUSTOM_145_EDITOR				1628
#define IMG_CUSTOM_146					1629
#define IMG_CUSTOM_146_EDITOR				1630
#define IMG_CUSTOM_147					1631
#define IMG_CUSTOM_147_EDITOR				1632
#define IMG_CUSTOM_148					1633
#define IMG_CUSTOM_148_EDITOR				1634
#define IMG_CUSTOM_149					1635
#define IMG_CUSTOM_149_EDITOR				1636
#define IMG_CUSTOM_150					1637
#define IMG_CUSTOM_150_EDITOR				1638
#define IMG_CUSTOM_151					1639
#define IMG_CUSTOM_151_EDITOR				1640
#define IMG_CUSTOM_152					1641
#define IMG_CUSTOM_152_EDITOR				1642
#define IMG_CUSTOM_153					1643
#define IMG_CUSTOM_153_EDITOR				1644
#define IMG_CUSTOM_154					1645
#define IMG_CUSTOM_154_EDITOR				1646
#define IMG_CUSTOM_155					1647
#define IMG_CUSTOM_155_EDITOR				1648
#define IMG_CUSTOM_156					1649
#define IMG_CUSTOM_156_EDITOR				1650
#define IMG_CUSTOM_157					1651
#define IMG_CUSTOM_157_EDITOR				1652
#define IMG_CUSTOM_158					1653
#define IMG_CUSTOM_158_EDITOR				1654
#define IMG_CUSTOM_159					1655
#define IMG_CUSTOM_159_EDITOR				1656
#define IMG_CUSTOM_160					1657
#define IMG_CUSTOM_160_EDITOR				1658
#define IMG_CUSTOM_161					1659
#define IMG_CUSTOM_161_EDITOR				1660
#define IMG_CUSTOM_162					1661
#define IMG_CUSTOM_162_EDITOR				1662
#define IMG_CUSTOM_163					1663
#define IMG_CUSTOM_163_EDITOR				1664
#define IMG_CUSTOM_164					1665
#define IMG_CUSTOM_164_EDITOR				1666
#define IMG_CUSTOM_165					1667
#define IMG_CUSTOM_165_EDITOR				1668
#define IMG_CUSTOM_166					1669
#define IMG_CUSTOM_166_EDITOR				1670
#define IMG_CUSTOM_167					1671
#define IMG_CUSTOM_167_EDITOR				1672
#define IMG_CUSTOM_168					1673
#define IMG_CUSTOM_168_EDITOR				1674
#define IMG_CUSTOM_169					1675
#define IMG_CUSTOM_169_EDITOR				1676
#define IMG_CUSTOM_170					1677
#define IMG_CUSTOM_170_EDITOR				1678
#define IMG_CUSTOM_171					1679
#define IMG_CUSTOM_171_EDITOR				1680
#define IMG_CUSTOM_172					1681
#define IMG_CUSTOM_172_EDITOR				1682
#define IMG_CUSTOM_173					1683
#define IMG_CUSTOM_173_EDITOR				1684
#define IMG_CUSTOM_174					1685
#define IMG_CUSTOM_174_EDITOR				1686
#define IMG_CUSTOM_175					1687
#define IMG_CUSTOM_175_EDITOR				1688
#define IMG_CUSTOM_176					1689
#define IMG_CUSTOM_176_EDITOR				1690
#define IMG_CUSTOM_177					1691
#define IMG_CUSTOM_177_EDITOR				1692
#define IMG_CUSTOM_178					1693
#define IMG_CUSTOM_178_EDITOR				1694
#define IMG_CUSTOM_179					1695
#define IMG_CUSTOM_179_EDITOR				1696
#define IMG_CUSTOM_180					1697
#define IMG_CUSTOM_180_EDITOR				1698
#define IMG_CUSTOM_181					1699
#define IMG_CUSTOM_181_EDITOR				1700
#define IMG_CUSTOM_182					1701
#define IMG_CUSTOM_182_EDITOR				1702
#define IMG_CUSTOM_183					1703
#define IMG_CUSTOM_183_EDITOR				1704
#define IMG_CUSTOM_184					1705
#define IMG_CUSTOM_184_EDITOR				1706
#define IMG_CUSTOM_185					1707
#define IMG_CUSTOM_185_EDITOR				1708
#define IMG_CUSTOM_186					1709
#define IMG_CUSTOM_186_EDITOR				1710
#define IMG_CUSTOM_187					1711
#define IMG_CUSTOM_187_EDITOR				1712
#define IMG_CUSTOM_188					1713
#define IMG_CUSTOM_188_EDITOR				1714
#define IMG_CUSTOM_189					1715
#define IMG_CUSTOM_189_EDITOR				1716
#define IMG_CUSTOM_190					1717
#define IMG_CUSTOM_190_EDITOR				1718
#define IMG_CUSTOM_191					1719
#define IMG_CUSTOM_191_EDITOR				1720
#define IMG_CUSTOM_192					1721
#define IMG_CUSTOM_192_EDITOR				1722
#define IMG_CUSTOM_193					1723
#define IMG_CUSTOM_193_EDITOR				1724
#define IMG_CUSTOM_194					1725
#define IMG_CUSTOM_194_EDITOR				1726
#define IMG_CUSTOM_195					1727
#define IMG_CUSTOM_195_EDITOR				1728
#define IMG_CUSTOM_196					1729
#define IMG_CUSTOM_196_EDITOR				1730
#define IMG_CUSTOM_197					1731
#define IMG_CUSTOM_197_EDITOR				1732
#define IMG_CUSTOM_198					1733
#define IMG_CUSTOM_198_EDITOR				1734
#define IMG_CUSTOM_199					1735
#define IMG_CUSTOM_199_EDITOR				1736
#define IMG_CUSTOM_200					1737
#define IMG_CUSTOM_200_EDITOR				1738
#define IMG_CUSTOM_201					1739
#define IMG_CUSTOM_201_EDITOR				1740
#define IMG_CUSTOM_202					1741
#define IMG_CUSTOM_202_EDITOR				1742
#define IMG_CUSTOM_203					1743
#define IMG_CUSTOM_203_EDITOR				1744
#define IMG_CUSTOM_204					1745
#define IMG_CUSTOM_204_EDITOR				1746
#define IMG_CUSTOM_205					1747
#define IMG_CUSTOM_205_EDITOR				1748
#define IMG_CUSTOM_206					1749
#define IMG_CUSTOM_206_EDITOR				1750
#define IMG_CUSTOM_207					1751
#define IMG_CUSTOM_207_EDITOR				1752
#define IMG_CUSTOM_208					1753
#define IMG_CUSTOM_208_EDITOR				1754
#define IMG_CUSTOM_209					1755
#define IMG_CUSTOM_209_EDITOR				1756
#define IMG_CUSTOM_210					1757
#define IMG_CUSTOM_210_EDITOR				1758
#define IMG_CUSTOM_211					1759
#define IMG_CUSTOM_211_EDITOR				1760
#define IMG_CUSTOM_212					1761
#define IMG_CUSTOM_212_EDITOR				1762
#define IMG_CUSTOM_213					1763
#define IMG_CUSTOM_213_EDITOR				1764
#define IMG_CUSTOM_214					1765
#define IMG_CUSTOM_214_EDITOR				1766
#define IMG_CUSTOM_215					1767
#define IMG_CUSTOM_215_EDITOR				1768
#define IMG_CUSTOM_216					1769
#define IMG_CUSTOM_216_EDITOR				1770
#define IMG_CUSTOM_217					1771
#define IMG_CUSTOM_217_EDITOR				1772
#define IMG_CUSTOM_218					1773
#define IMG_CUSTOM_218_EDITOR				1774
#define IMG_CUSTOM_219					1775
#define IMG_CUSTOM_219_EDITOR				1776
#define IMG_CUSTOM_220					1777
#define IMG_CUSTOM_220_EDITOR				1778
#define IMG_CUSTOM_221					1779
#define IMG_CUSTOM_221_EDITOR				1780
#define IMG_CUSTOM_222					1781
#define IMG_CUSTOM_222_EDITOR				1782
#define IMG_CUSTOM_223					1783
#define IMG_CUSTOM_223_EDITOR				1784
#define IMG_CUSTOM_224					1785
#define IMG_CUSTOM_224_EDITOR				1786
#define IMG_CUSTOM_225					1787
#define IMG_CUSTOM_225_EDITOR				1788
#define IMG_CUSTOM_226					1789
#define IMG_CUSTOM_226_EDITOR				1790
#define IMG_CUSTOM_227					1791
#define IMG_CUSTOM_227_EDITOR				1792
#define IMG_CUSTOM_228					1793
#define IMG_CUSTOM_228_EDITOR				1794
#define IMG_CUSTOM_229					1795
#define IMG_CUSTOM_229_EDITOR				1796
#define IMG_CUSTOM_230					1797
#define IMG_CUSTOM_230_EDITOR				1798
#define IMG_CUSTOM_231					1799
#define IMG_CUSTOM_231_EDITOR				1800
#define IMG_CUSTOM_232					1801
#define IMG_CUSTOM_232_EDITOR				1802
#define IMG_CUSTOM_233					1803
#define IMG_CUSTOM_233_EDITOR				1804
#define IMG_CUSTOM_234					1805
#define IMG_CUSTOM_234_EDITOR				1806
#define IMG_CUSTOM_235					1807
#define IMG_CUSTOM_235_EDITOR				1808
#define IMG_CUSTOM_236					1809
#define IMG_CUSTOM_236_EDITOR				1810
#define IMG_CUSTOM_237					1811
#define IMG_CUSTOM_237_EDITOR				1812
#define IMG_CUSTOM_238					1813
#define IMG_CUSTOM_238_EDITOR				1814
#define IMG_CUSTOM_239					1815
#define IMG_CUSTOM_239_EDITOR				1816
#define IMG_CUSTOM_240					1817
#define IMG_CUSTOM_240_EDITOR				1818
#define IMG_CUSTOM_241					1819
#define IMG_CUSTOM_241_EDITOR				1820
#define IMG_CUSTOM_242					1821
#define IMG_CUSTOM_242_EDITOR				1822
#define IMG_CUSTOM_243					1823
#define IMG_CUSTOM_243_EDITOR				1824
#define IMG_CUSTOM_244					1825
#define IMG_CUSTOM_244_EDITOR				1826
#define IMG_CUSTOM_245					1827
#define IMG_CUSTOM_245_EDITOR				1828
#define IMG_CUSTOM_246					1829
#define IMG_CUSTOM_246_EDITOR				1830
#define IMG_CUSTOM_247					1831
#define IMG_CUSTOM_247_EDITOR				1832
#define IMG_CUSTOM_248					1833
#define IMG_CUSTOM_248_EDITOR				1834
#define IMG_CUSTOM_249					1835
#define IMG_CUSTOM_249_EDITOR				1836
#define IMG_CUSTOM_250					1837
#define IMG_CUSTOM_250_EDITOR				1838
#define IMG_CUSTOM_251					1839
#define IMG_CUSTOM_251_EDITOR				1840
#define IMG_CUSTOM_252					1841
#define IMG_CUSTOM_252_EDITOR				1842
#define IMG_CUSTOM_253					1843
#define IMG_CUSTOM_253_EDITOR				1844
#define IMG_CUSTOM_254					1845
#define IMG_CUSTOM_254_EDITOR				1846
#define IMG_CUSTOM_255					1847
#define IMG_CUSTOM_255_EDITOR				1848
#define IMG_CUSTOM_256					1849
#define IMG_CUSTOM_256_EDITOR				1850
#define IMG_GROUP_1					1851
#define IMG_GROUP_1_EDITOR				1852
#define IMG_GROUP_2					1853
#define IMG_GROUP_2_EDITOR				1854
#define IMG_GROUP_3					1855
#define IMG_GROUP_3_EDITOR				1856
#define IMG_GROUP_4					1857
#define IMG_GROUP_4_EDITOR				1858
#define IMG_GROUP_5					1859
#define IMG_GROUP_5_EDITOR				1860
#define IMG_GROUP_6					1861
#define IMG_GROUP_6_EDITOR				1862
#define IMG_GROUP_7					1863
#define IMG_GROUP_7_EDITOR				1864
#define IMG_GROUP_8					1865
#define IMG_GROUP_8_EDITOR				1866
#define IMG_GROUP_9					1867
#define IMG_GROUP_9_EDITOR				1868
#define IMG_GROUP_10					1869
#define IMG_GROUP_10_EDITOR				1870
#define IMG_GROUP_11					1871
#define IMG_GROUP_11_EDITOR				1872
#define IMG_GROUP_12					1873
#define IMG_GROUP_12_EDITOR				1874
#define IMG_GROUP_13					1875
#define IMG_GROUP_13_EDITOR				1876
#define IMG_GROUP_14					1877
#define IMG_GROUP_14_EDITOR				1878
#define IMG_GROUP_15					1879
#define IMG_GROUP_15_EDITOR				1880
#define IMG_GROUP_16					1881
#define IMG_GROUP_16_EDITOR				1882
#define IMG_GROUP_17					1883
#define IMG_GROUP_17_EDITOR				1884
#define IMG_GROUP_18					1885
#define IMG_GROUP_18_EDITOR				1886
#define IMG_GROUP_19					1887
#define IMG_GROUP_19_EDITOR				1888
#define IMG_GROUP_20					1889
#define IMG_GROUP_20_EDITOR				1890
#define IMG_GROUP_21					1891
#define IMG_GROUP_21_EDITOR				1892
#define IMG_GROUP_22					1893
#define IMG_GROUP_22_EDITOR				1894
#define IMG_GROUP_23					1895
#define IMG_GROUP_23_EDITOR				1896
#define IMG_GROUP_24					1897
#define IMG_GROUP_24_EDITOR				1898
#define IMG_GROUP_25					1899
#define IMG_GROUP_25_EDITOR				1900
#define IMG_GROUP_26					1901
#define IMG_GROUP_26_EDITOR				1902
#define IMG_GROUP_27					1903
#define IMG_GROUP_27_EDITOR				1904
#define IMG_GROUP_28					1905
#define IMG_GROUP_28_EDITOR				1906
#define IMG_GROUP_29					1907
#define IMG_GROUP_29_EDITOR				1908
#define IMG_GROUP_30					1909
#define IMG_GROUP_30_EDITOR				1910
#define IMG_GROUP_31					1911
#define IMG_GROUP_31_EDITOR				1912
#define IMG_GROUP_32					1913
#define IMG_GROUP_32_EDITOR				1914
#define IMG_SP_FRAME_HORIZONTAL				1915
#define IMG_SP_FRAME_VERTICAL				1916
#define IMG_SP_FRAME_CORNER				1917
#define IMG_TOON_1					1918
#define IMG_TOON_2					1919
#define IMG_TOON_3					1920
#define IMG_TOON_4					1921
#define IMG_TOON_5					1922
#define IMG_TOON_6					1923
#define IMG_TOON_7					1924
#define IMG_TOON_8					1925
#define IMG_TOON_9					1926
#define IMG_TOON_10					1927
#define IMG_TOON_11					1928
#define IMG_TOON_12					1929
#define IMG_TOON_13					1930
#define IMG_TOON_14					1931
#define IMG_TOON_15					1932
#define IMG_TOON_16					1933
#define IMG_TOON_17					1934
#define IMG_TOON_18					1935
#define IMG_TOON_19					1936
#define IMG_TOON_20					1937
#define IMG_GFX_GLOBAL_ANIM_1				1938
#define IMG_GFX_GLOBAL_ANIM_2				1939
#define IMG_GFX_GLOBAL_ANIM_3				1940
#define IMG_GFX_GLOBAL_ANIM_4				1941
#define IMG_GFX_GLOBAL_ANIM_5				1942
#define IMG_GFX_GLOBAL_ANIM_6				1943
#define IMG_GFX_GLOBAL_ANIM_7				1944
#define IMG_GFX_GLOBAL_ANIM_8				1945
#define IMG_GFX_GLOBAL_ANIM_9				1946
#define IMG_GFX_GLOBAL_ANIM_10				1947
#define IMG_GFX_GLOBAL_ANIM_11				1948
#define IMG_GFX_GLOBAL_ANIM_12				1949
#define IMG_GFX_GLOBAL_ANIM_13				1950
#define IMG_GFX_GLOBAL_ANIM_14				1951
#define IMG_GFX_GLOBAL_ANIM_15				1952
#define IMG_GFX_GLOBAL_ANIM_16				1953
#define IMG_GFX_GLOBAL_ANIM_17				1954
#define IMG_GFX_GLOBAL_ANIM_18				1955
#define IMG_GFX_GLOBAL_ANIM_19				1956
#define IMG_GFX_GLOBAL_ANIM_20				1957
#define IMG_GFX_GLOBAL_ANIM_21				1958
#define IMG_GFX_GLOBAL_ANIM_22				1959
#define IMG_GFX_GLOBAL_ANIM_23				1960
#define IMG_GFX_GLOBAL_ANIM_24				1961
#define IMG_GFX_GLOBAL_ANIM_25				1962
#define IMG_GFX_GLOBAL_ANIM_26				1963
#define IMG_GFX_GLOBAL_ANIM_27				1964
#define IMG_GFX_GLOBAL_ANIM_28				1965
#define IMG_GFX_GLOBAL_ANIM_29				1966
#define IMG_GFX_GLOBAL_ANIM_30				1967
#define IMG_GFX_GLOBAL_ANIM_31				1968
#define IMG_GFX_GLOBAL_ANIM_32				1969
#define IMG_GLOBAL_ANIM_1				1970
#define IMG_GLOBAL_ANIM_2				1971
#define IMG_GLOBAL_ANIM_3				1972
#define IMG_GLOBAL_ANIM_4				1973
#define IMG_GLOBAL_ANIM_5				1974
#define IMG_GLOBAL_ANIM_6				1975
#define IMG_GLOBAL_ANIM_7				1976
#define IMG_GLOBAL_ANIM_8				1977
#define IMG_GLOBAL_ANIM_9				1978
#define IMG_GLOBAL_ANIM_10				1979
#define IMG_GLOBAL_ANIM_11				1980
#define IMG_GLOBAL_ANIM_12				1981
#define IMG_GLOBAL_ANIM_13				1982
#define IMG_GLOBAL_ANIM_14				1983
#define IMG_GLOBAL_ANIM_15				1984
#define IMG_GLOBAL_ANIM_16				1985
#define IMG_GLOBAL_ANIM_17				1986
#define IMG_GLOBAL_ANIM_18				1987
#define IMG_GLOBAL_ANIM_19				1988
#define IMG_GLOBAL_ANIM_20				1989
#define IMG_GLOBAL_ANIM_21				1990
#define IMG_GLOBAL_ANIM_22				1991
#define IMG_GLOBAL_ANIM_23				1992
#define IMG_GLOBAL_ANIM_24				1993
#define IMG_GLOBAL_ANIM_25				1994
#define IMG_GLOBAL_ANIM_26				1995
#define IMG_GLOBAL_ANIM_27				1996
#define IMG_GLOBAL_ANIM_28				1997
#define IMG_GLOBAL_ANIM_29				1998
#define IMG_GLOBAL_ANIM_30				1999
#define IMG_GLOBAL_ANIM_31				2000
#define IMG_GLOBAL_ANIM_32				2001
#define IMG_INTERNAL_GLOBAL_TOON_DEFAULT		2002
#define IMG_INTERNAL_GLOBAL_ANIM_DEFAULT		2003
#define IMG_MENU_CALIBRATE_RED				2004
#define IMG_MENU_CALIBRATE_BLUE				2005
#define IMG_MENU_CALIBRATE_YELLOW			2006
#define IMG_MENU_BUTTON					2007
#define IMG_MENU_BUTTON_ACTIVE				2008
#define IMG_MENU_BUTTON_LEFT				2009
#define IMG_MENU_BUTTON_LEFT_ACTIVE			2010
#define IMG_MENU_BUTTON_RIGHT				2011
#define IMG_MENU_BUTTON_RIGHT_ACTIVE			2012
#define IMG_MENU_BUTTON_UP				2013
#define IMG_MENU_BUTTON_UP_ACTIVE			2014
#define IMG_MENU_BUTTON_DOWN				2015
#define IMG_MENU_BUTTON_DOWN_ACTIVE			2016
#define IMG_MENU_BUTTON_ENTER_MENU			2017
#define IMG_MENU_BUTTON_ENTER_MENU_ACTIVE		2018
#define IMG_MENU_BUTTON_LEAVE_MENU			2019
#define IMG_MENU_BUTTON_LEAVE_MENU_ACTIVE		2020
#define IMG_MENU_BUTTON_NEXT_LEVEL			2021
#define IMG_MENU_BUTTON_NEXT_LEVEL_ACTIVE		2022
#define IMG_MENU_BUTTON_PREV_LEVEL			2023
#define IMG_MENU_BUTTON_PREV_LEVEL_ACTIVE		2024
#define IMG_MENU_BUTTON_NAME				2025
#define IMG_MENU_BUTTON_NAME_ACTIVE			2026
#define IMG_MENU_BUTTON_LEVELS				2027
#define IMG_MENU_BUTTON_LEVELS_ACTIVE			2028
#define IMG_MENU_BUTTON_SCORES				2029
#define IMG_MENU_BUTTON_SCORES_ACTIVE			2030
#define IMG_MENU_BUTTON_EDITOR				2031
#define IMG_MENU_BUTTON_EDITOR_ACTIVE			2032
#define IMG_MENU_BUTTON_INFO				2033
#define IMG_MENU_BUTTON_INFO_ACTIVE			2034
#define IMG_MENU_BUTTON_GAME				2035
#define IMG_MENU_BUTTON_GAME_ACTIVE			2036
#define IMG_MENU_BUTTON_SETUP				2037
#define IMG_MENU_BUTTON_SETUP_ACTIVE			2038
#define IMG_MENU_BUTTON_QUIT				2039
#define IMG_MENU_BUTTON_QUIT_ACTIVE			2040
#define IMG_MENU_BUTTON_FIRST_LEVEL			2041
#define IMG_MENU_BUTTON_FIRST_LEVEL_ACTIVE		2042
#define IMG_MENU_BUTTON_LAST_LEVEL			2043
#define IMG_MENU_BUTTON_LAST_LEVEL_ACTIVE		2044
#define IMG_MENU_BUTTON_LEVEL_NUMBER			2045
#define IMG_MENU_BUTTON_LEVEL_NUMBER_ACTIVE		2046
#define IMG_MENU_BUTTON_INSERT_SOLUTION			2047
#define IMG_MENU_BUTTON_INSERT_SOLUTION_ACTIVE		2048
#define IMG_MENU_BUTTON_PLAY_SOLUTION			2049
#define IMG_MENU_BUTTON_PLAY_SOLUTION_ACTIVE		2050
#define IMG_MENU_BUTTON_SWITCH_ECS_AGA			2051
#define IMG_MENU_BUTTON_SWITCH_ECS_AGA_ACTIVE		2052
#define IMG_MENU_BUTTON_TOUCH_BACK			2053
#define IMG_MENU_BUTTON_TOUCH_NEXT			2054
#define IMG_MENU_BUTTON_TOUCH_BACK2			2055
#define IMG_MENU_BUTTON_TOUCH_NEXT2			2056
#define IMG_MENU_SCROLLBAR				2057
#define IMG_MENU_SCROLLBAR_ACTIVE			2058
#define IMG_GFX_GAME_PANEL_TIME_ANIM			2059
#define IMG_GFX_GAME_PANEL_TIME_ANIM_ACTIVE		2060
#define IMG_GFX_GAME_PANEL_HEALTH_ANIM			2061
#define IMG_GFX_GAME_PANEL_HEALTH_ANIM_ACTIVE		2062
#define IMG_GFX_GAME_BUTTON_STOP			2063
#define IMG_GFX_GAME_BUTTON_PAUSE			2064
#define IMG_GFX_GAME_BUTTON_PLAY			2065
#define IMG_GFX_GAME_BUTTON_UNDO			2066
#define IMG_GFX_GAME_BUTTON_REDO			2067
#define IMG_GFX_GAME_BUTTON_SAVE			2068
#define IMG_GFX_GAME_BUTTON_PAUSE2			2069
#define IMG_GFX_GAME_BUTTON_LOAD			2070
#define IMG_GFX_GAME_BUTTON_SOUND_MUSIC			2071
#define IMG_GFX_GAME_BUTTON_SOUND_LOOPS			2072
#define IMG_GFX_GAME_BUTTON_SOUND_SIMPLE		2073
#define IMG_GFX_GAME_BUTTON_PANEL_STOP			2074
#define IMG_GFX_GAME_BUTTON_PANEL_PAUSE			2075
#define IMG_GFX_GAME_BUTTON_PANEL_PLAY			2076
#define IMG_GFX_GAME_BUTTON_PANEL_SOUND_MUSIC		2077
#define IMG_GFX_GAME_BUTTON_PANEL_SOUND_LOOPS		2078
#define IMG_GFX_GAME_BUTTON_PANEL_SOUND_SIMPLE		2079
#define IMG_GFX_GAME_BUTTON_TOUCH_STOP			2080
#define IMG_GFX_GAME_BUTTON_TOUCH_PAUSE			2081
#define IMG_GFX_TAPE_BUTTON_EJECT			2082
#define IMG_GFX_TAPE_BUTTON_EXTRA			2083
#define IMG_GFX_TAPE_BUTTON_STOP			2084
#define IMG_GFX_TAPE_BUTTON_PAUSE			2085
#define IMG_GFX_TAPE_BUTTON_RECORD			2086
#define IMG_GFX_TAPE_BUTTON_PLAY			2087
#define IMG_GFX_TAPE_BUTTON_INSERT_SOLUTION		2088
#define IMG_GFX_TAPE_BUTTON_PLAY_SOLUTION		2089
#define IMG_GFX_TAPE_SYMBOL_EJECT			2090
#define IMG_GFX_TAPE_SYMBOL_STOP			2091
#define IMG_GFX_TAPE_SYMBOL_PAUSE			2092
#define IMG_GFX_TAPE_SYMBOL_RECORD			2093
#define IMG_GFX_TAPE_SYMBOL_PLAY			2094
#define IMG_GFX_TAPE_SYMBOL_FAST_FORWARD		2095
#define IMG_GFX_TAPE_SYMBOL_WARP_FORWARD		2096
#define IMG_GFX_TAPE_SYMBOL_WARP_FORWARD_BLIND		2097
#define IMG_GFX_TAPE_SYMBOL_PAUSE_BEFORE_END		2098
#define IMG_GFX_TAPE_SYMBOL_SINGLE_STEP			2099
#define IMG_GFX_TAPE_LABEL_EJECT			2100
#define IMG_GFX_TAPE_LABEL_STOP				2101
#define IMG_GFX_TAPE_LABEL_PAUSE			2102
#define IMG_GFX_TAPE_LABEL_RECORD			2103
#define IMG_GFX_TAPE_LABEL_PLAY				2104
#define IMG_GFX_TAPE_LABEL_FAST_FORWARD			2105
#define IMG_GFX_TAPE_LABEL_WARP_FORWARD			2106
#define IMG_GFX_TAPE_LABEL_WARP_FORWARD_BLIND		2107
#define IMG_GFX_TAPE_LABEL_PAUSE_BEFORE_END		2108
#define IMG_GFX_TAPE_LABEL_SINGLE_STEP			2109
#define IMG_GFX_TAPE_LABEL_DATE				2110
#define IMG_GFX_TAPE_LABEL_TIME				2111
#define IMG_GFX_REQUEST_BUTTON_YES			2112
#define IMG_GFX_REQUEST_BUTTON_NO			2113
#define IMG_GFX_REQUEST_BUTTON_CONFIRM			2114
#define IMG_GFX_REQUEST_BUTTON_PLAYER_1			2115
#define IMG_GFX_REQUEST_BUTTON_PLAYER_2			2116
#define IMG_GFX_REQUEST_BUTTON_PLAYER_3			2117
#define IMG_GFX_REQUEST_BUTTON_PLAYER_4			2118
#define IMG_GFX_REQUEST_BUTTON_TOUCH_YES		2119
#define IMG_GFX_REQUEST_BUTTON_TOUCH_NO			2120
#define IMG_GFX_REQUEST_BUTTON_TOUCH_CONFIRM		2121
#define IMG_FONT_INITIAL_1				2122
#define IMG_FONT_INITIAL_2				2123
#define IMG_FONT_INITIAL_3				2124
#define IMG_FONT_INITIAL_4				2125
#define IMG_FONT_TITLE_1				2126
#define IMG_FONT_TITLE_2				2127
#define IMG_FONT_TITLE_2_SETUP				2128
#define IMG_FONT_MENU_1					2129
#define IMG_FONT_MENU_1_ACTIVE				2130
#define IMG_FONT_MENU_2					2131
#define IMG_FONT_MENU_2_ACTIVE				2132
#define IMG_FONT_TEXT_1					2133
#define IMG_FONT_TEXT_1_MAIN				2134
#define IMG_FONT_TEXT_1_LEVELS				2135
#define IMG_FONT_TEXT_1_LEVELNR				2136
#define IMG_FONT_TEXT_1_SETUP				2137
#define IMG_FONT_TEXT_1_NAMES				2138
#define IMG_FONT_TEXT_1_PREVIEW				2139
#define IMG_FONT_TEXT_1_SCORES				2140
#define IMG_FONT_TEXT_1_ACTIVE_SCORES			2141
#define IMG_FONT_TEXT_1_PANEL				2142
#define IMG_FONT_TEXT_1_DOOR				2143
#define IMG_FONT_TEXT_2					2144
#define IMG_FONT_TEXT_2_MAIN				2145
#define IMG_FONT_TEXT_2_LEVELS				2146
#define IMG_FONT_TEXT_2_LEVELNR				2147
#define IMG_FONT_TEXT_2_SETUP				2148
#define IMG_FONT_TEXT_2_NAMES				2149
#define IMG_FONT_TEXT_2_PREVIEW				2150
#define IMG_FONT_TEXT_2_SCORES				2151
#define IMG_FONT_TEXT_2_ACTIVE_SCORES			2152
#define IMG_FONT_TEXT_3					2153
#define IMG_FONT_TEXT_3_LEVELS				2154
#define IMG_FONT_TEXT_3_LEVELNR				2155
#define IMG_FONT_TEXT_3_SETUP				2156
#define IMG_FONT_TEXT_3_NAMES				2157
#define IMG_FONT_TEXT_3_PREVIEW				2158
#define IMG_FONT_TEXT_3_SCORES				2159
#define IMG_FONT_TEXT_3_ACTIVE_SCORES			2160
#define IMG_FONT_TEXT_4					2161
#define IMG_FONT_TEXT_4_MAIN				2162
#define IMG_FONT_TEXT_4_LEVELS				2163
#define IMG_FONT_TEXT_4_LEVELNR				2164
#define IMG_FONT_TEXT_4_SETUP				2165
#define IMG_FONT_TEXT_4_NAMES				2166
#define IMG_FONT_TEXT_4_SCORES				2167
#define IMG_FONT_TEXT_4_ACTIVE_SCORES			2168
#define IMG_FONT_ENVELOPE_1				2169
#define IMG_FONT_ENVELOPE_2				2170
#define IMG_FONT_ENVELOPE_3				2171
#define IMG_FONT_ENVELOPE_4				2172
#define IMG_FONT_REQUEST				2173
#define IMG_FONT_INPUT_1				2174
#define IMG_FONT_INPUT_1_MAIN				2175
#define IMG_FONT_INPUT_1_NAMES				2176
#define IMG_FONT_INPUT_1_ACTIVE				2177
#define IMG_FONT_INPUT_1_ACTIVE_MAIN			2178
#define IMG_FONT_INPUT_1_ACTIVE_NAMES			2179
#define IMG_FONT_INPUT_1_ACTIVE_SETUP			2180
#define IMG_FONT_INPUT_2				2181
#define IMG_FONT_INPUT_2_ACTIVE				2182
#define IMG_FONT_OPTION_OFF				2183
#define IMG_FONT_OPTION_OFF_NARROW			2184
#define IMG_FONT_OPTION_ON				2185
#define IMG_FONT_OPTION_ON_NARROW			2186
#define IMG_FONT_VALUE_1				2187
#define IMG_FONT_VALUE_2				2188
#define IMG_FONT_VALUE_OLD				2189
#define IMG_FONT_VALUE_OLD_NARROW			2190
#define IMG_FONT_VALUE_NARROW				2191
#define IMG_FONT_LEVEL_NUMBER				2192
#define IMG_FONT_LEVEL_NUMBER_ACTIVE			2193
#define IMG_FONT_TAPE_RECORDER				2194
#define IMG_FONT_GAME_INFO				2195
#define IMG_FONT_INFO_ELEMENTS				2196
#define IMG_FONT_INFO_LEVELSET				2197
#define IMG_FONT_MAIN_NETWORK_PLAYERS			2198
#define IMG_EDITOR_ELEMENT_BORDER			2199
#define IMG_EDITOR_ELEMENT_BORDER_INPUT			2200
#define IMG_EDITOR_COUNTER_DOWN				2201
#define IMG_EDITOR_COUNTER_UP				2202
#define IMG_EDITOR_COUNTER_INPUT			2203
#define IMG_EDITOR_SELECTBOX_INPUT			2204
#define IMG_EDITOR_SELECTBOX_BUTTON			2205
#define IMG_EDITOR_CHECKBOX				2206
#define IMG_EDITOR_RADIOBUTTON				2207
#define IMG_EDITOR_STICKYBUTTON				2208
#define IMG_EDITOR_TABBUTTON				2209
#define IMG_EDITOR_TEXTBUTTON				2210
#define IMG_EDITOR_INPUT_TEXT				2211
#define IMG_EDITOR_INPUT_TEXTAREA			2212
#define IMG_EDITOR_CASCADE_LIST				2213
#define IMG_EDITOR_CASCADE_LIST_ACTIVE			2214
#define IMG_EDITOR_PALETTE_BUTTON			2215
#define IMG_EDITOR_PALETTE_SCROLL_UP			2216
#define IMG_EDITOR_PALETTE_SCROLL_DOWN			2217
#define IMG_EDITOR_PALETTE_SCROLLBAR			2218
#define IMG_EDITOR_PLAYFIELD_SCROLL_UP			2219
#define IMG_EDITOR_PLAYFIELD_SCROLL_DOWN		2220
#define IMG_EDITOR_PLAYFIELD_SCROLL_LEFT		2221
#define IMG_EDITOR_PLAYFIELD_SCROLL_RIGHT		2222
#define IMG_EDITOR_PLAYFIELD_SCROLLBAR			2223
#define IMG_GFX_EDITOR_BUTTON_PREV_LEVEL		2224
#define IMG_GFX_EDITOR_BUTTON_NEXT_LEVEL		2225
#define IMG_GFX_EDITOR_BUTTON_PROPERTIES		2226
#define IMG_GFX_EDITOR_BUTTON_ELEMENT_LEFT		2227
#define IMG_GFX_EDITOR_BUTTON_ELEMENT_MIDDLE		2228
#define IMG_GFX_EDITOR_BUTTON_ELEMENT_RIGHT		2229
#define IMG_GFX_EDITOR_BUTTON_PALETTE			2230
#define IMG_EDITOR_NO_TOOLBOX_BUTTON			2231
#define IMG_GFX_EDITOR_BUTTON_DRAW_SINGLE		2232
#define IMG_GFX_EDITOR_BUTTON_DRAW_CONNECTED		2233
#define IMG_GFX_EDITOR_BUTTON_DRAW_LINE			2234
#define IMG_GFX_EDITOR_BUTTON_DRAW_ARC			2235
#define IMG_GFX_EDITOR_BUTTON_DRAW_RECTANGLE		2236
#define IMG_GFX_EDITOR_BUTTON_DRAW_FILLED_BOX		2237
#define IMG_GFX_EDITOR_BUTTON_ROTATE_UP			2238
#define IMG_GFX_EDITOR_BUTTON_DRAW_TEXT			2239
#define IMG_GFX_EDITOR_BUTTON_FLOOD_FILL		2240
#define IMG_GFX_EDITOR_BUTTON_ROTATE_LEFT		2241
#define IMG_GFX_EDITOR_BUTTON_ZOOM_LEVEL		2242
#define IMG_GFX_EDITOR_BUTTON_ROTATE_RIGHT		2243
#define IMG_GFX_EDITOR_BUTTON_DRAW_RANDOM		2244
#define IMG_GFX_EDITOR_BUTTON_GRAB_BRUSH		2245
#define IMG_GFX_EDITOR_BUTTON_ROTATE_DOWN		2246
#define IMG_GFX_EDITOR_BUTTON_PICK_ELEMENT		2247
#define IMG_GFX_EDITOR_BUTTON_CE_COPY_FROM		2248
#define IMG_GFX_EDITOR_BUTTON_CE_COPY_TO		2249
#define IMG_GFX_EDITOR_BUTTON_CE_SWAP			2250
#define IMG_GFX_EDITOR_BUTTON_CE_COPY			2251
#define IMG_GFX_EDITOR_BUTTON_CE_PASTE			2252
#define IMG_GFX_EDITOR_BUTTON_CP_COPY			2253
#define IMG_GFX_EDITOR_BUTTON_CP_PASTE			2254
#define IMG_GFX_EDITOR_BUTTON_UNDO			2255
#define IMG_GFX_EDITOR_BUTTON_CONF			2256
#define IMG_GFX_EDITOR_BUTTON_SAVE			2257
#define IMG_GFX_EDITOR_BUTTON_CLEAR			2258
#define IMG_GFX_EDITOR_BUTTON_TEST			2259
#define IMG_GFX_EDITOR_BUTTON_EXIT			2260
#define IMG_GFX_EDITOR_INPUT_LEVEL_NUMBER		2261
#define IMG_SETUP_INPUT_TEXT				2262
#define IMG_GLOBAL_BORDER				2263
#define IMG_GLOBAL_BORDER_MAIN				2264
#define IMG_GLOBAL_BORDER_SCORES			2265
#define IMG_GLOBAL_BORDER_EDITOR			2266
#define IMG_GLOBAL_BORDER_PLAYING			2267
#define IMG_GLOBAL_DOOR					2268
#define IMG_GLOBAL_BUSY					2269
#define IMG_GLOBAL_TILE_CURSOR				2270
#define IMG_BACKGROUND					2271
#define IMG_BACKGROUND_TITLE_INITIAL			2272
#define IMG_BACKGROUND_TITLE				2273
#define IMG_BACKGROUND_MAIN				2274
#define IMG_BACKGROUND_NAMES				2275
#define IMG_BACKGROUND_LEVELS				2276
#define IMG_BACKGROUND_LEVELNR				2277
#define IMG_BACKGROUND_SCORES				2278
#define IMG_BACKGROUND_EDITOR				2279
#define IMG_BACKGROUND_INFO				2280
#define IMG_BACKGROUND_INFO_ELEMENTS			2281
#define IMG_BACKGROUND_INFO_MUSIC			2282
#define IMG_BACKGROUND_INFO_CREDITS			2283
#define IMG_BACKGROUND_INFO_PROGRAM			2284
#define IMG_BACKGROUND_INFO_VERSION			2285
#define IMG_BACKGROUND_INFO_LEVELSET			2286
#define IMG_BACKGROUND_SETUP				2287
#define IMG_BACKGROUND_PLAYING				2288
#define IMG_BACKGROUND_DOOR				2289
#define IMG_BACKGROUND_TAPE				2290
#define IMG_BACKGROUND_PANEL				2291
#define IMG_BACKGROUND_PALETTE				2292
#define IMG_BACKGROUND_TOOLBOX				2293
#define IMG_BACKGROUND_TITLESCREEN_INITIAL_1		2294
#define IMG_BACKGROUND_TITLESCREEN_INITIAL_2		2295
#define IMG_BACKGROUND_TITLESCREEN_INITIAL_3		2296
#define IMG_BACKGROUND_TITLESCREEN_INITIAL_4		2297
#define IMG_BACKGROUND_TITLESCREEN_INITIAL_5		2298
#define IMG_BACKGROUND_TITLESCREEN_1			2299
#define IMG_BACKGROUND_TITLESCREEN_2			2300
#define IMG_BACKGROUND_TITLESCREEN_3			2301
#define IMG_BACKGROUND_TITLESCREEN_4			2302
#define IMG_BACKGROUND_TITLESCREEN_5			2303
#define IMG_BACKGROUND_TITLEMESSAGE_INITIAL_1		2304
#define IMG_BACKGROUND_TITLEMESSAGE_INITIAL_2		2305
#define IMG_BACKGROUND_TITLEMESSAGE_INITIAL_3		2306
#define IMG_BACKGROUND_TITLEMESSAGE_INITIAL_4		2307
#define IMG_BACKGROUND_TITLEMESSAGE_INITIAL_5		2308
#define IMG_BACKGROUND_TITLEMESSAGE_1			2309
#define IMG_BACKGROUND_TITLEMESSAGE_2			2310
#define IMG_BACKGROUND_TITLEMESSAGE_3			2311
#define IMG_BACKGROUND_TITLEMESSAGE_4			2312
#define IMG_BACKGROUND_TITLEMESSAGE_5			2313
#define IMG_BACKGROUND_ENVELOPE_1			2314
#define IMG_BACKGROUND_ENVELOPE_2			2315
#define IMG_BACKGROUND_ENVELOPE_3			2316
#define IMG_BACKGROUND_ENVELOPE_4			2317
#define IMG_BACKGROUND_REQUEST				2318
#define IMG_TITLESCREEN_INITIAL_1			2319
#define IMG_TITLESCREEN_INITIAL_2			2320
#define IMG_TITLESCREEN_INITIAL_3			2321
#define IMG_TITLESCREEN_INITIAL_4			2322
#define IMG_TITLESCREEN_INITIAL_5			2323
#define IMG_TITLESCREEN_1				2324
#define IMG_TITLESCREEN_2				2325
#define IMG_TITLESCREEN_3				2326
#define IMG_TITLESCREEN_4				2327
#define IMG_TITLESCREEN_5				2328
#define IMG_GFX_DOOR_1_PART_1				2329
#define IMG_GFX_DOOR_1_PART_2				2330
#define IMG_GFX_DOOR_1_PART_3				2331
#define IMG_GFX_DOOR_1_PART_4				2332
#define IMG_GFX_DOOR_1_PART_5				2333
#define IMG_GFX_DOOR_1_PART_6				2334
#define IMG_GFX_DOOR_1_PART_7				2335
#define IMG_GFX_DOOR_1_PART_8				2336
#define IMG_GFX_DOOR_2_PART_1				2337
#define IMG_GFX_DOOR_2_PART_2				2338
#define IMG_GFX_DOOR_2_PART_3				2339
#define IMG_GFX_DOOR_2_PART_4				2340
#define IMG_GFX_DOOR_2_PART_5				2341
#define IMG_GFX_DOOR_2_PART_6				2342
#define IMG_GFX_DOOR_2_PART_7				2343
#define IMG_GFX_DOOR_2_PART_8				2344
#define IMG_DOOR_2_TOP_BORDER_CORRECTION		2345
#define IMG_LAST_IMAGE_ENTRY_BUG			2346

#define NUM_IMAGE_FILES					2347

#endif	// CONF_GFX_H
