// ============================================================================
// Rocks'n'Diamonds - McDuffin Strikes Back!
// ----------------------------------------------------------------------------
// (c) 1995-2014 by Artsoft Entertainment
//                  Holger Schemel
//                  info@artsoft.org
//                  https://www.artsoft.org/
// ----------------------------------------------------------------------------
// conf_esg.c
// ============================================================================

// ------- this file was automatically generated -- do not edit by hand -------

#ifndef CONF_ESG_C
#define CONF_ESG_C

// values for element/graphics mapping configuration (special)

static struct
{
  int element;
  int special;

  int graphic;
}
element_to_special_graphic[] =
{
  {
    EL_BD_WALL,					GFX_SPECIAL_ARG_EDITOR,
    IMG_BD_WALL_EDITOR
  },
  {
    EL_BD_ROCK,					GFX_SPECIAL_ARG_EDITOR,
    IMG_BD_ROCK_EDITOR
  },
  {
    EL_BD_AMOEBA,				GFX_SPECIAL_ARG_EDITOR,
    IMG_BD_AMOEBA_EDITOR
  },
  {
    EL_BD_BUTTERFLY_RIGHT,			GFX_SPECIAL_ARG_EDITOR,
    IMG_BD_BUTTERFLY_RIGHT_EDITOR
  },
  {
    EL_BD_BUTTERFLY_UP,				GFX_SPECIAL_ARG_EDITOR,
    IMG_BD_BUTTERFLY_UP_EDITOR
  },
  {
    EL_BD_BUTTERFLY_LEFT,			GFX_SPECIAL_ARG_EDITOR,
    IMG_BD_BUTTERFLY_LEFT_EDITOR
  },
  {
    EL_BD_BUTTERFLY_DOWN,			GFX_SPECIAL_ARG_EDITOR,
    IMG_BD_BUTTERFLY_DOWN_EDITOR
  },
  {
    EL_BD_FIREFLY_RIGHT,			GFX_SPECIAL_ARG_EDITOR,
    IMG_BD_FIREFLY_RIGHT_EDITOR
  },
  {
    EL_BD_FIREFLY_UP,				GFX_SPECIAL_ARG_EDITOR,
    IMG_BD_FIREFLY_UP_EDITOR
  },
  {
    EL_BD_FIREFLY_LEFT,				GFX_SPECIAL_ARG_EDITOR,
    IMG_BD_FIREFLY_LEFT_EDITOR
  },
  {
    EL_BD_FIREFLY_DOWN,				GFX_SPECIAL_ARG_EDITOR,
    IMG_BD_FIREFLY_DOWN_EDITOR
  },
  {
    EL_SP_INFOTRON,				GFX_SPECIAL_ARG_EDITOR,
    IMG_SP_INFOTRON_EDITOR
  },
  {
    EL_SP_GRAVITY_PORT_RIGHT,			GFX_SPECIAL_ARG_EDITOR,
    IMG_SP_GRAVITY_PORT_RIGHT_EDITOR
  },
  {
    EL_SP_GRAVITY_PORT_DOWN,			GFX_SPECIAL_ARG_EDITOR,
    IMG_SP_GRAVITY_PORT_DOWN_EDITOR
  },
  {
    EL_SP_GRAVITY_PORT_LEFT,			GFX_SPECIAL_ARG_EDITOR,
    IMG_SP_GRAVITY_PORT_LEFT_EDITOR
  },
  {
    EL_SP_GRAVITY_PORT_UP,			GFX_SPECIAL_ARG_EDITOR,
    IMG_SP_GRAVITY_PORT_UP_EDITOR
  },
  {
    EL_SP_GRAVITY_ON_PORT_RIGHT,		GFX_SPECIAL_ARG_EDITOR,
    IMG_SP_GRAVITY_ON_PORT_RIGHT_EDITOR
  },
  {
    EL_SP_GRAVITY_ON_PORT_DOWN,			GFX_SPECIAL_ARG_EDITOR,
    IMG_SP_GRAVITY_ON_PORT_DOWN_EDITOR
  },
  {
    EL_SP_GRAVITY_ON_PORT_LEFT,			GFX_SPECIAL_ARG_EDITOR,
    IMG_SP_GRAVITY_ON_PORT_LEFT_EDITOR
  },
  {
    EL_SP_GRAVITY_ON_PORT_UP,			GFX_SPECIAL_ARG_EDITOR,
    IMG_SP_GRAVITY_ON_PORT_UP_EDITOR
  },
  {
    EL_SP_GRAVITY_OFF_PORT_RIGHT,		GFX_SPECIAL_ARG_EDITOR,
    IMG_SP_GRAVITY_OFF_PORT_RIGHT_EDITOR
  },
  {
    EL_SP_GRAVITY_OFF_PORT_DOWN,		GFX_SPECIAL_ARG_EDITOR,
    IMG_SP_GRAVITY_OFF_PORT_DOWN_EDITOR
  },
  {
    EL_SP_GRAVITY_OFF_PORT_LEFT,		GFX_SPECIAL_ARG_EDITOR,
    IMG_SP_GRAVITY_OFF_PORT_LEFT_EDITOR
  },
  {
    EL_SP_GRAVITY_OFF_PORT_UP,			GFX_SPECIAL_ARG_EDITOR,
    IMG_SP_GRAVITY_OFF_PORT_UP_EDITOR
  },
  {
    EL_SP_ELECTRON,				GFX_SPECIAL_ARG_EDITOR,
    IMG_SP_ELECTRON_EDITOR
  },
  {
    EL_SP_TERMINAL,				GFX_SPECIAL_ARG_EDITOR,
    IMG_SP_TERMINAL_EDITOR
  },
  {
    EL_SP_BUGGY_BASE,				GFX_SPECIAL_ARG_EDITOR,
    IMG_SP_BUGGY_BASE_EDITOR
  },
  {
    EL_EM_DYNAMITE_ACTIVE,			GFX_SPECIAL_ARG_EDITOR,
    IMG_EM_DYNAMITE_ACTIVE_EDITOR
  },
  {
    EL_YAMYAM_LEFT,				GFX_SPECIAL_ARG_EDITOR,
    IMG_YAMYAM_LEFT_EDITOR
  },
  {
    EL_YAMYAM_RIGHT,				GFX_SPECIAL_ARG_EDITOR,
    IMG_YAMYAM_RIGHT_EDITOR
  },
  {
    EL_YAMYAM_UP,				GFX_SPECIAL_ARG_EDITOR,
    IMG_YAMYAM_UP_EDITOR
  },
  {
    EL_YAMYAM_DOWN,				GFX_SPECIAL_ARG_EDITOR,
    IMG_YAMYAM_DOWN_EDITOR
  },
  {
    EL_QUICKSAND_FULL,				GFX_SPECIAL_ARG_EDITOR,
    IMG_QUICKSAND_FULL_EDITOR
  },
  {
    EL_QUICKSAND_FAST_FULL,			GFX_SPECIAL_ARG_EDITOR,
    IMG_QUICKSAND_FAST_FULL_EDITOR
  },
  {
    EL_AMOEBA_WET,				GFX_SPECIAL_ARG_EDITOR,
    IMG_AMOEBA_WET_EDITOR
  },
  {
    EL_AMOEBA_FULL,				GFX_SPECIAL_ARG_EDITOR,
    IMG_AMOEBA_FULL_EDITOR
  },
  {
    EL_AMOEBA_DEAD,				GFX_SPECIAL_ARG_EDITOR,
    IMG_AMOEBA_DEAD_EDITOR
  },
  {
    EL_EM_GATE_1_GRAY,				GFX_SPECIAL_ARG_EDITOR,
    IMG_EM_GATE_1_GRAY_EDITOR
  },
  {
    EL_EM_GATE_2_GRAY,				GFX_SPECIAL_ARG_EDITOR,
    IMG_EM_GATE_2_GRAY_EDITOR
  },
  {
    EL_EM_GATE_3_GRAY,				GFX_SPECIAL_ARG_EDITOR,
    IMG_EM_GATE_3_GRAY_EDITOR
  },
  {
    EL_EM_GATE_4_GRAY,				GFX_SPECIAL_ARG_EDITOR,
    IMG_EM_GATE_4_GRAY_EDITOR
  },
  {
    EL_DC_GATE_WHITE_GRAY,			GFX_SPECIAL_ARG_EDITOR,
    IMG_DC_GATE_WHITE_GRAY_EDITOR
  },
  {
    EL_INVISIBLE_STEELWALL,			GFX_SPECIAL_ARG_EDITOR,
    IMG_INVISIBLE_STEELWALL_EDITOR
  },
  {
    EL_INVISIBLE_WALL,				GFX_SPECIAL_ARG_EDITOR,
    IMG_INVISIBLE_WALL_EDITOR
  },
  {
    EL_INVISIBLE_SAND,				GFX_SPECIAL_ARG_EDITOR,
    IMG_INVISIBLE_SAND_EDITOR
  },
  {
    EL_GATE_1_GRAY,				GFX_SPECIAL_ARG_EDITOR,
    IMG_GATE_1_GRAY_EDITOR
  },
  {
    EL_GATE_2_GRAY,				GFX_SPECIAL_ARG_EDITOR,
    IMG_GATE_2_GRAY_EDITOR
  },
  {
    EL_GATE_3_GRAY,				GFX_SPECIAL_ARG_EDITOR,
    IMG_GATE_3_GRAY_EDITOR
  },
  {
    EL_GATE_4_GRAY,				GFX_SPECIAL_ARG_EDITOR,
    IMG_GATE_4_GRAY_EDITOR
  },
  {
    EL_EXPANDABLE_WALL_HORIZONTAL,		GFX_SPECIAL_ARG_EDITOR,
    IMG_EXPANDABLE_WALL_HORIZONTAL_EDITOR
  },
  {
    EL_EXPANDABLE_WALL_VERTICAL,		GFX_SPECIAL_ARG_EDITOR,
    IMG_EXPANDABLE_WALL_VERTICAL_EDITOR
  },
  {
    EL_EXPANDABLE_WALL_ANY,			GFX_SPECIAL_ARG_EDITOR,
    IMG_EXPANDABLE_WALL_ANY_EDITOR
  },
  {
    EL_EXPANDABLE_STEELWALL_HORIZONTAL,		GFX_SPECIAL_ARG_EDITOR,
    IMG_EXPANDABLE_STEELWALL_HORIZONTAL_EDITOR
  },
  {
    EL_EXPANDABLE_STEELWALL_VERTICAL,		GFX_SPECIAL_ARG_EDITOR,
    IMG_EXPANDABLE_STEELWALL_VERTICAL_EDITOR
  },
  {
    EL_EXPANDABLE_STEELWALL_ANY,		GFX_SPECIAL_ARG_EDITOR,
    IMG_EXPANDABLE_STEELWALL_ANY_EDITOR
  },
  {
    EL_BD_EXPANDABLE_WALL,			GFX_SPECIAL_ARG_EDITOR,
    IMG_BD_EXPANDABLE_WALL_EDITOR
  },
  {
    EL_STEELWALL_TOPLEFT,			GFX_SPECIAL_ARG_EDITOR,
    IMG_STEELWALL_TOPLEFT_EDITOR
  },
  {
    EL_STEELWALL_TOPRIGHT,			GFX_SPECIAL_ARG_EDITOR,
    IMG_STEELWALL_TOPRIGHT_EDITOR
  },
  {
    EL_STEELWALL_BOTTOMLEFT,			GFX_SPECIAL_ARG_EDITOR,
    IMG_STEELWALL_BOTTOMLEFT_EDITOR
  },
  {
    EL_STEELWALL_BOTTOMRIGHT,			GFX_SPECIAL_ARG_EDITOR,
    IMG_STEELWALL_BOTTOMRIGHT_EDITOR
  },
  {
    EL_STEELWALL_HORIZONTAL,			GFX_SPECIAL_ARG_EDITOR,
    IMG_STEELWALL_HORIZONTAL_EDITOR
  },
  {
    EL_STEELWALL_VERTICAL,			GFX_SPECIAL_ARG_EDITOR,
    IMG_STEELWALL_VERTICAL_EDITOR
  },
  {
    EL_INVISIBLE_STEELWALL_TOPLEFT,		GFX_SPECIAL_ARG_EDITOR,
    IMG_INVISIBLE_STEELWALL_TOPLEFT_EDITOR
  },
  {
    EL_INVISIBLE_STEELWALL_TOPRIGHT,		GFX_SPECIAL_ARG_EDITOR,
    IMG_INVISIBLE_STEELWALL_TOPRIGHT_EDITOR
  },
  {
    EL_INVISIBLE_STEELWALL_BOTTOMLEFT,		GFX_SPECIAL_ARG_EDITOR,
    IMG_INVISIBLE_STEELWALL_BOTTOMLEFT_EDITOR
  },
  {
    EL_INVISIBLE_STEELWALL_BOTTOMRIGHT,		GFX_SPECIAL_ARG_EDITOR,
    IMG_INVISIBLE_STEELWALL_BOTTOMRIGHT_EDITOR
  },
  {
    EL_INVISIBLE_STEELWALL_HORIZONTAL,		GFX_SPECIAL_ARG_EDITOR,
    IMG_INVISIBLE_STEELWALL_HORIZONTAL_EDITOR
  },
  {
    EL_INVISIBLE_STEELWALL_VERTICAL,		GFX_SPECIAL_ARG_EDITOR,
    IMG_INVISIBLE_STEELWALL_VERTICAL_EDITOR
  },
  {
    EL_EMC_GATE_5_GRAY,				GFX_SPECIAL_ARG_EDITOR,
    IMG_EMC_GATE_5_GRAY_EDITOR
  },
  {
    EL_EMC_GATE_6_GRAY,				GFX_SPECIAL_ARG_EDITOR,
    IMG_EMC_GATE_6_GRAY_EDITOR
  },
  {
    EL_EMC_GATE_7_GRAY,				GFX_SPECIAL_ARG_EDITOR,
    IMG_EMC_GATE_7_GRAY_EDITOR
  },
  {
    EL_EMC_GATE_8_GRAY,				GFX_SPECIAL_ARG_EDITOR,
    IMG_EMC_GATE_8_GRAY_EDITOR
  },
  {
    EL_EMC_FAKE_GRASS,				GFX_SPECIAL_ARG_EDITOR,
    IMG_EMC_FAKE_GRASS_EDITOR
  },
  {
    EL_EMC_DRIPPER,				GFX_SPECIAL_ARG_EDITOR,
    IMG_EMC_DRIPPER_EDITOR
  },
  {
    EL_DF_MIRROR_ROTATING_1,			GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_MIRROR_ROTATING_1_EDITOR
  },
  {
    EL_DF_MIRROR_ROTATING_2,			GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_MIRROR_ROTATING_2_EDITOR
  },
  {
    EL_DF_MIRROR_ROTATING_3,			GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_MIRROR_ROTATING_3_EDITOR
  },
  {
    EL_DF_MIRROR_ROTATING_4,			GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_MIRROR_ROTATING_4_EDITOR
  },
  {
    EL_DF_MIRROR_ROTATING_5,			GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_MIRROR_ROTATING_5_EDITOR
  },
  {
    EL_DF_MIRROR_ROTATING_6,			GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_MIRROR_ROTATING_6_EDITOR
  },
  {
    EL_DF_MIRROR_ROTATING_7,			GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_MIRROR_ROTATING_7_EDITOR
  },
  {
    EL_DF_MIRROR_ROTATING_8,			GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_MIRROR_ROTATING_8_EDITOR
  },
  {
    EL_DF_MIRROR_ROTATING_9,			GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_MIRROR_ROTATING_9_EDITOR
  },
  {
    EL_DF_MIRROR_ROTATING_10,			GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_MIRROR_ROTATING_10_EDITOR
  },
  {
    EL_DF_MIRROR_ROTATING_11,			GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_MIRROR_ROTATING_11_EDITOR
  },
  {
    EL_DF_MIRROR_ROTATING_12,			GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_MIRROR_ROTATING_12_EDITOR
  },
  {
    EL_DF_MIRROR_ROTATING_13,			GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_MIRROR_ROTATING_13_EDITOR
  },
  {
    EL_DF_MIRROR_ROTATING_14,			GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_MIRROR_ROTATING_14_EDITOR
  },
  {
    EL_DF_MIRROR_ROTATING_15,			GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_MIRROR_ROTATING_15_EDITOR
  },
  {
    EL_DF_MIRROR_ROTATING_16,			GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_MIRROR_ROTATING_16_EDITOR
  },
  {
    EL_DF_STEEL_GRID_ROTATING_1,		GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_STEEL_GRID_ROTATING_1_EDITOR
  },
  {
    EL_DF_STEEL_GRID_ROTATING_2,		GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_STEEL_GRID_ROTATING_2_EDITOR
  },
  {
    EL_DF_STEEL_GRID_ROTATING_3,		GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_STEEL_GRID_ROTATING_3_EDITOR
  },
  {
    EL_DF_STEEL_GRID_ROTATING_4,		GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_STEEL_GRID_ROTATING_4_EDITOR
  },
  {
    EL_DF_STEEL_GRID_ROTATING_5,		GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_STEEL_GRID_ROTATING_5_EDITOR
  },
  {
    EL_DF_STEEL_GRID_ROTATING_6,		GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_STEEL_GRID_ROTATING_6_EDITOR
  },
  {
    EL_DF_STEEL_GRID_ROTATING_7,		GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_STEEL_GRID_ROTATING_7_EDITOR
  },
  {
    EL_DF_STEEL_GRID_ROTATING_8,		GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_STEEL_GRID_ROTATING_8_EDITOR
  },
  {
    EL_DF_WOODEN_GRID_ROTATING_1,		GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_WOODEN_GRID_ROTATING_1_EDITOR
  },
  {
    EL_DF_WOODEN_GRID_ROTATING_2,		GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_WOODEN_GRID_ROTATING_2_EDITOR
  },
  {
    EL_DF_WOODEN_GRID_ROTATING_3,		GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_WOODEN_GRID_ROTATING_3_EDITOR
  },
  {
    EL_DF_WOODEN_GRID_ROTATING_4,		GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_WOODEN_GRID_ROTATING_4_EDITOR
  },
  {
    EL_DF_WOODEN_GRID_ROTATING_5,		GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_WOODEN_GRID_ROTATING_5_EDITOR
  },
  {
    EL_DF_WOODEN_GRID_ROTATING_6,		GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_WOODEN_GRID_ROTATING_6_EDITOR
  },
  {
    EL_DF_WOODEN_GRID_ROTATING_7,		GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_WOODEN_GRID_ROTATING_7_EDITOR
  },
  {
    EL_DF_WOODEN_GRID_ROTATING_8,		GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_WOODEN_GRID_ROTATING_8_EDITOR
  },
  {
    EL_DF_FIBRE_OPTIC_RED_1,			GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_FIBRE_OPTIC_RED_1_EDITOR
  },
  {
    EL_DF_FIBRE_OPTIC_RED_2,			GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_FIBRE_OPTIC_RED_2_EDITOR
  },
  {
    EL_DF_FIBRE_OPTIC_YELLOW_1,			GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_FIBRE_OPTIC_YELLOW_1_EDITOR
  },
  {
    EL_DF_FIBRE_OPTIC_YELLOW_2,			GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_FIBRE_OPTIC_YELLOW_2_EDITOR
  },
  {
    EL_DF_FIBRE_OPTIC_GREEN_1,			GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_FIBRE_OPTIC_GREEN_1_EDITOR
  },
  {
    EL_DF_FIBRE_OPTIC_GREEN_2,			GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_FIBRE_OPTIC_GREEN_2_EDITOR
  },
  {
    EL_DF_FIBRE_OPTIC_BLUE_1,			GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_FIBRE_OPTIC_BLUE_1_EDITOR
  },
  {
    EL_DF_FIBRE_OPTIC_BLUE_2,			GFX_SPECIAL_ARG_EDITOR,
    IMG_DF_FIBRE_OPTIC_BLUE_2_EDITOR
  },
  {
    EL_CUSTOM_1,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_1_EDITOR
  },
  {
    EL_CUSTOM_2,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_2_EDITOR
  },
  {
    EL_CUSTOM_3,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_3_EDITOR
  },
  {
    EL_CUSTOM_4,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_4_EDITOR
  },
  {
    EL_CUSTOM_5,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_5_EDITOR
  },
  {
    EL_CUSTOM_6,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_6_EDITOR
  },
  {
    EL_CUSTOM_7,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_7_EDITOR
  },
  {
    EL_CUSTOM_8,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_8_EDITOR
  },
  {
    EL_CUSTOM_9,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_9_EDITOR
  },
  {
    EL_CUSTOM_10,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_10_EDITOR
  },
  {
    EL_CUSTOM_11,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_11_EDITOR
  },
  {
    EL_CUSTOM_12,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_12_EDITOR
  },
  {
    EL_CUSTOM_13,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_13_EDITOR
  },
  {
    EL_CUSTOM_14,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_14_EDITOR
  },
  {
    EL_CUSTOM_15,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_15_EDITOR
  },
  {
    EL_CUSTOM_16,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_16_EDITOR
  },
  {
    EL_CUSTOM_17,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_17_EDITOR
  },
  {
    EL_CUSTOM_18,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_18_EDITOR
  },
  {
    EL_CUSTOM_19,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_19_EDITOR
  },
  {
    EL_CUSTOM_20,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_20_EDITOR
  },
  {
    EL_CUSTOM_21,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_21_EDITOR
  },
  {
    EL_CUSTOM_22,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_22_EDITOR
  },
  {
    EL_CUSTOM_23,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_23_EDITOR
  },
  {
    EL_CUSTOM_24,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_24_EDITOR
  },
  {
    EL_CUSTOM_25,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_25_EDITOR
  },
  {
    EL_CUSTOM_26,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_26_EDITOR
  },
  {
    EL_CUSTOM_27,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_27_EDITOR
  },
  {
    EL_CUSTOM_28,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_28_EDITOR
  },
  {
    EL_CUSTOM_29,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_29_EDITOR
  },
  {
    EL_CUSTOM_30,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_30_EDITOR
  },
  {
    EL_CUSTOM_31,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_31_EDITOR
  },
  {
    EL_CUSTOM_32,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_32_EDITOR
  },
  {
    EL_CUSTOM_33,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_33_EDITOR
  },
  {
    EL_CUSTOM_34,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_34_EDITOR
  },
  {
    EL_CUSTOM_35,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_35_EDITOR
  },
  {
    EL_CUSTOM_36,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_36_EDITOR
  },
  {
    EL_CUSTOM_37,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_37_EDITOR
  },
  {
    EL_CUSTOM_38,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_38_EDITOR
  },
  {
    EL_CUSTOM_39,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_39_EDITOR
  },
  {
    EL_CUSTOM_40,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_40_EDITOR
  },
  {
    EL_CUSTOM_41,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_41_EDITOR
  },
  {
    EL_CUSTOM_42,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_42_EDITOR
  },
  {
    EL_CUSTOM_43,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_43_EDITOR
  },
  {
    EL_CUSTOM_44,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_44_EDITOR
  },
  {
    EL_CUSTOM_45,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_45_EDITOR
  },
  {
    EL_CUSTOM_46,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_46_EDITOR
  },
  {
    EL_CUSTOM_47,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_47_EDITOR
  },
  {
    EL_CUSTOM_48,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_48_EDITOR
  },
  {
    EL_CUSTOM_49,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_49_EDITOR
  },
  {
    EL_CUSTOM_50,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_50_EDITOR
  },
  {
    EL_CUSTOM_51,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_51_EDITOR
  },
  {
    EL_CUSTOM_52,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_52_EDITOR
  },
  {
    EL_CUSTOM_53,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_53_EDITOR
  },
  {
    EL_CUSTOM_54,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_54_EDITOR
  },
  {
    EL_CUSTOM_55,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_55_EDITOR
  },
  {
    EL_CUSTOM_56,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_56_EDITOR
  },
  {
    EL_CUSTOM_57,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_57_EDITOR
  },
  {
    EL_CUSTOM_58,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_58_EDITOR
  },
  {
    EL_CUSTOM_59,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_59_EDITOR
  },
  {
    EL_CUSTOM_60,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_60_EDITOR
  },
  {
    EL_CUSTOM_61,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_61_EDITOR
  },
  {
    EL_CUSTOM_62,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_62_EDITOR
  },
  {
    EL_CUSTOM_63,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_63_EDITOR
  },
  {
    EL_CUSTOM_64,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_64_EDITOR
  },
  {
    EL_CUSTOM_65,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_65_EDITOR
  },
  {
    EL_CUSTOM_66,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_66_EDITOR
  },
  {
    EL_CUSTOM_67,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_67_EDITOR
  },
  {
    EL_CUSTOM_68,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_68_EDITOR
  },
  {
    EL_CUSTOM_69,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_69_EDITOR
  },
  {
    EL_CUSTOM_70,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_70_EDITOR
  },
  {
    EL_CUSTOM_71,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_71_EDITOR
  },
  {
    EL_CUSTOM_72,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_72_EDITOR
  },
  {
    EL_CUSTOM_73,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_73_EDITOR
  },
  {
    EL_CUSTOM_74,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_74_EDITOR
  },
  {
    EL_CUSTOM_75,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_75_EDITOR
  },
  {
    EL_CUSTOM_76,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_76_EDITOR
  },
  {
    EL_CUSTOM_77,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_77_EDITOR
  },
  {
    EL_CUSTOM_78,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_78_EDITOR
  },
  {
    EL_CUSTOM_79,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_79_EDITOR
  },
  {
    EL_CUSTOM_80,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_80_EDITOR
  },
  {
    EL_CUSTOM_81,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_81_EDITOR
  },
  {
    EL_CUSTOM_82,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_82_EDITOR
  },
  {
    EL_CUSTOM_83,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_83_EDITOR
  },
  {
    EL_CUSTOM_84,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_84_EDITOR
  },
  {
    EL_CUSTOM_85,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_85_EDITOR
  },
  {
    EL_CUSTOM_86,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_86_EDITOR
  },
  {
    EL_CUSTOM_87,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_87_EDITOR
  },
  {
    EL_CUSTOM_88,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_88_EDITOR
  },
  {
    EL_CUSTOM_89,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_89_EDITOR
  },
  {
    EL_CUSTOM_90,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_90_EDITOR
  },
  {
    EL_CUSTOM_91,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_91_EDITOR
  },
  {
    EL_CUSTOM_92,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_92_EDITOR
  },
  {
    EL_CUSTOM_93,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_93_EDITOR
  },
  {
    EL_CUSTOM_94,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_94_EDITOR
  },
  {
    EL_CUSTOM_95,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_95_EDITOR
  },
  {
    EL_CUSTOM_96,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_96_EDITOR
  },
  {
    EL_CUSTOM_97,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_97_EDITOR
  },
  {
    EL_CUSTOM_98,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_98_EDITOR
  },
  {
    EL_CUSTOM_99,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_99_EDITOR
  },
  {
    EL_CUSTOM_100,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_100_EDITOR
  },
  {
    EL_CUSTOM_101,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_101_EDITOR
  },
  {
    EL_CUSTOM_102,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_102_EDITOR
  },
  {
    EL_CUSTOM_103,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_103_EDITOR
  },
  {
    EL_CUSTOM_104,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_104_EDITOR
  },
  {
    EL_CUSTOM_105,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_105_EDITOR
  },
  {
    EL_CUSTOM_106,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_106_EDITOR
  },
  {
    EL_CUSTOM_107,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_107_EDITOR
  },
  {
    EL_CUSTOM_108,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_108_EDITOR
  },
  {
    EL_CUSTOM_109,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_109_EDITOR
  },
  {
    EL_CUSTOM_110,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_110_EDITOR
  },
  {
    EL_CUSTOM_111,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_111_EDITOR
  },
  {
    EL_CUSTOM_112,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_112_EDITOR
  },
  {
    EL_CUSTOM_113,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_113_EDITOR
  },
  {
    EL_CUSTOM_114,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_114_EDITOR
  },
  {
    EL_CUSTOM_115,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_115_EDITOR
  },
  {
    EL_CUSTOM_116,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_116_EDITOR
  },
  {
    EL_CUSTOM_117,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_117_EDITOR
  },
  {
    EL_CUSTOM_118,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_118_EDITOR
  },
  {
    EL_CUSTOM_119,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_119_EDITOR
  },
  {
    EL_CUSTOM_120,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_120_EDITOR
  },
  {
    EL_CUSTOM_121,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_121_EDITOR
  },
  {
    EL_CUSTOM_122,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_122_EDITOR
  },
  {
    EL_CUSTOM_123,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_123_EDITOR
  },
  {
    EL_CUSTOM_124,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_124_EDITOR
  },
  {
    EL_CUSTOM_125,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_125_EDITOR
  },
  {
    EL_CUSTOM_126,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_126_EDITOR
  },
  {
    EL_CUSTOM_127,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_127_EDITOR
  },
  {
    EL_CUSTOM_128,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_128_EDITOR
  },
  {
    EL_CUSTOM_129,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_129_EDITOR
  },
  {
    EL_CUSTOM_130,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_130_EDITOR
  },
  {
    EL_CUSTOM_131,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_131_EDITOR
  },
  {
    EL_CUSTOM_132,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_132_EDITOR
  },
  {
    EL_CUSTOM_133,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_133_EDITOR
  },
  {
    EL_CUSTOM_134,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_134_EDITOR
  },
  {
    EL_CUSTOM_135,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_135_EDITOR
  },
  {
    EL_CUSTOM_136,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_136_EDITOR
  },
  {
    EL_CUSTOM_137,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_137_EDITOR
  },
  {
    EL_CUSTOM_138,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_138_EDITOR
  },
  {
    EL_CUSTOM_139,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_139_EDITOR
  },
  {
    EL_CUSTOM_140,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_140_EDITOR
  },
  {
    EL_CUSTOM_141,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_141_EDITOR
  },
  {
    EL_CUSTOM_142,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_142_EDITOR
  },
  {
    EL_CUSTOM_143,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_143_EDITOR
  },
  {
    EL_CUSTOM_144,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_144_EDITOR
  },
  {
    EL_CUSTOM_145,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_145_EDITOR
  },
  {
    EL_CUSTOM_146,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_146_EDITOR
  },
  {
    EL_CUSTOM_147,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_147_EDITOR
  },
  {
    EL_CUSTOM_148,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_148_EDITOR
  },
  {
    EL_CUSTOM_149,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_149_EDITOR
  },
  {
    EL_CUSTOM_150,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_150_EDITOR
  },
  {
    EL_CUSTOM_151,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_151_EDITOR
  },
  {
    EL_CUSTOM_152,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_152_EDITOR
  },
  {
    EL_CUSTOM_153,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_153_EDITOR
  },
  {
    EL_CUSTOM_154,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_154_EDITOR
  },
  {
    EL_CUSTOM_155,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_155_EDITOR
  },
  {
    EL_CUSTOM_156,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_156_EDITOR
  },
  {
    EL_CUSTOM_157,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_157_EDITOR
  },
  {
    EL_CUSTOM_158,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_158_EDITOR
  },
  {
    EL_CUSTOM_159,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_159_EDITOR
  },
  {
    EL_CUSTOM_160,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_160_EDITOR
  },
  {
    EL_CUSTOM_161,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_161_EDITOR
  },
  {
    EL_CUSTOM_162,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_162_EDITOR
  },
  {
    EL_CUSTOM_163,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_163_EDITOR
  },
  {
    EL_CUSTOM_164,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_164_EDITOR
  },
  {
    EL_CUSTOM_165,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_165_EDITOR
  },
  {
    EL_CUSTOM_166,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_166_EDITOR
  },
  {
    EL_CUSTOM_167,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_167_EDITOR
  },
  {
    EL_CUSTOM_168,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_168_EDITOR
  },
  {
    EL_CUSTOM_169,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_169_EDITOR
  },
  {
    EL_CUSTOM_170,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_170_EDITOR
  },
  {
    EL_CUSTOM_171,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_171_EDITOR
  },
  {
    EL_CUSTOM_172,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_172_EDITOR
  },
  {
    EL_CUSTOM_173,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_173_EDITOR
  },
  {
    EL_CUSTOM_174,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_174_EDITOR
  },
  {
    EL_CUSTOM_175,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_175_EDITOR
  },
  {
    EL_CUSTOM_176,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_176_EDITOR
  },
  {
    EL_CUSTOM_177,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_177_EDITOR
  },
  {
    EL_CUSTOM_178,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_178_EDITOR
  },
  {
    EL_CUSTOM_179,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_179_EDITOR
  },
  {
    EL_CUSTOM_180,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_180_EDITOR
  },
  {
    EL_CUSTOM_181,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_181_EDITOR
  },
  {
    EL_CUSTOM_182,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_182_EDITOR
  },
  {
    EL_CUSTOM_183,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_183_EDITOR
  },
  {
    EL_CUSTOM_184,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_184_EDITOR
  },
  {
    EL_CUSTOM_185,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_185_EDITOR
  },
  {
    EL_CUSTOM_186,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_186_EDITOR
  },
  {
    EL_CUSTOM_187,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_187_EDITOR
  },
  {
    EL_CUSTOM_188,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_188_EDITOR
  },
  {
    EL_CUSTOM_189,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_189_EDITOR
  },
  {
    EL_CUSTOM_190,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_190_EDITOR
  },
  {
    EL_CUSTOM_191,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_191_EDITOR
  },
  {
    EL_CUSTOM_192,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_192_EDITOR
  },
  {
    EL_CUSTOM_193,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_193_EDITOR
  },
  {
    EL_CUSTOM_194,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_194_EDITOR
  },
  {
    EL_CUSTOM_195,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_195_EDITOR
  },
  {
    EL_CUSTOM_196,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_196_EDITOR
  },
  {
    EL_CUSTOM_197,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_197_EDITOR
  },
  {
    EL_CUSTOM_198,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_198_EDITOR
  },
  {
    EL_CUSTOM_199,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_199_EDITOR
  },
  {
    EL_CUSTOM_200,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_200_EDITOR
  },
  {
    EL_CUSTOM_201,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_201_EDITOR
  },
  {
    EL_CUSTOM_202,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_202_EDITOR
  },
  {
    EL_CUSTOM_203,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_203_EDITOR
  },
  {
    EL_CUSTOM_204,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_204_EDITOR
  },
  {
    EL_CUSTOM_205,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_205_EDITOR
  },
  {
    EL_CUSTOM_206,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_206_EDITOR
  },
  {
    EL_CUSTOM_207,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_207_EDITOR
  },
  {
    EL_CUSTOM_208,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_208_EDITOR
  },
  {
    EL_CUSTOM_209,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_209_EDITOR
  },
  {
    EL_CUSTOM_210,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_210_EDITOR
  },
  {
    EL_CUSTOM_211,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_211_EDITOR
  },
  {
    EL_CUSTOM_212,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_212_EDITOR
  },
  {
    EL_CUSTOM_213,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_213_EDITOR
  },
  {
    EL_CUSTOM_214,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_214_EDITOR
  },
  {
    EL_CUSTOM_215,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_215_EDITOR
  },
  {
    EL_CUSTOM_216,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_216_EDITOR
  },
  {
    EL_CUSTOM_217,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_217_EDITOR
  },
  {
    EL_CUSTOM_218,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_218_EDITOR
  },
  {
    EL_CUSTOM_219,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_219_EDITOR
  },
  {
    EL_CUSTOM_220,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_220_EDITOR
  },
  {
    EL_CUSTOM_221,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_221_EDITOR
  },
  {
    EL_CUSTOM_222,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_222_EDITOR
  },
  {
    EL_CUSTOM_223,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_223_EDITOR
  },
  {
    EL_CUSTOM_224,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_224_EDITOR
  },
  {
    EL_CUSTOM_225,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_225_EDITOR
  },
  {
    EL_CUSTOM_226,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_226_EDITOR
  },
  {
    EL_CUSTOM_227,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_227_EDITOR
  },
  {
    EL_CUSTOM_228,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_228_EDITOR
  },
  {
    EL_CUSTOM_229,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_229_EDITOR
  },
  {
    EL_CUSTOM_230,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_230_EDITOR
  },
  {
    EL_CUSTOM_231,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_231_EDITOR
  },
  {
    EL_CUSTOM_232,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_232_EDITOR
  },
  {
    EL_CUSTOM_233,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_233_EDITOR
  },
  {
    EL_CUSTOM_234,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_234_EDITOR
  },
  {
    EL_CUSTOM_235,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_235_EDITOR
  },
  {
    EL_CUSTOM_236,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_236_EDITOR
  },
  {
    EL_CUSTOM_237,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_237_EDITOR
  },
  {
    EL_CUSTOM_238,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_238_EDITOR
  },
  {
    EL_CUSTOM_239,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_239_EDITOR
  },
  {
    EL_CUSTOM_240,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_240_EDITOR
  },
  {
    EL_CUSTOM_241,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_241_EDITOR
  },
  {
    EL_CUSTOM_242,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_242_EDITOR
  },
  {
    EL_CUSTOM_243,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_243_EDITOR
  },
  {
    EL_CUSTOM_244,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_244_EDITOR
  },
  {
    EL_CUSTOM_245,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_245_EDITOR
  },
  {
    EL_CUSTOM_246,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_246_EDITOR
  },
  {
    EL_CUSTOM_247,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_247_EDITOR
  },
  {
    EL_CUSTOM_248,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_248_EDITOR
  },
  {
    EL_CUSTOM_249,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_249_EDITOR
  },
  {
    EL_CUSTOM_250,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_250_EDITOR
  },
  {
    EL_CUSTOM_251,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_251_EDITOR
  },
  {
    EL_CUSTOM_252,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_252_EDITOR
  },
  {
    EL_CUSTOM_253,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_253_EDITOR
  },
  {
    EL_CUSTOM_254,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_254_EDITOR
  },
  {
    EL_CUSTOM_255,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_255_EDITOR
  },
  {
    EL_CUSTOM_256,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CUSTOM_256_EDITOR
  },
  {
    EL_GROUP_1,					GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_1_EDITOR
  },
  {
    EL_GROUP_2,					GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_2_EDITOR
  },
  {
    EL_GROUP_3,					GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_3_EDITOR
  },
  {
    EL_GROUP_4,					GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_4_EDITOR
  },
  {
    EL_GROUP_5,					GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_5_EDITOR
  },
  {
    EL_GROUP_6,					GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_6_EDITOR
  },
  {
    EL_GROUP_7,					GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_7_EDITOR
  },
  {
    EL_GROUP_8,					GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_8_EDITOR
  },
  {
    EL_GROUP_9,					GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_9_EDITOR
  },
  {
    EL_GROUP_10,				GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_10_EDITOR
  },
  {
    EL_GROUP_11,				GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_11_EDITOR
  },
  {
    EL_GROUP_12,				GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_12_EDITOR
  },
  {
    EL_GROUP_13,				GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_13_EDITOR
  },
  {
    EL_GROUP_14,				GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_14_EDITOR
  },
  {
    EL_GROUP_15,				GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_15_EDITOR
  },
  {
    EL_GROUP_16,				GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_16_EDITOR
  },
  {
    EL_GROUP_17,				GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_17_EDITOR
  },
  {
    EL_GROUP_18,				GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_18_EDITOR
  },
  {
    EL_GROUP_19,				GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_19_EDITOR
  },
  {
    EL_GROUP_20,				GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_20_EDITOR
  },
  {
    EL_GROUP_21,				GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_21_EDITOR
  },
  {
    EL_GROUP_22,				GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_22_EDITOR
  },
  {
    EL_GROUP_23,				GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_23_EDITOR
  },
  {
    EL_GROUP_24,				GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_24_EDITOR
  },
  {
    EL_GROUP_25,				GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_25_EDITOR
  },
  {
    EL_GROUP_26,				GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_26_EDITOR
  },
  {
    EL_GROUP_27,				GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_27_EDITOR
  },
  {
    EL_GROUP_28,				GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_28_EDITOR
  },
  {
    EL_GROUP_29,				GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_29_EDITOR
  },
  {
    EL_GROUP_30,				GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_30_EDITOR
  },
  {
    EL_GROUP_31,				GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_31_EDITOR
  },
  {
    EL_GROUP_32,				GFX_SPECIAL_ARG_EDITOR,
    IMG_GROUP_32_EDITOR
  },
  {
    EL_CHAR_SPACE,				GFX_SPECIAL_ARG_EDITOR,
    IMG_CHAR_SPACE_EDITOR
  },
  {
    EL_STEEL_CHAR_SPACE,			GFX_SPECIAL_ARG_EDITOR,
    IMG_STEEL_CHAR_SPACE_EDITOR
  },
  {
    -1,						-1,
    -1
  },
};

#endif	// CONF_ESG_C
