// ----------------------------------------------------------------------------
// Electrons.h
// ----------------------------------------------------------------------------

#ifndef ELECTRONS_H
#define ELECTRONS_H

#include "global.h"


void subAnimateElectrons(int);
void subDrawAnimatedElectrons(int);
void subDrawElectronFromAbove(int, int);
void subDrawElectronFromBelow(int, int);
void subDrawElectronFromLeft(int, int);
void subDrawElectronFromRight(int, int);
void subDrawElectronTurnLeft(int, int);
void subDrawElectronTurnRight(int, int);
void subElectronFromAbove(int, int);
void subElectronFromBelow(int, int);
void subElectronFromLeft(int, int);
void subElectronFromRight(int, int);
void subElectronTurnLeft(int, int);
void subElectronTurnRight(int, int);

#endif // ELECTRONS_H
