// ============================================================================
// Rocks'n'Diamonds - McDuffin Strikes Back!
// ----------------------------------------------------------------------------
// (c) 1995-2014 by Artsoft Entertainment
//                  Holger Schemel
//                  info@artsoft.org
//                  https://www.artsoft.org/
// ----------------------------------------------------------------------------
// conf_chr.c
// ============================================================================

// ------- this file was automatically generated -- do not edit by hand -------

#ifndef CONF_CHR_C
#define CONF_CHR_C

// values for graphics configuration (character elements)

  { "char_space",				"RocksFontEM.png"	},
  { "char_space.xpos",				"0"			},
  { "char_space.ypos",				"0"			},
  { "char_space.frames",			"1"			},
  { "char_space.EDITOR",			"RocksFontEM.png"	},
  { "char_space.EDITOR.xpos",			"7"			},
  { "char_space.EDITOR.ypos",			"4"			},
  { "char_space.EDITOR.frames",			"1"			},

  { "char_exclam",				"RocksFontEM.png"	},
  { "char_exclam.xpos",				"1"			},
  { "char_exclam.ypos",				"0"			},
  { "char_exclam.frames",			"1"			},

  { "char_quotedbl",				"RocksFontEM.png"	},
  { "char_quotedbl.xpos",			"2"			},
  { "char_quotedbl.ypos",			"0"			},
  { "char_quotedbl.frames",			"1"			},

  { "char_numbersign",				"RocksFontEM.png"	},
  { "char_numbersign.xpos",			"3"			},
  { "char_numbersign.ypos",			"0"			},
  { "char_numbersign.frames",			"1"			},

  { "char_dollar",				"RocksFontEM.png"	},
  { "char_dollar.xpos",				"4"			},
  { "char_dollar.ypos",				"0"			},
  { "char_dollar.frames",			"1"			},

  { "char_percent",				"RocksFontEM.png"	},
  { "char_percent.xpos",			"5"			},
  { "char_percent.ypos",			"0"			},
  { "char_percent.frames",			"1"			},

  { "char_ampersand",				"RocksFontEM.png"	},
  { "char_ampersand.xpos",			"6"			},
  { "char_ampersand.ypos",			"0"			},
  { "char_ampersand.frames",			"1"			},

  { "char_apostrophe",				"RocksFontEM.png"	},
  { "char_apostrophe.xpos",			"7"			},
  { "char_apostrophe.ypos",			"0"			},
  { "char_apostrophe.frames",			"1"			},

  { "char_parenleft",				"RocksFontEM.png"	},
  { "char_parenleft.xpos",			"8"			},
  { "char_parenleft.ypos",			"0"			},
  { "char_parenleft.frames",			"1"			},

  { "char_parenright",				"RocksFontEM.png"	},
  { "char_parenright.xpos",			"9"			},
  { "char_parenright.ypos",			"0"			},
  { "char_parenright.frames",			"1"			},

  { "char_asterisk",				"RocksFontEM.png"	},
  { "char_asterisk.xpos",			"10"			},
  { "char_asterisk.ypos",			"0"			},
  { "char_asterisk.frames",			"1"			},

  { "char_plus",				"RocksFontEM.png"	},
  { "char_plus.xpos",				"11"			},
  { "char_plus.ypos",				"0"			},
  { "char_plus.frames",				"1"			},

  { "char_comma",				"RocksFontEM.png"	},
  { "char_comma.xpos",				"12"			},
  { "char_comma.ypos",				"0"			},
  { "char_comma.frames",			"1"			},

  { "char_minus",				"RocksFontEM.png"	},
  { "char_minus.xpos",				"13"			},
  { "char_minus.ypos",				"0"			},
  { "char_minus.frames",			"1"			},

  { "char_period",				"RocksFontEM.png"	},
  { "char_period.xpos",				"14"			},
  { "char_period.ypos",				"0"			},
  { "char_period.frames",			"1"			},

  { "char_slash",				"RocksFontEM.png"	},
  { "char_slash.xpos",				"15"			},
  { "char_slash.ypos",				"0"			},
  { "char_slash.frames",			"1"			},

  { "char_0",					"RocksFontEM.png"	},
  { "char_0.xpos",				"0"			},
  { "char_0.ypos",				"1"			},
  { "char_0.frames",				"1"			},

  { "char_1",					"RocksFontEM.png"	},
  { "char_1.xpos",				"1"			},
  { "char_1.ypos",				"1"			},
  { "char_1.frames",				"1"			},

  { "char_2",					"RocksFontEM.png"	},
  { "char_2.xpos",				"2"			},
  { "char_2.ypos",				"1"			},
  { "char_2.frames",				"1"			},

  { "char_3",					"RocksFontEM.png"	},
  { "char_3.xpos",				"3"			},
  { "char_3.ypos",				"1"			},
  { "char_3.frames",				"1"			},

  { "char_4",					"RocksFontEM.png"	},
  { "char_4.xpos",				"4"			},
  { "char_4.ypos",				"1"			},
  { "char_4.frames",				"1"			},

  { "char_5",					"RocksFontEM.png"	},
  { "char_5.xpos",				"5"			},
  { "char_5.ypos",				"1"			},
  { "char_5.frames",				"1"			},

  { "char_6",					"RocksFontEM.png"	},
  { "char_6.xpos",				"6"			},
  { "char_6.ypos",				"1"			},
  { "char_6.frames",				"1"			},

  { "char_7",					"RocksFontEM.png"	},
  { "char_7.xpos",				"7"			},
  { "char_7.ypos",				"1"			},
  { "char_7.frames",				"1"			},

  { "char_8",					"RocksFontEM.png"	},
  { "char_8.xpos",				"8"			},
  { "char_8.ypos",				"1"			},
  { "char_8.frames",				"1"			},

  { "char_9",					"RocksFontEM.png"	},
  { "char_9.xpos",				"9"			},
  { "char_9.ypos",				"1"			},
  { "char_9.frames",				"1"			},

  { "char_colon",				"RocksFontEM.png"	},
  { "char_colon.xpos",				"10"			},
  { "char_colon.ypos",				"1"			},
  { "char_colon.frames",			"1"			},

  { "char_semicolon",				"RocksFontEM.png"	},
  { "char_semicolon.xpos",			"11"			},
  { "char_semicolon.ypos",			"1"			},
  { "char_semicolon.frames",			"1"			},

  { "char_less",				"RocksFontEM.png"	},
  { "char_less.xpos",				"12"			},
  { "char_less.ypos",				"1"			},
  { "char_less.frames",				"1"			},

  { "char_equal",				"RocksFontEM.png"	},
  { "char_equal.xpos",				"13"			},
  { "char_equal.ypos",				"1"			},
  { "char_equal.frames",			"1"			},

  { "char_greater",				"RocksFontEM.png"	},
  { "char_greater.xpos",			"14"			},
  { "char_greater.ypos",			"1"			},
  { "char_greater.frames",			"1"			},

  { "char_question",				"RocksFontEM.png"	},
  { "char_question.xpos",			"15"			},
  { "char_question.ypos",			"1"			},
  { "char_question.frames",			"1"			},

  { "char_at",					"RocksFontEM.png"	},
  { "char_at.xpos",				"0"			},
  { "char_at.ypos",				"2"			},
  { "char_at.frames",				"1"			},

  { "char_a",					"RocksFontEM.png"	},
  { "char_a.xpos",				"1"			},
  { "char_a.ypos",				"2"			},
  { "char_a.frames",				"1"			},

  { "char_b",					"RocksFontEM.png"	},
  { "char_b.xpos",				"2"			},
  { "char_b.ypos",				"2"			},
  { "char_b.frames",				"1"			},

  { "char_c",					"RocksFontEM.png"	},
  { "char_c.xpos",				"3"			},
  { "char_c.ypos",				"2"			},
  { "char_c.frames",				"1"			},

  { "char_d",					"RocksFontEM.png"	},
  { "char_d.xpos",				"4"			},
  { "char_d.ypos",				"2"			},
  { "char_d.frames",				"1"			},

  { "char_e",					"RocksFontEM.png"	},
  { "char_e.xpos",				"5"			},
  { "char_e.ypos",				"2"			},
  { "char_e.frames",				"1"			},

  { "char_f",					"RocksFontEM.png"	},
  { "char_f.xpos",				"6"			},
  { "char_f.ypos",				"2"			},
  { "char_f.frames",				"1"			},

  { "char_g",					"RocksFontEM.png"	},
  { "char_g.xpos",				"7"			},
  { "char_g.ypos",				"2"			},
  { "char_g.frames",				"1"			},

  { "char_h",					"RocksFontEM.png"	},
  { "char_h.xpos",				"8"			},
  { "char_h.ypos",				"2"			},
  { "char_h.frames",				"1"			},

  { "char_i",					"RocksFontEM.png"	},
  { "char_i.xpos",				"9"			},
  { "char_i.ypos",				"2"			},
  { "char_i.frames",				"1"			},

  { "char_j",					"RocksFontEM.png"	},
  { "char_j.xpos",				"10"			},
  { "char_j.ypos",				"2"			},
  { "char_j.frames",				"1"			},

  { "char_k",					"RocksFontEM.png"	},
  { "char_k.xpos",				"11"			},
  { "char_k.ypos",				"2"			},
  { "char_k.frames",				"1"			},

  { "char_l",					"RocksFontEM.png"	},
  { "char_l.xpos",				"12"			},
  { "char_l.ypos",				"2"			},
  { "char_l.frames",				"1"			},

  { "char_m",					"RocksFontEM.png"	},
  { "char_m.xpos",				"13"			},
  { "char_m.ypos",				"2"			},
  { "char_m.frames",				"1"			},

  { "char_n",					"RocksFontEM.png"	},
  { "char_n.xpos",				"14"			},
  { "char_n.ypos",				"2"			},
  { "char_n.frames",				"1"			},

  { "char_o",					"RocksFontEM.png"	},
  { "char_o.xpos",				"15"			},
  { "char_o.ypos",				"2"			},
  { "char_o.frames",				"1"			},

  { "char_p",					"RocksFontEM.png"	},
  { "char_p.xpos",				"0"			},
  { "char_p.ypos",				"3"			},
  { "char_p.frames",				"1"			},

  { "char_q",					"RocksFontEM.png"	},
  { "char_q.xpos",				"1"			},
  { "char_q.ypos",				"3"			},
  { "char_q.frames",				"1"			},

  { "char_r",					"RocksFontEM.png"	},
  { "char_r.xpos",				"2"			},
  { "char_r.ypos",				"3"			},
  { "char_r.frames",				"1"			},

  { "char_s",					"RocksFontEM.png"	},
  { "char_s.xpos",				"3"			},
  { "char_s.ypos",				"3"			},
  { "char_s.frames",				"1"			},

  { "char_t",					"RocksFontEM.png"	},
  { "char_t.xpos",				"4"			},
  { "char_t.ypos",				"3"			},
  { "char_t.frames",				"1"			},

  { "char_u",					"RocksFontEM.png"	},
  { "char_u.xpos",				"5"			},
  { "char_u.ypos",				"3"			},
  { "char_u.frames",				"1"			},

  { "char_v",					"RocksFontEM.png"	},
  { "char_v.xpos",				"6"			},
  { "char_v.ypos",				"3"			},
  { "char_v.frames",				"1"			},

  { "char_w",					"RocksFontEM.png"	},
  { "char_w.xpos",				"7"			},
  { "char_w.ypos",				"3"			},
  { "char_w.frames",				"1"			},

  { "char_x",					"RocksFontEM.png"	},
  { "char_x.xpos",				"8"			},
  { "char_x.ypos",				"3"			},
  { "char_x.frames",				"1"			},

  { "char_y",					"RocksFontEM.png"	},
  { "char_y.xpos",				"9"			},
  { "char_y.ypos",				"3"			},
  { "char_y.frames",				"1"			},

  { "char_z",					"RocksFontEM.png"	},
  { "char_z.xpos",				"10"			},
  { "char_z.ypos",				"3"			},
  { "char_z.frames",				"1"			},

  { "char_bracketleft",				"RocksFontEM.png"	},
  { "char_bracketleft.xpos",			"11"			},
  { "char_bracketleft.ypos",			"3"			},
  { "char_bracketleft.frames",			"1"			},

  { "char_backslash",				"RocksFontEM.png"	},
  { "char_backslash.xpos",			"12"			},
  { "char_backslash.ypos",			"3"			},
  { "char_backslash.frames",			"1"			},

  { "char_bracketright",			"RocksFontEM.png"	},
  { "char_bracketright.xpos",			"13"			},
  { "char_bracketright.ypos",			"3"			},
  { "char_bracketright.frames",			"1"			},

  { "char_asciicircum",				"RocksFontEM.png"	},
  { "char_asciicircum.xpos",			"14"			},
  { "char_asciicircum.ypos",			"3"			},
  { "char_asciicircum.frames",			"1"			},

  { "char_underscore",				"RocksFontEM.png"	},
  { "char_underscore.xpos",			"15"			},
  { "char_underscore.ypos",			"3"			},
  { "char_underscore.frames",			"1"			},

  { "char_copyright",				"RocksFontEM.png"	},
  { "char_copyright.xpos",			"0"			},
  { "char_copyright.ypos",			"4"			},
  { "char_copyright.frames",			"1"			},

  { "char_aumlaut",				"RocksFontEM.png"	},
  { "char_aumlaut.xpos",			"1"			},
  { "char_aumlaut.ypos",			"4"			},
  { "char_aumlaut.frames",			"1"			},

  { "char_oumlaut",				"RocksFontEM.png"	},
  { "char_oumlaut.xpos",			"2"			},
  { "char_oumlaut.ypos",			"4"			},
  { "char_oumlaut.frames",			"1"			},

  { "char_uumlaut",				"RocksFontEM.png"	},
  { "char_uumlaut.xpos",			"3"			},
  { "char_uumlaut.ypos",			"4"			},
  { "char_uumlaut.frames",			"1"			},

  { "char_degree",				"RocksFontEM.png"	},
  { "char_degree.xpos",				"4"			},
  { "char_degree.ypos",				"4"			},
  { "char_degree.frames",			"1"			},

  { "char_trademark",				"RocksFontEM.png"	},
  { "char_trademark.xpos",			"5"			},
  { "char_trademark.ypos",			"4"			},
  { "char_trademark.frames",			"1"			},

  { "char_cursor",				"RocksFontEM.png"	},
  { "char_cursor.xpos",				"6"			},
  { "char_cursor.ypos",				"4"			},
  { "char_cursor.frames",			"1"			},

  { "char_button",				"RocksFontEM.png"	},
  { "char_button.xpos",				"13"			},
  { "char_button.ypos",				"4"			},
  { "char_button.frames",			"1"			},

  { "char_up",					"RocksFontEM.png"	},
  { "char_up.xpos",				"14"			},
  { "char_up.ypos",				"4"			},
  { "char_up.frames",				"1"			},

  { "char_down",				"RocksFontEM.png"	},
  { "char_down.xpos",				"15"			},
  { "char_down.ypos",				"4"			},
  { "char_down.frames",				"1"			},

  { "steel_char_space",				"RocksFontDC.png"	},
  { "steel_char_space.xpos",			"0"			},
  { "steel_char_space.ypos",			"0"			},
  { "steel_char_space.frames",			"1"			},
  { "steel_char_space.EDITOR",			"RocksFontDC.png"	},
  { "steel_char_space.EDITOR.xpos",		"7"			},
  { "steel_char_space.EDITOR.ypos",		"4"			},
  { "steel_char_space.EDITOR.frames",		"1"			},

  { "steel_char_exclam",			"RocksFontDC.png"	},
  { "steel_char_exclam.xpos",			"1"			},
  { "steel_char_exclam.ypos",			"0"			},
  { "steel_char_exclam.frames",			"1"			},

  { "steel_char_quotedbl",			"RocksFontDC.png"	},
  { "steel_char_quotedbl.xpos",			"2"			},
  { "steel_char_quotedbl.ypos",			"0"			},
  { "steel_char_quotedbl.frames",		"1"			},

  { "steel_char_numbersign",			"RocksFontDC.png"	},
  { "steel_char_numbersign.xpos",		"3"			},
  { "steel_char_numbersign.ypos",		"0"			},
  { "steel_char_numbersign.frames",		"1"			},

  { "steel_char_dollar",			"RocksFontDC.png"	},
  { "steel_char_dollar.xpos",			"4"			},
  { "steel_char_dollar.ypos",			"0"			},
  { "steel_char_dollar.frames",			"1"			},

  { "steel_char_percent",			"RocksFontDC.png"	},
  { "steel_char_percent.xpos",			"5"			},
  { "steel_char_percent.ypos",			"0"			},
  { "steel_char_percent.frames",		"1"			},

  { "steel_char_ampersand",			"RocksFontDC.png"	},
  { "steel_char_ampersand.xpos",		"6"			},
  { "steel_char_ampersand.ypos",		"0"			},
  { "steel_char_ampersand.frames",		"1"			},

  { "steel_char_apostrophe",			"RocksFontDC.png"	},
  { "steel_char_apostrophe.xpos",		"7"			},
  { "steel_char_apostrophe.ypos",		"0"			},
  { "steel_char_apostrophe.frames",		"1"			},

  { "steel_char_parenleft",			"RocksFontDC.png"	},
  { "steel_char_parenleft.xpos",		"8"			},
  { "steel_char_parenleft.ypos",		"0"			},
  { "steel_char_parenleft.frames",		"1"			},

  { "steel_char_parenright",			"RocksFontDC.png"	},
  { "steel_char_parenright.xpos",		"9"			},
  { "steel_char_parenright.ypos",		"0"			},
  { "steel_char_parenright.frames",		"1"			},

  { "steel_char_asterisk",			"RocksFontDC.png"	},
  { "steel_char_asterisk.xpos",			"10"			},
  { "steel_char_asterisk.ypos",			"0"			},
  { "steel_char_asterisk.frames",		"1"			},

  { "steel_char_plus",				"RocksFontDC.png"	},
  { "steel_char_plus.xpos",			"11"			},
  { "steel_char_plus.ypos",			"0"			},
  { "steel_char_plus.frames",			"1"			},

  { "steel_char_comma",				"RocksFontDC.png"	},
  { "steel_char_comma.xpos",			"12"			},
  { "steel_char_comma.ypos",			"0"			},
  { "steel_char_comma.frames",			"1"			},

  { "steel_char_minus",				"RocksFontDC.png"	},
  { "steel_char_minus.xpos",			"13"			},
  { "steel_char_minus.ypos",			"0"			},
  { "steel_char_minus.frames",			"1"			},

  { "steel_char_period",			"RocksFontDC.png"	},
  { "steel_char_period.xpos",			"14"			},
  { "steel_char_period.ypos",			"0"			},
  { "steel_char_period.frames",			"1"			},

  { "steel_char_slash",				"RocksFontDC.png"	},
  { "steel_char_slash.xpos",			"15"			},
  { "steel_char_slash.ypos",			"0"			},
  { "steel_char_slash.frames",			"1"			},

  { "steel_char_0",				"RocksFontDC.png"	},
  { "steel_char_0.xpos",			"0"			},
  { "steel_char_0.ypos",			"1"			},
  { "steel_char_0.frames",			"1"			},

  { "steel_char_1",				"RocksFontDC.png"	},
  { "steel_char_1.xpos",			"1"			},
  { "steel_char_1.ypos",			"1"			},
  { "steel_char_1.frames",			"1"			},

  { "steel_char_2",				"RocksFontDC.png"	},
  { "steel_char_2.xpos",			"2"			},
  { "steel_char_2.ypos",			"1"			},
  { "steel_char_2.frames",			"1"			},

  { "steel_char_3",				"RocksFontDC.png"	},
  { "steel_char_3.xpos",			"3"			},
  { "steel_char_3.ypos",			"1"			},
  { "steel_char_3.frames",			"1"			},

  { "steel_char_4",				"RocksFontDC.png"	},
  { "steel_char_4.xpos",			"4"			},
  { "steel_char_4.ypos",			"1"			},
  { "steel_char_4.frames",			"1"			},

  { "steel_char_5",				"RocksFontDC.png"	},
  { "steel_char_5.xpos",			"5"			},
  { "steel_char_5.ypos",			"1"			},
  { "steel_char_5.frames",			"1"			},

  { "steel_char_6",				"RocksFontDC.png"	},
  { "steel_char_6.xpos",			"6"			},
  { "steel_char_6.ypos",			"1"			},
  { "steel_char_6.frames",			"1"			},

  { "steel_char_7",				"RocksFontDC.png"	},
  { "steel_char_7.xpos",			"7"			},
  { "steel_char_7.ypos",			"1"			},
  { "steel_char_7.frames",			"1"			},

  { "steel_char_8",				"RocksFontDC.png"	},
  { "steel_char_8.xpos",			"8"			},
  { "steel_char_8.ypos",			"1"			},
  { "steel_char_8.frames",			"1"			},

  { "steel_char_9",				"RocksFontDC.png"	},
  { "steel_char_9.xpos",			"9"			},
  { "steel_char_9.ypos",			"1"			},
  { "steel_char_9.frames",			"1"			},

  { "steel_char_colon",				"RocksFontDC.png"	},
  { "steel_char_colon.xpos",			"10"			},
  { "steel_char_colon.ypos",			"1"			},
  { "steel_char_colon.frames",			"1"			},

  { "steel_char_semicolon",			"RocksFontDC.png"	},
  { "steel_char_semicolon.xpos",		"11"			},
  { "steel_char_semicolon.ypos",		"1"			},
  { "steel_char_semicolon.frames",		"1"			},

  { "steel_char_less",				"RocksFontDC.png"	},
  { "steel_char_less.xpos",			"12"			},
  { "steel_char_less.ypos",			"1"			},
  { "steel_char_less.frames",			"1"			},

  { "steel_char_equal",				"RocksFontDC.png"	},
  { "steel_char_equal.xpos",			"13"			},
  { "steel_char_equal.ypos",			"1"			},
  { "steel_char_equal.frames",			"1"			},

  { "steel_char_greater",			"RocksFontDC.png"	},
  { "steel_char_greater.xpos",			"14"			},
  { "steel_char_greater.ypos",			"1"			},
  { "steel_char_greater.frames",		"1"			},

  { "steel_char_question",			"RocksFontDC.png"	},
  { "steel_char_question.xpos",			"15"			},
  { "steel_char_question.ypos",			"1"			},
  { "steel_char_question.frames",		"1"			},

  { "steel_char_at",				"RocksFontDC.png"	},
  { "steel_char_at.xpos",			"0"			},
  { "steel_char_at.ypos",			"2"			},
  { "steel_char_at.frames",			"1"			},

  { "steel_char_a",				"RocksFontDC.png"	},
  { "steel_char_a.xpos",			"1"			},
  { "steel_char_a.ypos",			"2"			},
  { "steel_char_a.frames",			"1"			},

  { "steel_char_b",				"RocksFontDC.png"	},
  { "steel_char_b.xpos",			"2"			},
  { "steel_char_b.ypos",			"2"			},
  { "steel_char_b.frames",			"1"			},

  { "steel_char_c",				"RocksFontDC.png"	},
  { "steel_char_c.xpos",			"3"			},
  { "steel_char_c.ypos",			"2"			},
  { "steel_char_c.frames",			"1"			},

  { "steel_char_d",				"RocksFontDC.png"	},
  { "steel_char_d.xpos",			"4"			},
  { "steel_char_d.ypos",			"2"			},
  { "steel_char_d.frames",			"1"			},

  { "steel_char_e",				"RocksFontDC.png"	},
  { "steel_char_e.xpos",			"5"			},
  { "steel_char_e.ypos",			"2"			},
  { "steel_char_e.frames",			"1"			},

  { "steel_char_f",				"RocksFontDC.png"	},
  { "steel_char_f.xpos",			"6"			},
  { "steel_char_f.ypos",			"2"			},
  { "steel_char_f.frames",			"1"			},

  { "steel_char_g",				"RocksFontDC.png"	},
  { "steel_char_g.xpos",			"7"			},
  { "steel_char_g.ypos",			"2"			},
  { "steel_char_g.frames",			"1"			},

  { "steel_char_h",				"RocksFontDC.png"	},
  { "steel_char_h.xpos",			"8"			},
  { "steel_char_h.ypos",			"2"			},
  { "steel_char_h.frames",			"1"			},

  { "steel_char_i",				"RocksFontDC.png"	},
  { "steel_char_i.xpos",			"9"			},
  { "steel_char_i.ypos",			"2"			},
  { "steel_char_i.frames",			"1"			},

  { "steel_char_j",				"RocksFontDC.png"	},
  { "steel_char_j.xpos",			"10"			},
  { "steel_char_j.ypos",			"2"			},
  { "steel_char_j.frames",			"1"			},

  { "steel_char_k",				"RocksFontDC.png"	},
  { "steel_char_k.xpos",			"11"			},
  { "steel_char_k.ypos",			"2"			},
  { "steel_char_k.frames",			"1"			},

  { "steel_char_l",				"RocksFontDC.png"	},
  { "steel_char_l.xpos",			"12"			},
  { "steel_char_l.ypos",			"2"			},
  { "steel_char_l.frames",			"1"			},

  { "steel_char_m",				"RocksFontDC.png"	},
  { "steel_char_m.xpos",			"13"			},
  { "steel_char_m.ypos",			"2"			},
  { "steel_char_m.frames",			"1"			},

  { "steel_char_n",				"RocksFontDC.png"	},
  { "steel_char_n.xpos",			"14"			},
  { "steel_char_n.ypos",			"2"			},
  { "steel_char_n.frames",			"1"			},

  { "steel_char_o",				"RocksFontDC.png"	},
  { "steel_char_o.xpos",			"15"			},
  { "steel_char_o.ypos",			"2"			},
  { "steel_char_o.frames",			"1"			},

  { "steel_char_p",				"RocksFontDC.png"	},
  { "steel_char_p.xpos",			"0"			},
  { "steel_char_p.ypos",			"3"			},
  { "steel_char_p.frames",			"1"			},

  { "steel_char_q",				"RocksFontDC.png"	},
  { "steel_char_q.xpos",			"1"			},
  { "steel_char_q.ypos",			"3"			},
  { "steel_char_q.frames",			"1"			},

  { "steel_char_r",				"RocksFontDC.png"	},
  { "steel_char_r.xpos",			"2"			},
  { "steel_char_r.ypos",			"3"			},
  { "steel_char_r.frames",			"1"			},

  { "steel_char_s",				"RocksFontDC.png"	},
  { "steel_char_s.xpos",			"3"			},
  { "steel_char_s.ypos",			"3"			},
  { "steel_char_s.frames",			"1"			},

  { "steel_char_t",				"RocksFontDC.png"	},
  { "steel_char_t.xpos",			"4"			},
  { "steel_char_t.ypos",			"3"			},
  { "steel_char_t.frames",			"1"			},

  { "steel_char_u",				"RocksFontDC.png"	},
  { "steel_char_u.xpos",			"5"			},
  { "steel_char_u.ypos",			"3"			},
  { "steel_char_u.frames",			"1"			},

  { "steel_char_v",				"RocksFontDC.png"	},
  { "steel_char_v.xpos",			"6"			},
  { "steel_char_v.ypos",			"3"			},
  { "steel_char_v.frames",			"1"			},

  { "steel_char_w",				"RocksFontDC.png"	},
  { "steel_char_w.xpos",			"7"			},
  { "steel_char_w.ypos",			"3"			},
  { "steel_char_w.frames",			"1"			},

  { "steel_char_x",				"RocksFontDC.png"	},
  { "steel_char_x.xpos",			"8"			},
  { "steel_char_x.ypos",			"3"			},
  { "steel_char_x.frames",			"1"			},

  { "steel_char_y",				"RocksFontDC.png"	},
  { "steel_char_y.xpos",			"9"			},
  { "steel_char_y.ypos",			"3"			},
  { "steel_char_y.frames",			"1"			},

  { "steel_char_z",				"RocksFontDC.png"	},
  { "steel_char_z.xpos",			"10"			},
  { "steel_char_z.ypos",			"3"			},
  { "steel_char_z.frames",			"1"			},

  { "steel_char_bracketleft",			"RocksFontDC.png"	},
  { "steel_char_bracketleft.xpos",		"11"			},
  { "steel_char_bracketleft.ypos",		"3"			},
  { "steel_char_bracketleft.frames",		"1"			},

  { "steel_char_backslash",			"RocksFontDC.png"	},
  { "steel_char_backslash.xpos",		"12"			},
  { "steel_char_backslash.ypos",		"3"			},
  { "steel_char_backslash.frames",		"1"			},

  { "steel_char_bracketright",			"RocksFontDC.png"	},
  { "steel_char_bracketright.xpos",		"13"			},
  { "steel_char_bracketright.ypos",		"3"			},
  { "steel_char_bracketright.frames",		"1"			},

  { "steel_char_asciicircum",			"RocksFontDC.png"	},
  { "steel_char_asciicircum.xpos",		"14"			},
  { "steel_char_asciicircum.ypos",		"3"			},
  { "steel_char_asciicircum.frames",		"1"			},

  { "steel_char_underscore",			"RocksFontDC.png"	},
  { "steel_char_underscore.xpos",		"15"			},
  { "steel_char_underscore.ypos",		"3"			},
  { "steel_char_underscore.frames",		"1"			},

  { "steel_char_copyright",			"RocksFontDC.png"	},
  { "steel_char_copyright.xpos",		"0"			},
  { "steel_char_copyright.ypos",		"4"			},
  { "steel_char_copyright.frames",		"1"			},

  { "steel_char_aumlaut",			"RocksFontDC.png"	},
  { "steel_char_aumlaut.xpos",			"1"			},
  { "steel_char_aumlaut.ypos",			"4"			},
  { "steel_char_aumlaut.frames",		"1"			},

  { "steel_char_oumlaut",			"RocksFontDC.png"	},
  { "steel_char_oumlaut.xpos",			"2"			},
  { "steel_char_oumlaut.ypos",			"4"			},
  { "steel_char_oumlaut.frames",		"1"			},

  { "steel_char_uumlaut",			"RocksFontDC.png"	},
  { "steel_char_uumlaut.xpos",			"3"			},
  { "steel_char_uumlaut.ypos",			"4"			},
  { "steel_char_uumlaut.frames",		"1"			},

  { "steel_char_degree",			"RocksFontDC.png"	},
  { "steel_char_degree.xpos",			"4"			},
  { "steel_char_degree.ypos",			"4"			},
  { "steel_char_degree.frames",			"1"			},

  { "steel_char_trademark",			"RocksFontDC.png"	},
  { "steel_char_trademark.xpos",		"5"			},
  { "steel_char_trademark.ypos",		"4"			},
  { "steel_char_trademark.frames",		"1"			},

  { "steel_char_cursor",			"RocksFontDC.png"	},
  { "steel_char_cursor.xpos",			"6"			},
  { "steel_char_cursor.ypos",			"4"			},
  { "steel_char_cursor.frames",			"1"			},

  { "steel_char_button",			"RocksFontDC.png"	},
  { "steel_char_button.xpos",			"13"			},
  { "steel_char_button.ypos",			"4"			},
  { "steel_char_button.frames",			"1"			},

  { "steel_char_up",				"RocksFontDC.png"	},
  { "steel_char_up.xpos",			"14"			},
  { "steel_char_up.ypos",			"4"			},
  { "steel_char_up.frames",			"1"			},

  { "steel_char_down",				"RocksFontDC.png"	},
  { "steel_char_down.xpos",			"15"			},
  { "steel_char_down.ypos",			"4"			},
  { "steel_char_down.frames",			"1"			},


#endif	// CONF_CHR_C
