// ============================================================================
// Rocks'n'Diamonds - McDuffin Strikes Back!
// ----------------------------------------------------------------------------
// (c) 1995-2014 by Artsoft Entertainment
//     		    Holger Schemel
//		    info@artsoft.org
//		    https://www.artsoft.org/
// ----------------------------------------------------------------------------
// config.h
// ============================================================================

#ifndef CONFIG_H
#define CONFIG_H

#include "main.h"

char *getSourceDateString(void);
char *getSourceHashString(void);
char *getProgramTitleString(void);
char *getProgramRealVersionString(void);
char *getProgramVersionString(void);
char *getProgramInitString(void);
char *getConfigProgramTitleString(void);
char *getConfigProgramCopyrightString(void);
char *getConfigProgramCompanyString(void);
char *getWindowTitleString(void);

#endif	// CONFIG_H
